<?php
/**
 * @package     CSVI
 * @subpackage  Form2Content
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace form2content\com_form2content\model\export;

defined('_JEXEC') or die;

/**
 * Export Form2Content content type fields.
 *
 * @package     CSVI
 * @subpackage  Form2Content
 * @since       7.0
 */
class ContentTypeFields extends \CsviModelExports
{
	/**
	 * Export the data.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			$userFields   = array();
			$exportFields = $this->fields->getFields();

			// Group by fields
			$groupFields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupBy       = array();

			if (isset($groupFields->name))
			{
				$groupByFields = array_flip($groupFields->name);
			}
			else
			{
				$groupByFields = array();
			}

			// Sort selected fields
			$sortFields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortBy     = array();

			if (isset($sortFields->name))
			{
				$sortByFields = array_flip($sortFields->name);
			}
			else
			{
				$sortByFields = array();
			}

			foreach ($exportFields as $field)
			{
				switch ($field->field_name)
				{
					case 'fieldname':
					case 'title':
					case 'description':
					case 'fieldtypeid':
					case 'settings':
					case 'id':
					case 'projectid':
					case 'ordering':
					case 'frontvisible':
						$userFields[] = $this->db->quoteName('pf.' . $field->field_name);

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('pf.' . $field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('pf.' . $field->field_name);
						}
						break;
					case 'fieldtypename':
						$userFields[] = $this->db->quoteName('pf.fieldtypeid');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('pf.fieldtypeid');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('pf.fieldtypeid');
						}
						break;
					case 'projecttitle':
						$userFields[] = $this->db->quoteName('p.title');
						$userFields[] = $this->db->quoteName('pf.projectid');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('p.title');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('p.title');
						}
						break;
					default:
						$userFields[] = $this->db->quoteName($field->field_name);

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName($field->field_name);
						}

						break;
				}
			}

			// Build the query
			$userFields = array_unique($userFields);
			$query      = $this->db->getQuery(true);
			$query->select(implode(",\n", $userFields));
			$query->from($this->db->quoteName('#__f2c_project', 'p'));
			$query->leftJoin(
					$this->db->quoteName('#__f2c_projectfields', 'pf') . ' ON ' .
					$this->db->quoteName('pf.projectid') . ' = ' . $this->db->quoteName('p.id')
				);
			$query->leftJoin(
				$this->db->quoteName('#__f2c_fieldtype', 'ft') . ' ON ' .
				$this->db->quoteName('ft.id') . ' = ' . $this->db->quoteName('pf.fieldtypeid')
			);

			// Group the fields
			$groupBy = array_unique($groupBy);

			if (0 !== count($groupBy))
			{
				$query->group($groupBy);
			}

			// Sort set fields
			$sortBy = array_unique($sortBy);

			if (0 !== count($sortBy))
			{
				$query->order($sortBy);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logCount = $this->db->getNumRows();

			if ($logCount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					foreach ($exportFields as $field)
					{
						$fieldName = $field->field_name;

						// Set the field value
						if (isset($record->$fieldName))
						{
							$fieldValue = $record->$fieldName;
						}
						else
						{
							$fieldValue = '';
						}

						// Process the field
						switch ($fieldName)
						{
							case 'projecttitle':
								$query->clear()
									->select($this->db->quoteName('title'))
									->from($this->db->quoteName('#__f2c_project'))
									->where($this->db->quoteName('id') . ' = ' . (int) $record->projectid);
								$this->db->setQuery($query);
								$fieldValue = $this->db->loadResult();
								break;
							case 'fieldtypename':
								$query->clear()
									->select($this->db->quoteName('name'))
									->from($this->db->quoteName('#__f2c_fieldtype'))
									->where($this->db->quoteName('id') . ' = ' . (int) $record->fieldtypeid);
								$this->db->setQuery($query);
								$fieldValue = $this->db->loadResult();
								break;
							default:
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldValue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}
}
