<?php
/**
 * @package     CSVI
 * @subpackage  Com_Form2Content
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace form2content\com_form2content\model\import;

defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;

/**
 * Project - content types import.
 *
 * @package     CSVI
 * @subpackage  Com_Form2Content
 * @since       7.0
 */
class ContentType extends \RantaiImportEngine
{
	/**
	 * Form2Content table.
	 *
	 * @var    \Form2ContentTableContentType
	 * @since  7.0
	 */
	private $contentTypeTable = null;

	/**
	 * The Form2Content helper
	 *
	 * @var    \Com_Form2ContentHelperCom_Form2Content
	 * @since  7.0
	 */
	protected $helper = null;

	/**
	 * Start the content types import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   7.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'title':
						if ($value)
						{
							$projectId = $this->helper->getProjectId($value, false);
							$this->setState('id', $projectId);
							$this->setState($name, $value);
						}
						break;
					case 'featured':
					case 'published':
						switch (strtolower($value))
						{
							case 'n':
							case 'no':
							case '0':
								$value = 0;
								break;
							default:
								$value = 1;
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		$this->loaded = true;

		// Check if there is all required fields for import
		$requiredFields = array('title');
		$checkMissingFields = $this->fields->checkRequiredFields($requiredFields);

		if ($checkMissingFields)
		{
			$this->log->addStats('skipped', $checkMissingFields);

			$this->loaded = false;
		}
		else
		{
			if ($this->contentTypeTable->load($this->getState('id', 0)))
			{
				if (!$this->template->get('overwrite_existing_data'))
				{
					$this->log->add(\JText::sprintf('COM_FORM2CONTENT_WARNING_OVERWRITING_SET_TO_NO', $this->getState('title')), false);
					$this->loaded = false;
				}
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   7.0
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			if (!$this->getState('id', false) && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new content types when user chooses to ignore new content types
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('title', '')));
			}
			else
			{
				// Check if we have a title
				if (empty($this->contentTypeTable->title))
				{
					$this->contentTypeTable->title = $this->getState('title');
				}

				// Set the attributes
				$this->setAttributes();

				// Set the images
				$this->setImages();

				// Set the urls
				$this->setUrls();

				// Set the settings
				$this->setSettings();

				// Check for meta data
				$this->setMetadata();

				// Data must be in an array
				$data = ArrayHelper::fromObject($this->state);

				// Bind the data
				if ($this->contentTypeTable->bind($data))
				{
					// Add a creating date if there is no id
					if (!$this->getState('id', false))
					{
						$this->contentTypeTable->created    = $this->date->toSql();
						$this->contentTypeTable->created_by = $this->userId;
					}

					if (!$this->getState('modified', false))
					{
						$this->contentTypeTable->modified = $this->date->toSql();
					}

					try
					{
						$this->contentTypeTable->check();
						$this->contentTypeTable->store();
						$this->log->add('Content types added');
						$this->log->addStats('Added', \JText::_('COM_CSVI_TABLE_FORM2CONTENTTABLECONTENTTYPES_ADDED'));
					}
					catch (\Exception $e)
					{
						$this->log->add('Cannot add content types. Error: ' . $e->getMessage(), false);
						$this->log->addStats('incorrect', \JText::_('COM_CSVI_TABLE_FORM2CONTENTTABLECONTENTTYPES_ERROR'));

						return false;
					}
				}
			}

			return true;
		}

		return false;
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function loadTables()
	{
		$this->contentTypeTable = $this->getTable('ContentType');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function clearTables()
	{
		$this->contentTypeTable->reset();
	}

	/**
	 * Set the attributes field.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	private function setAttributes()
	{
		// Check for attributes
		if (!$this->getState('attribs', false))
		{
			$attributeFields = array
			(
				'show_title',
				'link_titles',
				'show_intro',
				'show_category',
				'link_category',
				'show_parent_category',
				'link_parent_category',
				'show_author',
				'link_author',
				'show_create_date',
				'show_modify_date',
				'show_publish_date',
				'show_item_navigation',
				'show_icons',
				'show_print_icon',
				'show_email_icon',
				'show_vote',
				'show_hits',
				'show_noauth',
				'urls_position',
				'alternative_readmore',
				'article_layout',
				'show_publishing_options',
				'show_article_options',
				'show_urls_images_backend',
				'show_urls_images_frontend'
			);

			// Load the current attributes
			$attributes = json_decode($this->contentTypeTable->attribs);

			if (!is_object($attributes))
			{
				$attributes = new \stdClass;
			}

			foreach ($attributeFields as $field)
			{
				$attributes->$field = $this->getState($field, '');

				if (!$this->getState($field, false))
				{
					$attributes->$field = $this->getState($field, '');

					if ($this->$field == '*')
					{
						$attributes->$field = '';
					}
				}
			}

			// Store the new attributes
			$this->setState('attribs', json_encode($attributes));
		}
	}

	/**
	 * Set the images.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	private function setImages()
	{
		if (!$this->getState('images'))
		{
			$imageFields = array
			(
				'image_intro',
				'float_intro',
				'image_intro_alt',
				'image_intro_caption',
				'image_fulltext',
				'float_fulltext',
				'image_fulltext_alt',
				'image_fulltext_caption'
			);

			// Load the current images
			$images = json_decode($this->contentTypeTable->images);

			if (!is_object($images))
			{
				$images = new \stdClass;
			}

			foreach ($imageFields as $field)
			{
				$images->$field = $this->getState($field, '');
			}

			// Store the new attributes
			$this->setState('images', json_encode($images));
		}
	}

	/**
	 * Set the urls.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	private function setUrls()
	{
		if (!$this->getState('urls'))
		{
			$urlFields = array
			(
				'urla',
				'urlatext',
				'targeta',
				'urlb',
				'urlbtext',
				'targetb',
				'urlc',
				'urlctext',
				'targetc',
			);

			// Load the current images
			$urls = json_decode($this->contentTypeTable->urls);

			if (!is_object($urls))
			{
				$urls = new \stdClass;
			}

			foreach ($urlFields as $field)
			{
				$urls->$field = $this->getState($field, '');
			}

			// Store the new attributes
			$this->setState('urls', json_encode($urls));
		}
	}

	/**
	 * Set the meta data.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	private function setMetadata()
	{
		if (!$this->getState('metadata', false))
		{
			$metadataFields = array
			(
				'robots',
				'author',
				'rights',
				'xreference'
			);

			// Load the current images
			$metadata = json_decode($this->contentTypeTable->metadata);

			if (!is_object($metadata))
			{
				$metadata = new \stdClass;
			}

			foreach ($metadataFields as $field)
			{
				$metadata->$field = $this->getState($field, '');
			}

			// Store the new attributes
			$this->setState('metadata', json_encode($metadata));
		}
	}

	/**
	 * Save the settings.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	private function setSettings()
	{
		if (!$this->getState('settings'))
		{
			$settingFields = array
			(
				'article_caption',
				'title_front_end',
				'title_caption',
				'title_default',
				'title_alias_front_end',
				'title_alias_caption',
				'frontend_templsel',
				'intro_template',
				'main_template',
				'frontend_pubsel',
				'tags_front_end',
				'tags_caption',
				'tag_field_ajax_mode',
				'tags_allow_custom'
			);

			// Load the current settings
			$settings = json_decode($this->contentTypeTable->settings);

			if (!is_object($settings))
			{
				$settings = new \stdClass;
			}

			foreach ($settingFields as $field)
			{
				$settings->$field = $this->getState($field, '');
			}

			// Store the new attributes
			$this->setState('settings', json_encode($settings));
		}
	}
}

