<?php
/**
 * @package     CSVI
 * @subpackage  Com_Form2Content
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace form2content\com_form2content\model\import;

defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;

/**
 * Project - content types fields import.
 *
 * @package     CSVI
 * @subpackage  Com_Form2Content
 * @since       7.0
 */
class ContentTypeFields extends \RantaiImportEngine
{
	/**
	 * Form2Content Content type fields table.
	 *
	 * @var    \Form2ContentTableContentTypeFields
	 * @since  7.0
	 */
	private $contentTypeFieldsTable = null;

	/**
	 * The Form2Content helper
	 *
	 * @var    \Com_Form2ContentHelperCom_Form2Content
	 * @since  7.0
	 */
	protected $helper = null;

	/**
	 * Start the content types import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   7.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'fieldtypename':
						if ($value)
						{
							$fieldTypeId = $this->helper->getFieldTypeId($value);
							$this->setState('fieldtypeid', $fieldTypeId);
						}
						break;
					case 'projecttitle':
						if ($value)
						{
							$projectId = $this->helper->getProjectId($value, true);
							$this->setState('projectid', $projectId);
							$this->setState($name, $value);
						}
						break;
					case 'ssl_show_empty_choice_text':
						switch (strtolower($value))
						{
							case 'n':
							case 'no':
							case '0':
								$value = 0;
								break;
							default:
								$value = 1;
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		$this->loaded = true;

		// Check if there is all required fields for import
		$requiredFields = array('projectid~projecttitle', 'fieldname');
		$checkMissingFields = $this->fields->checkRequiredFields($requiredFields);

		// There must be a project id or project title and field name
		if (!$checkMissingFields)
		{
			if ($this->getState('projectid', false))
			{
				if ($this->contentTypeFieldsTable->load($this->getState('id', 0)))
				{
					if (!$this->template->get('overwrite_existing_data'))
					{
						$this->log->add(\JText::sprintf('COM_FORM2CONTENT_WARNING_OVERWRITING_SET_TO_NO', $this->getState('fieldname')), false);
						$this->loaded = false;
					}
				}

				if (!$this->getState('fieldtypeid', false))
				{
					$this->log->addStats('error', \JText::_('COM_CSVI_FIELDTYPEID_CANNOT_FOUND'));

					$this->loaded = false;
				}
			}
			else
			{
				$this->log->addStats('error', \JText::sprintf('COM_CSVI_CANNOT_FIND_PROJECT_TITLE', $this->getState('projecttitle')));

				$this->loaded = false;
			}
		}
		else
		{
			$this->log->addStats('skipped', $checkMissingFields);

			$this->loaded = false;
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   7.0
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			if (!$this->getState('id', false) && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new content types when user chooses to ignore new content types
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('title', '')));
			}
			else
			{
				// Check if the given fieldname and project id values already there
				if (!$this->getState('id', false))
				{
					$this->state->set('id', $this->helper->getFieldId(
						$this->getState('fieldname', false),
						$this->getState('projectid', false)
					)
					);
				}

				if ($this->getState('id', 0))
				{
					$this->contentTypeFieldsTable->load($this->getState('id', 0));
				}

				// Check if there are select field options set
				if (!$this->getState('settings', false))
				{
					$this->checkSettingOptionsSelect();
				}

				// Data must be in an array
				$data = ArrayHelper::fromObject($this->state);

				// Bind the data
				if ($this->contentTypeFieldsTable->bind($data))
				{
					try
					{
						$this->contentTypeFieldsTable->store();
						$this->log->add('Content type fields added');
						$this->log->addStats('Added', \JText::_('COM_CSVI_TABLE_FORM2CONTENT_FIELDS_ADDED'));
					}
					catch (\Exception $e)
					{
						$this->log->add('Cannot add content types fields. Error: ' . $e->getMessage(), false);
						$this->log->addStats('incorrect', \JText::_('COM_CSVI_TABLE_FORM2CONTENTTABLECONTENTTYPES_ERROR'));

						return false;
					}
				}
			}

			return true;
		}

		return false;
	}

	/**
	 * Check if there are any options for select list field
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function checkSettingOptionsSelect()
	{
		$optionsArray = array(
			'ssl_show_empty_choice_text',
			'ssl_empty_choice_text',
			'ssl_options',
			);

		$settings = json_decode($this->contentTypeFieldsTable->settings);

		if (!is_object($settings))
		{
			$settings = new \stdClass;
		}

		$displayText = array();
		$displayTextValue = array();
		$sslOptions = new \stdClass;

		if ($this->getState('ssl_options_display_text', false))
		{
			$displayText = explode('~', $this->getState('ssl_options_display_text', false));
		}

		if ($this->getState('ssl_options_display_text_value', false))
		{
			$displayTextValue = explode('~', $this->getState('ssl_options_display_text_value', false));
		}

		foreach ($displayText as $key => $value)
		{
			$sslOptions->$value = $displayTextValue[$key];
		}

		if (!empty($settings->ssl_options))
		{
			$this->setState('ssl_options', $settings->ssl_options);
		}

		if (!empty($displayText) || !empty($displayTextValue))
		{
			$this->setState('ssl_options', $sslOptions);
		}

		foreach ($optionsArray as $option)
		{
			$settings->$option = $this->getState($option, '');

			if (!$this->getState($option, ''))
			{
				$settings->$option = $this->getState($option, '');
			}
		}

		// Store the new settings
		$this->setState('settings', json_encode($settings));
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function loadTables()
	{
		$this->contentTypeFieldsTable = $this->getTable('ContentTypeFields');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function clearTables()
	{
		$this->contentTypeFieldsTable->reset();
	}
}
