<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

use Joomla\String\StringHelper;

defined('_JEXEC') or die;

/**
 * The HikaShop Config Class.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class Com_HikashopHelperCom_Hikashop
{
	/**
	 * Template helper
	 *
	 * @var    CsviHelperTemplate
	 * @since  6.0
	 */
	protected $template = null;

	/**
	 * Logger helper
	 *
	 * @var    CsviHelperLog
	 * @since  6.0
	 */
	protected $log = null;

	/**
	 * Fields helper
	 *
	 * @var    CsviHelperFields
	 * @since  6.0
	 */
	protected $fields = null;

	/**
	 * Database connector
	 *
	 * @var    JDatabaseDriver
	 * @since  6.0
	 */
	protected $db = null;

	/**
	 * The ICEcat helper
	 *
	 * @var    CsviHelperIcecat
	 * @since  6.0
	 */
	private $icecat = null;

	/**
	 * Transliterate class
	 *
	 * @var    \CsviHelperTranslit
	 *
	 * @since  7.7.0
	 */
	protected $translit;

	/**
	 * Constructor.
	 *
	 * @param   CsviHelperTemplate  $template  An instance of CsviHelperTemplate.
	 * @param   CsviHelperLog       $log       An instance of CsviHelperLog.
	 * @param   CsviHelperFields    $fields    An instance of CsviHelperFields.
	 * @param   JDatabaseDriver     $db        Database connector.
	 *
	 * @since   4.0
	 */
	public function __construct(CsviHelperTemplate $template, CsviHelperLog $log, CsviHelperFields $fields, JDatabaseDriver $db)
	{
		$this->template = $template;
		$this->log = $log;
		$this->fields = $fields;
		$this->db = $db;
		$this->icecatFields = array(
			'PROD_ID' => 'related_products',
			'PROD_NAME' => 'product_name',
			'PROD_HIGHPIC' => 'file_path',
			'PROD_RELEASEDATE' => 'product_sale_start',
			'PROD_CATEGORY_PATH' => 'category_path',
			'PROD_LONGDESC' => 'product_description',
			'PROD_SHORTDESC' => '',
			'PROD_MANUFACTURER_NAME' => 'brand',
			'PROD_FEATURES' => 'features'
		);

		// Load the transliterate class
		$this->translit = new \CsviHelperTranslit($this->template);
	}

	/**
	 * Get the brand ID.
	 *
	 * @param   string  $manufacturer  The name of the manufacturer to get the ID for.
	 *
	 * @return  int  The ID of the manufacturer.
	 *
	 * @since   6.0
	 */
	public function getManufacturerId($manufacturer)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('category_id'))
			->from($this->db->quoteName('#__hikashop_category'))
			->where($this->db->quoteName('category_type') . ' = ' . $this->db->quote('manufacturer'))
			->where($this->db->quoteName('category_name') . ' = ' . $this->db->quote($manufacturer));
		$this->db->setQuery($query);

		$this->log->add('Find manufacturer ID for ' . $manufacturer);

		return $this->db->loadResult();
	}

	/**
	 * Get the access level ID.
	 *
	 * @param   string  $name  The name of the access level to get the ID for.
	 *
	 * @return  int  The ID of the access level.
	 *
	 * @since   6.3.0
	 *
	 * @throws  RuntimeException
	 */
	public function getAccessLevelId($name)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('id'))
			->from($this->db->quoteName('#__usergroups'))
			->where($this->db->quoteName('title') . ' = ' . $this->db->quote($name));
		$this->db->setQuery($query);

		$this->log->add('Find access level ID for name ' . $name);

		return $this->db->loadResult();
	}

	/**
	 * Get the access level name.
	 *
	 * @param   int  $id  The ID of the access level to get the name for.
	 *
	 * @return  string  The name of the access level.
	 *
	 * @since   6.5.0
	 *
	 * @throws  RuntimeException
	 */
	public function getAccessLevelName($id)
	{
		$result = '';

		if ($id)
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('title'))
				->from($this->db->quoteName('#__usergroups'))
				->where($this->db->quoteName('id') . ' = ' . (int) $id);
			$this->db->setQuery($query);

			$this->log->add('Find access level name for ID ' . $id);

			return $this->db->loadResult();
		}

		return $result;
	}

	/**
	 * Get the currency ID.
	 *
	 * @param   string  $currency_code  The code of the currency to get the ID for.
	 *
	 * @return  int  The ID of the currency.
	 *
	 * @since   6.3.0
	 */
	public function getCurrencyId($currency_code)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('currency_id'))
			->from($this->db->quoteName('#__hikashop_currency'))
			->where($this->db->quoteName('currency_code') . ' = ' . $this->db->quote($currency_code));
		$this->db->setQuery($query);

		$this->log->add('Find currency ID for ' . $currency_code);

		return $this->db->loadResult();
	}

	/**
	 * Get the zone ID for a state.
	 *
	 * @param   string  $name  The name of the state to get the zone ID for.
	 * @param   string  $type  The type of zone.
	 *
	 * @return  int  The ID of the state zone | False if not both variables are set.
	 *
	 * @since   6.3.0
	 *
	 * @throws  RuntimeException
	 */
	public function getZoneNameKey($name, $type = '')
	{
		if ($name)
		{
			// Check for state ID
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('zone_namekey'))
				->from($this->db->quoteName('#__hikashop_zone'))
				->where($this->db->quoteName('zone_name_english') . ' = ' . $this->db->quote($name));

			if ('' !== $type)
			{
				$query->where($this->db->quoteName('zone_type') . ' = ' . $this->db->quote($type));
			}

			$this->db->setQuery($query);

			$this->log->add('Find zone ' . $name . ' for type ' . $type);

			return $this->db->loadResult();
		}

		return false;
	}

	/**
	 * Load the ICEcat data for a product.
	 *
	 * @return  mixed  True on success | false on failure.
	 *
	 * @since   3.0
	 */
	public function getIcecat()
	{
		if ($this->template->get('use_icecat'))
		{
			// Load the ICEcat helper
			if (is_null($this->icecat))
			{
				$this->icecat = new CsviHelperIcecat($this->template, $this->log, $this->db);
			}

			// Clean up any previous ICEcat data
			$this->fields->clearIcecatData();
			$fields = array();

			// Check conditions
			$mpn = $this->fields->get('product_code');

			if ($mpn)
			{
				$this->log->add('Found ICEcat mpn reference: ' . $mpn, false);

				// 2. Do we have a manufacturer name for HikaShop
				$mf_name = $this->fields->get('brand');
				$this->log->add('Found ICEcat manufacturer name: ' . $mf_name, false);

				if ($mf_name)
				{
					// Get the data from ICEcat
					$data = $this->icecat->getData($mpn, $mf_name);

					// Get the list of fields not to populate with ICEcat data
					$update_fields = $this->template->get('icecat_update_fields', array(), 'array');

					// Map the ICEcat data to HikaShop fields
					foreach ($data as $name => $value)
					{
						if (isset($this->icecatFields[$name]))
						{
							if (!in_array($this->icecatFields[$name], $update_fields))
							{
								$fields[$this->icecatFields[$name]] = $value;
							}
						}
					}

					// Add the ICEcat data to the fields
					$this->fields->setIcecatData($fields);

					return true;
				}
				else
				{
					$this->log->add('ICEcat manufacturer not found', false);

					return false;
				}
			}
			else
			{
				$this->log->add('ICEcat mpn reference not found', false);

				return false;
			}
		}

		return false;
	}

	/**
	 * Get the product id, this is necessary for updating existing products.
	 *
	 * @param   string  $product_code  The product code to find the ID for
	 *
	 * @return  int  The product ID is returned.
	 *
	 * @since   6.5.0
	 */
	public function getProductId($product_code = null)
	{
		if (is_null($product_code))
		{
			$product_code = $this->fields->get('product_code');
		}

		if ($product_code)
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('product_id'))
				->from($this->db->quoteName('#__hikashop_product'))
				->where($this->db->quoteName('product_code') . ' = ' . $this->db->quote($product_code));
			$this->db->setQuery($query);
			$this->log->add('Found product code');

			return $this->db->loadResult();
		}

		return false;
	}

	/**
	 * Process the shopper groups.
	 *
	 * @param   string  $field  The name of access field to check.
	 *
	 * @return  bool  Returns true on OK | False on failure.
	 *
	 * @since   4.5.2
	 *
	 * @throws  RuntimeException
	 */
	public function processAccessLevel($field)
	{
		$accessIds = '';
		$accessField = $this->fields->get($field, false);

		if ($accessField && $accessField != 'all')
		{
			// Get the shopper group names
			$names = explode('|', $accessField);
			$ids = array();

			foreach ($names as $name)
			{
				$id = $this->getAccessLevelId($name);

				if ($id)
				{
					$ids[] = $id;
				}
			}

			if ($ids)
			{
				// Add the empty entry at the start of the ids
				array_unshift($ids, '');

				// Add the empty entry at the end of the ids
				$ids[] = '';

				// Set the access level ID for other updates
				$accessIds = implode(',', $ids);
			}
		}

		return $accessIds;
	}

	/**
	 * Get the order id from order number
	 *
	 * @param   string  $orderNumber  The order number to find the ID for
	 *
	 * @return  int  The order ID is returned.
	 *
	 * @since   7.0
	 */
	public function getOrderId($orderNumber = null)
	{
		if ($orderNumber)
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('order_id'))
				->from($this->db->quoteName('#__hikashop_order'))
				->where($this->db->quoteName('order_number') . ' = ' . $this->db->quote($orderNumber));
			$this->db->setQuery($query);
			$this->log->add('Found order id');

			return $this->db->loadResult();
		}

		return false;
	}

	/**
	 * Unpublish products before import.
	 *
	 * @return  void.
	 *
	 * @since   7.1.0
	 *
	 * @throws  RuntimeException
	 */
	public function unpublishBeforeImport()
	{
		if ($this->template->get('unpublish_before_import', 0))
		{
			$query = $this->db->getQuery(true)
				->update($this->db->quoteName('#__hikashop_product'))
				->set($this->db->quoteName('product_published') . ' = 0');
			$this->db->setQuery($query);

			if ($this->db->execute())
			{
				$this->log->add('Unpublishing products before import');
			}
			else
			{
				$this->log->add('Cannot unpublish products before import');
			}
		}
	}

	/**
	 * Get a list of subcategory IDs for a given parent category ID.
	 *
	 * @param   int  $category_id  The category ID to get the subcategories for.
	 *
	 * @return  array  List of subcategory IDs.
	 *
	 * @since   7.1.0
	 */
	public function getSubCategoryIds($category_id)
	{
		$subcats = $this->getChildren($category_id);

		if (!empty($subcats))
		{
			foreach ($subcats as $subcat)
			{
				$newcats = $this->getSubCategoryIds($subcat);

				$subcats = array_merge($subcats, $newcats);
			}
		}

		return $subcats;

	}

	/**
	 * Get all children category ids for a given parent category ID.
	 *
	 * @param   int  $parentCategoryId  The parent category ID to get the subcategories for.
	 *
	 * @return  array  List of child category IDs.
	 *
	 * @since   7.1.0
	 */
	private function getChildren($parentCategoryId)
	{
		require_once JPATH_PLUGINS . '/csviaddon/hikashop/com_hikashop/helper/com_hikashop_category.php';
		$categoryHelper = new Com_HikashopHelperCom_Hikashop_Category($this->db);
		$query = $categoryHelper->getChildrenIds($parentCategoryId);
		$this->db->setQuery($query);

		return $this->db->loadColumn();
	}

	/**
	 * Create a product alias.
	 *
	 * @param   string  $name  The string to turn into a alias
	 *
	 * @return  string  The alias created.
	 *
	 * @since   7.7.0
	 */
	public function createProductAlias($name)
	{
		$slug = $this->translit->stringURLSafe($name);

		// Trim white spaces at beginning and end of alias and make lowercase
		$slug = trim(StringHelper::strtolower($slug));

		// Remove any duplicate whitespace, and ensure all characters are alphanumeric
		$slug = preg_replace('/(\s|[^A-Za-z0-9\-])+/', '-', $slug);

		// Trim dashes at beginning and end of alias
		$slug = trim($slug, '-');

		// If we are left with an empty string, make a date with random number
		if (trim(str_replace('-', '', $slug)) == '')
		{
			$jdate = JFactory::getDate();
			$slug   = $jdate->format("Y-m-d-h-i-s") . mt_rand();
		}

		$this->log->add('Create new slug from Product name', false);

		return $slug;
	}
}
