<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\export;

defined('_JEXEC') or die;

/**
 * Exports HikaShop categories.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class Category extends \CsviModelExports
{
	/**
	 * The domain name for URLs.
	 *
	 * @var    string
	 * @since  6.0
	 */
	private $domainname = null;

	/**
	 * Export the data.
	 *
	 * @return  bool  True if body is exported | False if body is not exported.
	 *
	 * @since   6.0
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Get some basic data
			$this->domainname = $this->settings->get('hostname');
			require_once JPATH_PLUGINS . '/csviaddon/hikashop/com_hikashop/helper/com_hikashop_category_product.php';
			$categoryHelper = new \Com_HikashopHelperCom_Hikashop_Category_Product($this->db, $this->template, $this->log);
			$exportfields = $this->fields->getFields();

			// Group by fields
			$groupbyfields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupby = array();

			if (isset($groupbyfields->name))
			{
				$groupbyfields = array_flip($groupbyfields->name);
			}
			else
			{
				$groupbyfields = array();
			}

			// Sort selected fields
			$sortfields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortby = array();

			if (isset($sortfields->name))
			{
				$sortbyfields = array_flip($sortfields->name);
			}
			else
			{
				$sortbyfields = array();
			}

			// Build something fancy to only get the fieldnames the user wants
			$userfields = array();

			foreach ($exportfields as $field)
			{
				switch ($field->field_name)
				{
					case 'category_path':
					case 'picture_url':
					case 'picture_url_thumb':
						$userfields[] = $this->db->quoteName('c.category_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('c.category_id');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('c.category_id');
						}
						break;
					case 'custom':
						break;
					default:
						$userfields[] = $this->db->quoteName('c.' . $field->field_name);

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('c.' . $field->field_name);
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('c.' . $field->field_name);
						}
						break;
				}
			}

			/**
			 * Export SQL Query
			 * Get all products - including items
			 * as well as products without a price
			 */
			$userfields = array_unique($userfields);
			$query = $this->db->getQuery(true)
				->select(implode(",\n", $userfields))
				->from($this->db->quoteName('#__hikashop_category', 'c'))
				->where($this->db->quoteName('c.category_id') . ' > 1');

			// Filter by category parent
			$root_categories = $this->template->get('root_categories', array());

			if ($root_categories)
			{
				$subquery = $categoryHelper->getChildrenIds($root_categories);

				// Get the leaf node IDs
				$query->where($this->db->quoteName('c.category_id') . ' IN (' . $subquery . ')');
			}

			// Group the fields
			$groupby = array_unique($groupby);

			if (!empty($groupby))
			{
				$query->group($groupby);
			}

			// Sort set fields
			$sortby = array_unique($sortby);

			if (!empty($sortby))
			{
				$query->order($sortby);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->db->getNumRows();

			if ($logcount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					// Process all the export fields
					foreach ($exportfields as $field)
					{
						// Get the field name
						$fieldname = $field->field_name;

						// Set the field value
						if (isset($record->$fieldname))
						{
							$fieldvalue = $record->$fieldname;
						}
						else
						{
							$fieldvalue = '';
						}

						// Process the field
						switch ($fieldname)
						{
							case 'category_path':
								$fieldvalue = $categoryHelper->createCategoryPathById($record->category_id);
								break;
							case 'picture_url':
							case 'picture_url_thumb':
								$path = $this->domainname . '/media/com_hikashop/upload/';

								if ($fieldname == 'picture_url_thumb')
								{
									$path .= 'thumbnail_' . $this->helperConfig->get('thumbnail_x') . 'x' . $this->helperConfig->get('thumbnail_y') . '/';
								}

								$query = $this->db->getQuery(true)
									->select('CONCAT(' . $this->db->quote($path) . ', ' . $this->db->quoteName('file_path') . ')')
									->from($this->db->quoteName('#__hikashop_file'))
									->where($this->db->quoteName('file_ref_id') . ' = ' . (int) $record->category_id)
									->where($this->db->quoteName('file_type') . ' = ' . $this->db->quote('category'))
									->order($this->db->quoteName('file_ordering'));
								$this->db->setQuery($query);
								$images = $this->db->loadColumn();

								$fieldvalue = implode(',', $images);
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldvalue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}
}
