<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\export;

defined('_JEXEC') or die;

/**
 * Exports HikaShop customers.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class Customer extends \CsviModelExports
{
	/**
	 * Export the data.
	 *
	 * @return  bool  True if body is exported | False if body is not exported.
	 *
	 * @since   6.0
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Get some basic data
			$exportfields = $this->fields->getFields();

			// Group by fields
			$groupbyfields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupby = array();

			if (isset($groupbyfields->name))
			{
				$groupbyfields = array_flip($groupbyfields->name);
			}
			else
			{
				$groupbyfields = array();
			}

			// Sort selected fields
			$sortfields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortby = array();

			if (isset($sortfields->name))
			{
				$sortbyfields = array_flip($sortfields->name);
			}
			else
			{
				$sortbyfields = array();
			}

			// Build something fancy to only get the fieldnames the user wants
			$userfields = array();

			foreach ($exportfields as $field)
			{
				switch ($field->field_name)
				{
					case 'country_2_code':
					case 'country_3_code':
					case 'country_name':
						$userfields[] = $this->db->quoteName('a.address_country');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('a.address_country');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('a.address_country');
						}
						break;
					case 'state_3_code':
					case 'state_name':
						$userfields[] = $this->db->quoteName('a.address_state');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('a.address_state');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('a.address_state');
						}
						break;
					case 'usergroup_name':
						$userfields[] = $this->db->quoteName('u.user_cms_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('u.user_cms_id');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('u.user_cms_id');
						}
						break;
					case 'custom':
						break;
					default:
						$userfields[] = $this->db->quoteName($field->field_name);

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName($field->field_name);
						}
						break;
				}
			}

			// Export SQL Query
			$userfields = array_unique($userfields);
			$query = $this->db->getQuery(true);
			$query->select(implode(",\n", $userfields));
			$query->from($this->db->quoteName('#__hikashop_user', 'u'));
			$query->leftJoin(
				$this->db->quoteName('#__hikashop_address', 'a')
				. ' ON ' . $this->db->quoteName('a.address_user_id') . ' = ' . $this->db->quoteName('u.user_id')
			);
			$query->leftJoin(
				$this->db->quoteName('#__users', 'j')
				. ' ON ' . $this->db->quoteName('j.id') . ' = ' . $this->db->quoteName('u.user_cms_id')
			);

			// Filter by block status
			$blocked = $this->template->get('blocked', false);

			if ($blocked != '')
			{
				$query->where($this->db->quoteName('j.block') . ' = ' . (int) (($blocked) ? 1 : 0));
			}

			// Filter by activated status
			$activated = $this->template->get('activated', false);

			if ($activated !== false)
			{
				if ($activated == '0')
				{
					$query->where($query->length($this->db->quoteName('j.activation')) . ' = 32');
				}
				elseif ($activated == '1')
				{
					$query->where($this->db->quoteName('j.activation') . ' = ' . $this->db->quote(''));
				}
			}

			// Filter by creation date
			$daterange = $this->template->get('user_created', '');

			if ($daterange != '')
			{
				$jdate = \JFactory::getDate();

				switch ($daterange)
				{
					case 'lastrun':
						if (substr($this->template->getLastrun(), 0, 4) != '0000')
						{
							$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) > ' . $this->db->quote($this->template->getLastrun()));
						}
						break;
					case 'yesterday':
						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) = DATE_SUB(CURDATE(), INTERVAL 1 DAY)');
						break;
					case 'thisweek':
						// Get the current day of the week
						$dayofweek = $jdate->__get('dayofweek');
						$offset = $dayofweek - 1;
						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= DATE_SUB(CURDATE(), INTERVAL ' . $offset . ' DAY)');
						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) <= CURDATE()');
						break;
					case 'lastweek':
						// Get the current day of the week
						$dayofweek = $jdate->__get('dayofweek');
						$offset = $dayofweek + 6;
						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= DATE_SUB(CURDATE(), INTERVAL ' . $offset . ' DAY)');
						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) <= DATE_SUB(CURDATE(), INTERVAL ' . $dayofweek . ' DAY)');
						break;
					case 'thismonth':
						// Get the current day of the week
						$dayofmonth = $jdate->__get('day');
						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= DATE_SUB(CURDATE(), INTERVAL ' . $dayofmonth . ' DAY)');
						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) <= CURDATE()');
						break;
					case 'lastmonth':
						// Get the current day of the week
						$dayofmonth = $jdate->__get('day');
						$month = date('n');
						$year = date('y');

						if ($month > 1)
						{
							$month--;
						}
						else
						{
							$month = 12;
							$year--;
						}

						$daysinmonth = date('t', mktime(0, 0, 0, $month, 25, $year));
						$offset = ($daysinmonth + $dayofmonth) - 1;

						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= DATE_SUB(CURDATE(), INTERVAL ' . $offset . ' DAY)');
						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) <= DATE_SUB(CURDATE(), INTERVAL ' . $dayofmonth . ' DAY)');
						break;
					case 'thisquarter':
						// Find out which quarter we are in
						$month = $jdate->__get('month');
						$year = date('Y');
						$quarter = ceil($month / 3);

						switch ($quarter)
						{
							case '1':
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= ' . $this->db->quote($year . '-01-01'));
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) < ' . $this->db->quote($year . '-04-01'));
								break;
							case '2':
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= ' . $this->db->quote($year . '-04-01'));
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) < ' . $this->db->quote($year . '-07-01'));
								break;
							case '3':
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= ' . $this->db->quote($year . '-07-01'));
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) < ' . $this->db->quote($year . '-10-01'));
								break;
							case '4':
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= ' . $this->db->quote($year . '-10-01'));
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) < ' . $this->db->quote($year++ . '-01-01'));
								break;
						}
						break;
					case 'lastquarter':
						// Find out which quarter we are in
						$month = $jdate->__get('month');
						$year = date('Y');
						$quarter = ceil($month / 3);

						if ($quarter == 1)
						{
							$quarter = 4;
							$year--;
						}
						else
						{
							$quarter--;
						}

						switch ($quarter)
						{
							case '1':
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= ' . $this->db->quote($year . '-01-01'));
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) < ' . $this->db->quote($year . '-04-01'));
								break;
							case '2':
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= ' . $this->db->quote($year . '-04-01'));
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) < ' . $this->db->quote($year . '-07-01'));
								break;
							case '3':
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= ' . $this->db->quote($year . '-07-01'));
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) < ' . $this->db->quote($year . '-10-01'));
								break;
							case '4':
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= ' . $this->db->quote($year . '-10-01'));
								$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) < ' . $this->db->quote($year++ . '-01-01'));
								break;
						}
						break;
					case 'thisyear':
						$year = date('Y');
						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= ' . $this->db->quote($year . '-01-01'));
						$year++;
						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) < ' . $this->db->quote($year . '-01-01'));
						break;
					case 'lastyear':
						$year = date('Y');
						$year--;
						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) >= ' . $this->db->quote($year . '-01-01'));
						$year++;
						$query->where('DATE(FROM_UNIXTIME(' . $this->db->quoteName('u.user_created') . ')) < ' . $this->db->quote($year . '-01-01'));
						break;
				}
			}

			// Group the fields
			$groupby = array_unique($groupby);

			if (!empty($groupby))
			{
				$query->group($groupby);
			}

			// Sort set fields
			$sortby = array_unique($sortby);

			if (!empty($sortby))
			{
				$query->order($sortby);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->db->getNumRows();

			if ($logcount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					// Process all the export fields
					foreach ($exportfields as $field)
					{
						// Get the field name
						$fieldname = $field->field_name;

						// Set the field value
						if (isset($record->$fieldname))
						{
							$fieldvalue = $record->$fieldname;
						}
						else
						{
							$fieldvalue = '';
						}

						// Process the field
						switch ($fieldname)
						{
							case 'country_name':
								$query = $this->db->getQuery(true)
									->select($this->db->quoteName('zone_name_english'))
									->from($this->db->quoteName('#__hikashop_zone'))
									->where($this->db->quoteName('zone_namekey') . ' = ' . $this->db->quote($record->address_country))
									->where($this->db->quoteName('zone_type') . ' = ' . $this->db->quote('country'));
								$this->db->setQuery($query);

								$fieldvalue = $this->db->loadResult();
								break;
							case 'country_2_code':
								if ($record->address_country)
								{
									$query = $this->db->getQuery(true)
										->select($this->db->quoteName('zone_code_2'))
										->from($this->db->quoteName('#__hikashop_zone'))
										->where($this->db->quoteName('zone_namekey') . ' = ' . $this->db->quote($record->address_country))
										->where($this->db->quoteName('zone_type') . ' = ' . $this->db->quote('country'));
									$this->db->setQuery($query);

									$fieldvalue = $this->db->loadResult();
								}
								break;
							case 'country_3_code':
								if ($record->address_country)
								{
									$query = $this->db->getQuery(true)
										->select($this->db->quoteName('zone_code_3'))
										->from($this->db->quoteName('#__hikashop_zone'))
										->where($this->db->quoteName('zone_namekey') . ' = ' . $this->db->quote($record->address_country))
										->where($this->db->quoteName('zone_type') . ' = ' . $this->db->quote('country'));
									$this->db->setQuery($query);

									$fieldvalue = $this->db->loadResult();
								}
								break;
							case 'state_name':
								if ($record->address_state)
								{
									$query = $this->db->getQuery(true)
										->select($this->db->quoteName('zone_name_english'))
										->from($this->db->quoteName('#__hikashop_zone'))
										->where($this->db->quoteName('zone_namekey') . ' = ' . $this->db->quote($record->address_state))
										->where($this->db->quoteName('zone_type') . ' = ' . $this->db->quote('state'));
									$this->db->setQuery($query);

									$fieldvalue = $this->db->loadResult();
								}
								break;
							case 'state_3_code':
								if ($record->address_state)
								{
									$query = $this->db->getQuery(true)
										->select($this->db->quoteName('zone_code_3'))
										->from($this->db->quoteName('#__hikashop_zone'))
										->where($this->db->quoteName('zone_namekey') . ' = ' . $this->db->quote($record->address_state))
										->where($this->db->quoteName('zone_type') . ' = ' . $this->db->quote('state'));
									$this->db->setQuery($query);

									$fieldvalue = $this->db->loadResult();
								}
								break;
							case 'usergroup_name':
								$query = $this->db->getQuery(true)
									->select($this->db->quoteName('title'))
									->from($this->db->quoteName('#__usergroups'))
									->leftJoin(
										$this->db->quoteName('#__user_usergroup_map')
										. ' ON ' . $this->db->quoteName('#__user_usergroup_map.group_id') . ' = ' . $this->db->quoteName('#__usergroups.id')
									)
									->where($this->db->quoteName('user_id') . ' = ' . (int) $record->user_cms_id);
								$this->db->setQuery($query);
								$groups = $this->db->loadColumn();
								$this->log->add('Usergroup name');

								if (is_array($groups))
								{
									$fieldvalue = implode('|', $groups);
								}
								else
								{
									$fieldvalue = '';
								}
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldvalue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}
}
