<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\export;

defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;


/**
 * Exports HikaShop price.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class Price extends \CsviModelExports
{
	/**
	 * Export the data.
	 *
	 * @return  bool  True if body is exported | False if body is not exported.
	 *
	 * @since   6.0
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Get some basic data
			$exportfields = $this->fields->getFields();

			// Group by fields
			$groupbyfields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupby = array();

			if (isset($groupbyfields->name))
			{
				$groupbyfields = array_flip($groupbyfields->name);
			}
			else
			{
				$groupbyfields = array();
			}

			// Sort selected fields
			$sortfields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortby = array();

			if (isset($sortfields->name))
			{
				$sortbyfields = array_flip($sortfields->name);
			}
			else
			{
				$sortbyfields = array();
			}

			// Build something fancy to only get the fieldnames the user wants
			$userfields = array();

			foreach ($exportfields as $field)
			{
				switch ($field->field_name)
				{
					case 'currency_code':
					case 'currency_name':
						$userfields[] = $this->db->quoteName('price_currency_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('price_currency_id');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('price_currency_id');
						}
						break;
					case 'product_code':
					case 'product_name':
						$userfields[] = $this->db->quoteName('price_product_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('price_product_id');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('price_product_id');
						}
						break;
					case 'custom':
						break;
					default:
						$userfields[] = $this->db->quoteName($field->field_name);

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName($field->field_name);
						}
						break;
				}
			}

			/**
			 * Export SQL Query
			 * Get all products - including items
			 * as well as products without a price
			 */
			$userfields = array_unique($userfields);
			$query = $this->db->getQuery(true);
			$query->select(implode(",\n", $userfields));
			$query->from($this->db->quoteName('#__hikashop_price'));

			// Currencies selector
			$currencies = $this->template->get('currency', array());

			if ($currencies && $currencies[0] != 'all')
			{
				$query->where($this->db->quoteName('#__hikashop_price.price_currency_id') . " IN ('" . implode("','", $currencies) . "')");
			}

			// Shopper group selector
			$shopper_group = $this->template->get('shopper_groups', array());

			if ($shopper_group)
			{
				$where = array();

				foreach ($shopper_group as $group)
				{
					$where[] = $this->db->quoteName('#__hikashop_price.price_access') . " LIKE (" . $this->db->quote('%,' . $group . ',%') . ")";
				}

				if ($where)
				{
					$query->where('(' . implode(' OR ', $where) . ')');
				}
			}

			// Filter by category
			$productCategories = $this->template->get('product_categories', false);

			// Sanity check
			ArrayHelper::toInteger($productCategories);

			if ($productCategories && $productCategories[0] !== '')
			{
				if ($this->template->get('incl_subcategory', false))
				{
					foreach ($productCategories as $productcategory)
					{
						$subids = $this->helper->getSubCategoryIds($productcategory);

						if ($subids)
						{
							$productCategories = array_merge($productCategories, $subids);
						}
					}
				}

				$query->leftJoin(
					$this->db->quoteName('#__hikashop_product_category')
					. ' ON ' . $this->db->quoteName('#__hikashop_product_category.product_id') . ' = ' . $this->db->quoteName('#__hikashop_price.price_product_id')
				);

				$query->where($this->db->quoteName('#__hikashop_product_category.category_id') . ' IN (' . implode(',', $productCategories) . ')');
			}

			// Group the fields
			$groupby = array_unique($groupby);

			if (!empty($groupby))
			{
				$query->group($groupby);
			}

			// Sort set fields
			$sortby = array_unique($sortby);

			if (!empty($sortby))
			{
				$query->order($sortby);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->db->getNumRows();

			if ($logcount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					// Process all the export fields
					foreach ($exportfields as $field)
					{
						// Get the field name
						$fieldname = $field->field_name;

						// Set the field value
						if (isset($record->$fieldname))
						{
							$fieldvalue = $record->$fieldname;
						}
						else
						{
							$fieldvalue = '';
						}

						// Process the field
						switch ($fieldname)
						{
							case 'product_code':
							case 'product_name':
								$query = $this->db->getQuery(true)
									->select($this->db->quoteName($fieldname))
									->from($this->db->quoteName('#__hikashop_product'))
									->where($this->db->quoteName('product_id') . ' = ' . (int) $record->price_product_id);
								$this->db->setQuery($query);
								$fieldvalue = $this->db->loadResult();
								break;
							case 'currency_code':
							case 'currency_name':
								$query = $this->db->getQuery(true)
									->select($this->db->quoteName($fieldname))
									->from($this->db->quoteName('#__hikashop_currency'))
									->where($this->db->quoteName('currency_id') . ' = ' . (int) $record->price_currency_id);
								$this->db->setQuery($query);
								$fieldvalue = $this->db->loadResult();
								break;
							case 'price_access':
								$access = explode(',', $record->price_access);
								$access = ArrayHelper::toInteger($access);

								$query = $this->db->getQuery(true)
									->select($this->db->quoteName('title'))
									->from($this->db->quoteName('#__usergroups'))
									->where($this->db->quoteName('id') . ' IN (' . implode(',', $access) . ')');
								$this->db->setQuery($query);
								$titles = $this->db->loadColumn();
								$fieldvalue = implode('|', $titles);
								break;
							case 'price_min_quantity':
								$fieldvalue = $record->price_min_quantity;

								// If price_min_quantity is 0 show as 1
								if ($fieldvalue == 0)
								{
									$fieldvalue = 1;
								}

								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldvalue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}
}
