<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\import;

defined('_JEXEC') or die;

/**
 * Main processor for handling category details.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class Category extends \RantaiImportEngine
{
	/**
	 * The product table
	 *
	 * @var    \HikashopTableCategory
	 * @since  6.0
	 */
	private $categoryTable = null;

	/**
	 * The media table
	 *
	 * @var    \HikashopTableFile
	 * @since  6.0
	 */
	private $fileTable = null;

	/**
	 * Category model helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop_Category
	 * @since  6.0
	 */
	private $categoryModel = null;

	/**
	 * Category separator
	 *
	 * @var    string
	 * @since  6.0
	 */
	private $categorySeparator = null;

	/**
	 * Here starts the processing.
	 *
	 * @return  bool  Returns true on success | False on failure.
	 *
	 * @since   3.0
	 */
	public function getStart()
	{
		require_once JPATH_PLUGINS . '/csviaddon/hikashop/com_hikashop/helper/com_hikashop_category_category.php';
		$this->categoryModel = new \Com_HikashopHelperCom_Hikashop_Category_Category($this->db, $this->template, $this->log);

		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				// Check if the field needs extra treatment
				switch ($name)
				{
					case 'category_published':
						switch ($value)
						{
							case 'n':
							case 'N':
							case '0':
								$value = 0;
								break;
							default:
								$value = 1;
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		// If we have no category path we cannot continue
		if (!$this->getState('category_path', false))
		{
			$this->log->addStats('incorrect', 'COM_CSVI_NO_CATEGORY_PATH_SET');

			return false;
		}

		// If we have no category type we cannot continue
		// @todo Warning that it is missing
		if (!$this->getState('category_type', false))
		{
			$this->log->addStats('incorrect', 'COM_CSVI_NO_CATEGORY_TYPE_SET');

			return false;
		}

		return true;
	}

	/**
	 * Process each record and store it in the database.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   3.0
	 */
	public function getProcessRecord()
	{
		// Load the category separator
		if (is_null($this->categorySeparator))
		{
			$this->categorySeparator = $this->template->get('category_separator', '/');
		}

		$paths = explode($this->categorySeparator, $this->getState('category_path'));
		$category_type = $this->getState('category_type');

		// Get the last category
		$last = end($paths);
		reset($paths);

		foreach ($paths as $key => $path)
		{
			// Construct the full path
			$fullpath = array();

			for ($i = 0; $i <= $key; $i++)
			{
				$fullpath[] = $paths[$i];
			}

			$workPath = implode($this->categorySeparator, $fullpath);

			// First get the category ID
			try
			{
				$categoryid = $this->categoryModel->getCategoryIdByPath($workPath, $category_type);

				$this->setState('category_id', $categoryid);
			}
			catch (\Exception $e)
			{
				$this->log->addStats('incorrect', $e->getMessage());

				return false;
			}

			if ($this->getState('category_id', false) && !$this->template->get('overwrite_existing_data'))
			{
				$this->log->add(\JText::sprintf('COM_CSVI_DATA_EXISTS_PRODUCT_SKU', $categoryid));
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_PRODUCT_SKU', $categoryid));
			}
			elseif ($last == $path)
			{
				// We have the category ID, lets see if it should be deleted
				if ($this->getState('category_delete', 'N') == 'Y')
				{
					$this->deleteCategory();
				}
				else
				{
					// Set some basic values
					if (!$this->getState('modified_on', false))
					{
						$this->setState('category_modified', time());
					}

					$this->setState('category_name', $this->getState('category_name', $path));

					// Save the category
					$categoryid = $this->categoryModel->save($this->state, $this->getState('category_type'));

					// All fields have been processed, bind the data
					if (!$categoryid)
					{
						$this->log->addStats('incorrect', \JText::_('COM_CSVI_CATEGORY_DETAILS_NOT_ADDED'));

						return false;
					}

					$this->setState('category_id', $categoryid);

					// Handle the images
					$this->processMedia();
				}
			}
			else
			{
				$this->setState('category_parent_id', $categoryid);
			}
		}

		return true;
	}

	/**
	 * Load the product related tables.
	 *
	 * @return  void.
	 *
	 * @since   3.0
	 */
	public function loadTables()
	{
		// Load the main tables
		$this->categoryTable = $this->getTable('Category');
		$this->fileTable = $this->getTable('File');
	}

	/**
	 * Cleaning the product related tables.
	 *
	 * @return  void.
	 *
	 * @since   3.0
	 */
	public function clearTables()
	{
		// Clean the main tables
		$this->categoryTable->reset();
		$this->fileTable->reset();
	}

	/**
	 * Delete a category and its references.
	 *
	 * @return  void.
	 *
	 * @since   4.0
	 */
	private function deleteCategory()
	{
		// Delete the product
		if ($this->categoryTable->delete($this->getState('category_id')))
		{
			$this->log->addstats('deleted', 'com_csvi_category_deleted');

			// Delete media
			$query = $this->db->getquery(true)
				->delete($this->db->quotename('#__hikashop_file'))
				->where($this->db->quotename('file_type') . ' = ' . $this->db->quote('category'))
				->where($this->db->quotename('file_ref_id') . ' = ' . (int) $this->getState('category_id'));
			$this->db->setquery($query);
			$this->log->add('Delete the category media');
			$this->db->execute();

			// Reset the products that link to this category
			$query = $this->db->getquery(true)
				->delete($this->db->quotename('#__hikashop_product_category'))
				->where($this->db->quotename('category_id') . ' = ' . (int) $this->getState('category_id'));
			$this->db->setquery($query);
			$this->log->add('Delete the product category cross reference');
			$this->db->execute();
		}
		else
		{
			$this->log->addStats('incorrect', 'COM_CSVI_CATEGORY_NOT_DELETED');
		}
	}

	/**
	 * Process media files.
	 *
	 * @return  bool Returns true on OK | False on failure.
	 *
	 * @since   4.0
	 */
	private function processMedia()
	{
		$generate_image = $this->template->get('auto_generate_image_name', false);

		// Check if any image handling needs to be done
		if ($this->getState('file_path', false) || $generate_image)
		{
			// Check if we have any images
			if (!$this->getState('file_path', false) && $generate_image)
			{
				$this->createImageName();
			}

			// Create an array of images to process
			$images = explode('|', $this->getState('file_path'));
			$titles = explode('|', $this->getState('file_name'));
			$descriptions = explode('|', $this->getState('file_description'));
			$ordering = 1;
			$max_width = $this->template->get('resize_max_width', 1024);
			$max_height = $this->template->get('resize_max_height', 768);

			// Image handling
			$imagehelper = new \CsviHelperImage($this->template, $this->log, $this->csvihelper);

			// Delete existing image links
			if ($this->template->get('delete_product_images', false))
			{
				$query = $this->db->getQuery(true)
					->delete($this->db->quoteName('#__hikashop_file'))
					->where($this->db->quoteName('file_type') . '=' . $this->db->quote('category'))
					->where($this->db->quoteName('file_ref_id') . '=' . (int) $this->getState('category_id'));
				$this->db->setQuery($query)->execute();
				$this->log->add('Delete images');
			}

			foreach ($images as $key => $image)
			{
				$image = trim($image);

				// Create image name if needed
				if (count($images) == 1)
				{
					$img_counter = 0;
				}
				else
				{
					$img_counter = $key + 1;
				}

				if ($generate_image)
				{
					$name = null;
					$name = $this->createImageName($img_counter);

					if (!empty($name))
					{
						$image = $name;
					}
				}

				if (!empty($image))
				{
					// Get the image path from the HikaShop config
					$imgpath = $this->helperconfig->get('uploadfolder');

					// Verify the original image
					if ($imagehelper->isRemote($image))
					{
						$original = $image;
						$full_path = $imgpath;
					}
					else
					{
						// Check if the image contains the image path
						$dirname = dirname($image);

						if (strpos($imgpath, $dirname) !== false)
						{
							$image = basename($image);
						}

						$original = $imgpath . $image;

						// Get subfolders
						$path_parts = pathinfo($original);
						$full_path = $path_parts['dirname'] . '/';
					}

					// Generate image names
					if ($this->template->get('process_image', false))
					{
						if ($generate_image)
						{
							$file_details = $imagehelper->ProcessImage($original, $full_path, $this->getState('product_full_image_output'));
						}
						else
						{
							$file_details = $imagehelper->ProcessImage($original, $full_path);
						}
					}
					else
					{
						$file_details['exists'] = true;
						$file_details['isimage'] = $imagehelper->isImage(JPATH_SITE . '/' . $image);
						$file_details['name'] = $image;
						$file_details['output_name'] = basename($image);
						$file_details['output_path'] = $full_path;
					}

					// Process the file details
					if ($file_details['exists'])
					{
						// Check if the image is an external image
						if (substr($file_details['name'], 0, 4) == 'http')
						{
							$this->log->addStats('incorrect', 'COM_CSVI_HIKASHOP_NOSUPPORT_URL');
						}
						else
						{
							$media = array();

							if ($this->template->get('autofill'))
							{
								$media['file_name'] = $file_details['output_name'];
								$media['file_description'] = $file_details['output_name'];
							}
							else
							{
								$media['file_name'] = (!empty($titles[$key])) ? $titles[$key] : $file_details['output_name'];
								$media['file_description'] = (isset($descriptions[$key])) ? $descriptions[$key] : '';
							}

							$media['file_path'] = $file_details['output_name'];
							$media['file_type'] = 'category';
							$media['file_ref_id'] = $this->getState('category_id');
							$media['file_free_download'] = 0;
							$media['file_ordering'] = $ordering[$key];
							$media['file_limit'] = 0;

							// Create the thumbnail
							if ($file_details['isimage'] && $this->template->get('thumb_create'))
							{
								$thumb_sizes = getimagesize(JPATH_SITE . '/' . $file_details['output_path'] . $media['file_name']);

								if ($thumb_sizes[0] < $max_width || $thumb_sizes[1] < $max_height)
								{
									$destination = $imgpath . 'thumbnail_' . $this->template->get('thumb_height') . 'x' . $this->template->get('thumb_width') . '/';
									$imagehelper->createThumbnail($file_details['output_path'] . $media['file_name'], $destination, $file_details['output_name']);
								}
							}

							// Bind the media data
							$this->fileTable->bind($media);

							// Check if the media image already exists
							$this->fileTable->check();

							// Store the media data
							if ($this->fileTable->store())
							{
								// Watermark the image
								if ($this->template->get('full_watermark', 'image') && $file_details['isimage'])
								{
									$imagehelper->addWatermark(JPATH_SITE . '/' . $media['file_url']);
								}
							}
							else
							{
								$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_MEDIA_NOT_ADDED', $this->fileTable->getError()));

								return false;
							}

							// Reset the product media table
							$this->fileTable->reset();
						}
					}
				}
			}
		}

		return true;
	}

	/**
	 * Create image name.
	 *
	 * Check if the user wants to have CSVI VirtueMart create the image names if so
	 * create the image names without path.
	 *
	 * @param   int  $ordering  The number to apply to a generated image name.
	 *
	 * @return  string  The name of the image.
	 *
	 * @since   3.0
	 */
	private function createImageName($ordering = 0)
	{
		$this->log->add('Create image name');

		// Create extension
		$ext = $this->template->get('autogenerateext');

		// Check if the user wants to convert the images to a different type
		switch ($this->template->get('type_generate_image_name'))
		{
			case 'product_sku':
				$this->log->add('Create image based on product SKU');

				if (!is_null($this->product_sku))
				{
					$name = $this->product_sku;
				}
				else
				{
					$this->log->addStats('error', 'COM_CSVI_CANNOT_FIND_PRODUCT_SKU');

					return false;
				}
				break;
			case 'product_name':
				$this->log->add('Create image based on product name');

				if (!is_null($this->productLangTable->product_name))
				{
					$name = str_replace(' ', '_', $this->productLangTable->product_name);
				}
				else
				{
					$this->log->addStats('error', 'COM_CSVI_CANNOT_FIND_PRODUCT_NAME');

					return false;
				}
				break;
			case 'product_id':
				$this->log->add('Create image based on product ID');

				if (!is_null($this->virtuemart_product_id))
				{
					$name = $this->virtuemart_product_id;
				}
				else
				{
					$this->log->addStats('error', 'COM_CSVI_CANNOT_FIND_PRODUCT_ID');

					return false;
				}
				break;
			case 'random':
				$this->log->add('Create image based on a random value');
				$name = mt_rand();
				break;
			default:
				$this->log->addStats('error', 'COM_CSVI_CANNOT_FIND_PRODUCT_SKU');

				return false;
				break;
		}

		// Build the new name
		if ($ordering > 0)
		{
			$image_name = $name . '_' . $ordering . '.' . $ext;
		}
		else
		{
			$image_name = $name . '.' . $ext;
		}

		$this->log->add('Created image name ' . $image_name);
		$this->product_full_image_output = $image_name;

		// Check if the user is supplying image data
		if (is_null($this->file_url))
		{
			$this->file_url = $this->product_full_image_output;
		}

		return $this->file_url;
	}
}
