<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\import;

defined('_JEXEC') or die;

/**
 * Main processor for handling Characteristic details.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.6.0
 */
class Characteristic extends \RantaiImportEngine
{
	/**
	 * The characteristic table
	 *
	 * @var    \HikashopTableCharacteristic
	 * @since  6.6.0
	 */
	private $characteristicTable;

	/**
	 * The addon helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop
	 * @since  6.6.0
	 */
	protected $helper;

	/**
	 * CSVI fields
	 *
	 * @var    \CsviHelperImportfields
	 * @since  6.0
	 */
	protected $fields;

	/**
	 * Here starts the processing.
	 *
	 * @return  bool  Returns true on success | False on failure.
	 *
	 * @since   6.6.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				// Check if the field needs extra treatment
				switch ($name)
				{
					case 'characteristic_parent_alias':
						$characteristicParentId = 0;

						if ($value)
						{
							$characteristicParentId = $this->getCharacteristicId($value);
						}

						$this->setState('characteristic_parent_id', $characteristicParentId);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		$this->loaded = true;

		if (!$this->getState('characteristic_alias', false))
		{
			$this->loaded = false;
			$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_MISSING_REQUIRED_FIELDS', 'characteristic_alias'));
		}

		return true;
	}


	/**
	 * Process each record and store it in the database.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \InvalidArgumentException
	 * @throws  \RuntimeException
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			// Get the needed data
			$characteristicId = $this->getCharacteristicId($this->getState('characteristic_alias', false));
			$this->setState('characteristic_id', $characteristicId);
			$characteristic_delete = $this->getState('characteristic_delete', 'N');

			// User wants to delete the product
			if ($characteristicId && $characteristic_delete === 'Y')
			{
				$this->deleteCharacteristic();
			}
			elseif (!$characteristicId && $characteristic_delete === 'Y')
			{
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_NO_CHARACTERISTIC_FOUND', $this->getState('characteristic_alias')));
			}
			elseif ($this->getState('characteristic_alias', false) && !$this->template->get('overwrite_existing_data'))
			{
				$this->log->add(\JText::sprintf('COM_CSVI_DATA_EXISTS_CONTENT', $this->getState('characteristic_alias')));
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_CONTENT', $this->getState('characteristic_alias')));
			}
			elseif ($this->getState('characteristic_parent_id') === null)
			{
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_NO_PARENT_PRODUCT_FOUND', $this->getState('characteristic_parent_alias')));
			}
			else
			{
				// Bind the data
				$this->characteristicTable->bind($this->state);

				// Store the price
				if ($this->characteristicTable->store())
				{
					$this->log->add('Characteristic added successfully');
				}
			}

			return true;
		}
		else
		{
			return false;
		}
	}


	/**
	 * Load the characteristic related tables.
	 *
	 * @return  void.
	 *
	 * @since   6.6.0
	 */
	public function loadTables()
	{
		// Load the main tables
		$this->characteristicTable = $this->getTable('Characteristic');
	}

	/**
	 * Cleaning the product related tables.
	 *
	 * @return  void.
	 *
	 * @since   6.6.0
	 */
	public function clearTables()
	{
		// Clean the main tables
		$this->characteristicTable->reset();
	}

	/**
	 * Get the characteristic id of the given characteristic name
	 *
	 * @param   string  $characteristicName  The name of characteristic.
	 *
	 * @return  int  Returns characteristic id corresponding to corresponding field.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \RuntimeException
	 */
	private function getCharacteristicId($characteristicName)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('characteristic_id'))
			->from($this->db->quoteName('#__hikashop_characteristic'))
			->where($this->db->quoteName('characteristic_alias') . ' = ' . $this->db->quote($characteristicName));
		$this->db->setQuery($query);

		return $this->db->loadResult();
	}

	/**
	 * Delete a characteristic.
	 *
	 * @return  bool  Always returns true.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \RuntimeException
	 */
	private function deleteCharacteristic()
	{
		// Deletion based on the real primary key
		$this->characteristicTable->setKeyName('characteristic_id');

		// Get the characteristic ID
		$characteristic_id = $this->getState('characteristic_id');

		// Delete the characteristic
		if ($this->characteristicTable->delete($characteristic_id))
		{
			$this->log->add('Delete the characteristic');

			// Delete all related characteristics
			$query = $this->db->getQuery(true)
				->delete($this->db->quotename('#__hikashop_characteristic'))
				->where($this->db->quotename('characteristic_parent_id') . ' = ' . $this->db->quote($characteristic_id));
			$this->db->setQuery($query)->execute();
			$this->log->add('Delete all the child characteristics');
		}
		else
		{
			$this->log->addStats('incorrect', 'COM_CSVI_CHARCTERISTIC_NOT_DELETED');
		}

		return true;
	}
}
