<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\import;

defined('_JEXEC') or die;

/**
 * Customer import.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class Customer extends \RantaiImportEngine
{
	/**
	 * HikaShop User table.
	 *
	 * @var    \HikaShopTableHsuser
	 * @since  6.0
	 */
	private $hsuserTable = null;

	/**
	 * Joomla User table.
	 *
	 * @var    \HikaShopTableUser
	 * @since  6.0
	 */
	private $userTable = null;

	/**
	 * Address table.
	 *
	 * @var    \HikaShopTableAddress
	 * @since  6.0
	 */
	private $addressTable = null;

	/**
	 * Start the product import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   6.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'user_email':
						$this->setState($name, $value);
						$this->setState('email', $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		// Reset loaded state
		$this->loaded = true;

		// Required fields are
		if ($this->getState('user_cms_id', false) || $this->getState('user_email', false))
		{
			// See if we have a user_id or user_email
			if (!$this->getState('user_cms_id', false) && $this->getState('user_email'))
			{
				// We have an e-mail address, find the user_id
				$query = $this->db->getQuery(true)
					->select($this->db->quoteName('user_cms_id'))
					->from($this->db->quoteName('#__hikashop_user'))
					->leftJoin(
						$this->db->quoteName('#__users')
						. ' ON ' . $this->db->quoteName('#__users.id') . ' = ' . $this->db->quoteName('#__hikashop_user.user_cms_id')
					)
					->where($this->db->quoteName('#__hikashop_user.user_email') . ' = ' . $this->db->quote($this->getState('user_email')));

				$this->db->setQuery($query);
				$this->log->add('Find the HikaShop user table id');
				$this->setState('user_cms_id', $this->db->loadResult());
				$this->log->add('Found the HikaShop user table id ' . $this->getState('user_cms_id'), false);

				$query->clear()
					->select($this->db->quoteName('id'))
					->from($this->db->quoteName('#__users'))
					->where($this->db->quoteName('email') . ' = ' . $this->db->quote($this->getState('user_email')));
				$this->db->setQuery($query);
				$this->log->add('Find the user ID from Joomla');
				$this->setState('id', $this->db->loadResult());
				$this->log->add('Found the Joomla user id ' . $this->getState('id'), false);
			}

			// Bind the values
			$this->hsuserTable->bind($this->state);

			if ($this->hsuserTable->check())
			{
				$this->setState('user_id', $this->hsuserTable->get('user_id'));

				// Check if we have an existing item
				if ($this->getState('user_id', 0) > 0 && !$this->template->get('overwrite_existing_data', true))
				{
					$this->log->add('The item ' . $this->getState('user_email') . ' already exists and the option Overwrite data is set to No', false);
					$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_CONTENT', $this->getState('user_email')));
					$this->loaded = false;
				}
				else
				{
					// Load the current content data
					$this->hsuserTable->load();
					$this->loaded = true;
				}
			}
		}
		else
		{
			$this->loaded = false;

			$user_cms_id = $this->getState('user_cms_id', false);
			$user_email = $this->getState('user_email', false);
			$missing = '';

			if (!$user_cms_id)
			{
				$missing = 'user_cms_id';
			}

			if (!$user_email)
			{
				$missing = 'user_email';
			}

			$this->log->addStats(
				'skipped',
				\JText::sprintf('COM_CSVI_MISSING_REQUIRED_FIELDS_USER', $missing)
			);
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   6.0
	 */
	public function getProcessRecord()
	{
		// Bind the values
		$this->userTable->bind($this->state);

		if ($this->loaded)
		{
			if (!$this->getState('user_id', false) && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new customers when user chooses to ignore new customers
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('user_email')));

				return false;
			}
			elseif (!$this->userTable->checkUsername())
			{
				// Check if the username already exist in joomla user table
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_USERNAME', $this->getState('username')));

				return false;
			}
			else
			{
				$this->userTable->load($this->getState('id', false));

				// Store the Joomla user
				$this->storeJoomlaUser();

				// Store the HikaShop user
				$this->storeHikashopuser();

				// Store the customer address
				$this->storeCustomerAddress();

				return true;
			}
		}
		else
		{
			return false;
		}
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function loadTables()
	{
		$this->hsuserTable = $this->getTable('Hsuser');
		$this->userTable = $this->getTable('User');
		$this->addressTable = $this->getTable('Address');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function clearTables()
	{
		$this->hsuserTable->reset();
		$this->userTable->reset();
		$this->addressTable->reset();
	}

	/**
	 * Store the Joomla user.
	 *
	 * @return  void.
	 *
	 * @since   1.0
	 *
	 * @throws  \InvalidArgumentException
	 */
	private function storeJoomlaUser()
	{
		$userdata = array();
		jimport('joomla.user.helper');

		// If it is a new Joomla user but no username is set, we must set one
		if ((!$this->getState('id', false) || !$this->getState('id')) && !$this->getState('username'))
		{
			$userdata['username'] = $this->getState('user_email');
		}
		// Set the username
		elseif ($this->getState('username', false))
		{
			$userdata['username'] = $this->getState('username');
		}

		// Check if we have an encrypted password
		if ($this->getState('password_crypt', false))
		{
			$userdata['password'] = $this->getState('password_crypt');
			$this->setState('password', true);
		}
		elseif ($this->getState('password', false))
		{
			// Check if we have an encrypted password
			$userdata['password'] = \JUserHelper::hashPassword($this->getState('password'));
		}

		// No user id, need to create a user if possible
		if (!$this->getState('id', false)
			&& $this->getState('user_email', false)
			&& $this->getState('password', false))
		{
			// Set the creation date
			$userdata['registerDate'] = $this->date->toSql();

			// Set the basic parameters
			$this->setState('params', '{"admin_style":"","admin_language":"","language":"","editor":"","helpsite":"","timezone":""}');
		}
		elseif (!$this->getState('id', false)
			&& (!$this->getState('user_email', false)
				|| !$this->getState('password', false)))
		{
			$this->log->addStats('incorrect', 'COM_CSVI_NO_NEW_USER_PASSWORD_EMAIL');

			throw new \InvalidArgumentException(\JText::_('COM_CSVI_NO_NEW_USER_PASSWORD_EMAIL'));
		}
		else
		{
			// Set the id
			$userdata['id'] = $this->getState('id');
		}

		// Only store the Joomla user if there is an e-mail address supplied
		if ($this->getState('user_email', false))
		{
			// Set the name
			if ($this->getState('name', false))
			{
				$userdata['name'] = $this->getState('name');
			}
			else
			{
				$fullname = false;

				if ($this->getState('first_name', false))
				{
					$fullname .= $this->getState('first_name') . ' ';
				}

				if ($this->getState('last_name', false))
				{
					$fullname .= $this->getState('last_name');
				}

				if (!$fullname)
				{
					$fullname = $this->getState('user_email');
				}

				$userdata['name'] = trim($fullname);
			}

			// Set the email
			$userdata['email'] = $this->getState('user_email');

			// Set if the user is blocked
			if ($this->getState('block', false))
			{
				$userdata['block'] = $this->getState('block');
			}

			// Set the sendEmail
			if ($this->getState('sendemail', false))
			{
				$userdata['sendEmail'] = $this->getState('sendemail');
			}

			// Set the registerDate
			if ($this->getState('registerdate', false))
			{
				$userdata['registerDate'] = $this->getState('registerdate');
			}

			// Set the lastvisitDate
			if ($this->getState('lastvisitdate', false))
			{
				$userdata['lastvisitDate'] = $this->getState('lastvisitdate');
			}

			// Set the activation
			if ($this->getState('activation', false))
			{
				$userdata['activation'] = $this->getState('activation');
			}

			// Set the params
			if ($this->getState('params', false))
			{
				$userdata['params'] = $this->getState('params');
			}

			// Set the reset count
			if ($this->getState('lastResetTime', false))
			{
				$userdata['lastResetTime'] = $this->getState('lastResetTime');
			}

			// Set the reset count
			if ($this->getState('resetCount', false))
			{
				$userdata['resetCount'] = $this->getState('resetCount');
			}

			// Set the one time key
			if ($this->getState('otpKey', false))
			{
				$userdata['otpKey'] = $this->getState('otpKey');
			}

			// Set the one time emergency key
			if ($this->getState('otep', false))
			{
				$userdata['otep'] = $this->getState('otep');
			}

			// Set the require reset
			if ($this->getState('requireReset', false))
			{
				$userdata['requireReset'] = $this->getState('requireReset');
			}

			// Check if we have a group ID
			if (!$this->getState('group_id', false) && strlen($this->getState('usergroup_name', '')) == 0)
			{
				$this->log->addStats('incorrect', 'COM_CSVI_NO_USERGROUP_NAME_FOUND');

				throw new \InvalidArgumentException(\JText::_('COM_CSVI_NO_USERGROUP_NAME_FOUND'));
			}
			elseif (!$this->getState('group_id', false))
			{
				$groups = explode('|', $this->getState('usergroup_name'));
				$usergroups = array();

				foreach ($groups as $group)
				{
					$query = $this->db->getQuery(true)
						->select($this->db->quoteName('id'))
						->from($this->db->quoteName('#__usergroups'))
						->where($this->db->quoteName('title') . ' = ' . $this->db->quote($group));
					$this->db->setQuery($query);
					$usergroups[] = $this->db->loadResult();
				}

				if (empty($usergroups))
				{
					$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_NO_USERGROUP_FOUND', $this->getState('usergroup_name')));

					throw new \InvalidArgumentException(\JText::_(\JText::sprintf('COM_CSVI_NO_USERGROUP_FOUND', $this->getState('usergroup_name'))));
				}
			}
			else
			{
				$usergroups = array($this->getState('group_id'));
			}

			// Bind the data
			$this->userTable->bind($userdata);

			// Store/update the user
			if ($this->userTable->store())
			{
				$this->log->add('Joomla user stored', false);

				// Get the new user ID
				$this->setState('user_cms_id', $this->userTable->get('id'));

				// Empty the usergroup map table
				$query = $this->db->getQuery(true)
					->delete($this->db->quoteName('#__user_usergroup_map'))
					->where($this->db->quoteName('user_id') . ' = ' . (int) $this->getState('user_cms_id'));
				$this->db->setQuery($query)->execute();
				$this->log->add('Delete old user groups');

				// Store the user in the usergroup map table
				$query = $this->db->getQuery(true);
				$query->insert($this->db->quoteName('#__user_usergroup_map'));

				if (!empty($usergroups))
				{
					foreach ($usergroups as $group)
					{
						$query->values($this->userTable->id . ', ' . $group);
					}
				}
				else
				{
					$query->values($this->userTable->id . ', ' . $this->getState('group_id'));
				}

				$this->db->setQuery($query);

				// Store the map
				if ($this->db->execute())
				{
					$this->log->add('Joomla user map stored');
				}
				else
				{
					$this->log->add('Joomla user map not stored');
				}
			}
			else
			{
				$this->log->add('Joomla user not found');
			}
		}
		else
		{
			$this->log->add('Joomla user skipped because no email was found');
		}
	}

	/**
	 * Store the HikaShop user.
	 *
	 * @return  void.
	 *
	 * @since   1.0
	 */
	private function storeHikashopUser()
	{
		$this->hsuserTable->load($this->getState('user_id'));

		// Set a create date if needed
		if (!$this->getState('user_id', false))
		{
			$this->setState('user_created', time());
			$this->setState('user_created_ip', $_SERVER['REMOTE_ADDR']);
		}

		$this->hsuserTable->bind($this->state);
		$this->hsuserTable->check();

		try
		{
			$this->hsuserTable->save($this->state);
		}
		catch (\Exception $exception)
		{
			$this->log->add('Cannot add HikaShop user. Error: ' . $exception->getMessage(), false);
			$this->log->addStats('incorrect', $exception->getMessage());
		}

		$this->setState('address_user_id', $this->hsuserTable->get('user_id'));
	}

	/**
	 * Store the HikaShop customer address.
	 *
	 * @return  void.
	 *
	 * @since   1.0
	 */
	private function storeCustomerAddress()
	{
		$this->addressTable->load($this->getState('address_user_id'));

		// Set the default address value
		$this->setState('address_default', $this->getState('address_default', 1));

		// Check for country ID
		if ($this->getState('country_name', false))
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('zone_namekey'))
				->from($this->db->quoteName('#__hikashop_zone'))
				->where($this->db->quoteName('zone_name_english') . ' = ' . $this->db->quote($this->getState('country_name')))
				->where($this->db->quoteName('zone_type') . ' = ' . $this->db->quote('country'));
			$this->db->setQuery($query);
			$this->setState('address_country', $this->db->loadResult());
		}

		// Check for state ID
		if ($this->getState('state_name', false))
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('zone_namekey'))
				->from($this->db->quoteName('#__hikashop_zone'))
				->where($this->db->quoteName('zone_name_english') . ' = ' . $this->db->quote($this->getState('state_name')))
				->where($this->db->quoteName('zone_type') . ' = ' . $this->db->quote('state'));
			$this->db->setQuery($query);
			$this->setState('address_state', $this->db->loadResult());
		}

		$this->addressTable->bind($this->state);
		$this->addressTable->check();

		try
		{
			// Bind the HikaShop address data
			$this->addressTable->save($this->state);
		}
		catch (\Exception $exception)
		{
			$this->log->add('Cannot add HikaShop address. Error: ' . $exception->getMessage(), false);
			$this->log->addStats('incorrect', $exception->getMessage());
		}
	}
}
