<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\import;

defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;

/**
 * Main processor for handling discount details.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.3.0
 */
class Discount extends \RantaiImportEngine
{
	/**
	 * The product table
	 *
	 * @var    \HikashopTableCategory
	 * @since  6.3.0
	 */
	private $discountTable = null;

	/**
	 * The HikaShop helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop
	 * @since  6.3.0
	 */
	protected $helper = null;

	/**
	 * The addon config helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop_Config
	 * @since  6.3.0
	 */
	protected $helperconfig = null;

	/**
	 * The HikaShop Category helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop_Category_Category
	 * @since  6.3.0
	 */
	protected $categoryHelper = null;


	/**
	 * Here starts the processing.
	 *
	 * @return  bool  Returns true on success | False on failure.
	 *
	 * @since   6.3.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				// Check if the field needs extra treatment
				switch ($name)
				{
					case 'discount_published':
					case 'discount_category_childs':
					case 'discount_auto_load':
					case 'discount_coupon_product_only':
						switch ($value)
						{
							case 'n':
							case 'N':
							case '0':
								$value = 0;
								break;
							default:
								$value = 1;
								break;
						}

						$this->setState($name, $value);
						break;
					case 'category_id':
					case 'category_path':
						if (strlen(trim($value)) > 0)
						{
							if (stripos($value, '|') > 0)
							{
								$category_ids[$name] = explode("|", $value);
							}
							else
							{
								$category_ids[$name][] = $value;
							}

							$this->setState('category_ids', $category_ids);
						}

						$this->setState($name, $value);
						break;
					case 'discount_start':
					case 'discount_end':
						$this->setState($name, $this->convertDate($value, 'unix', 'user'));
						break;
					case 'discount_flat_amount':
					case 'discount_percent_amount':
					case 'discount_minimum_order':
						$this->setState($name, $this->toPeriod($value));
						break;
					case 'discount_coupon_nodoubling':
						// Make sure the value is valid
						if (!in_array($value, array(1,2)))
						{
							$value = 0;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		if ($this->getState('discount_code', false))
		{
			// Load the current discount data
			$this->loaded = true;

			// See if we have a discount_id
			if (!$this->getState('discount_id', false))
			{
				// Find the discount ID
				$query = $this->db->getQuery(true)
					->select($this->db->quoteName('discount_id'))
					->from($this->db->quoteName('#__hikashop_discount'))
					->where($this->db->quoteName('discount_code') . ' = ' . $this->db->quote($this->getState('discount_code')));
				$this->db->setQuery($query);
				$this->log->add('Find the discount ID', false);
				$this->setState('discount_id', $this->db->loadResult());
			}

			if (!$this->getState('discount_code', false))
			{

			}
			else
			{
				if ($this->discountTable->load($this->getState('discount_id', false)))
				{
					if (!$this->template->get('overwrite_existing_data'))
					{
						$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_CONTENT', $this->getState('discount_code', '')));
						$this->loaded = false;
					}
				}
			}
		}
		else
		{
			$this->loaded = false;

			$this->log->addStats('skipped', \JText::_('COM_CSVI_NO_DISCOUNT_CODE_SET'), 'discount');
		}

		return true;
	}

	/**
	 * Process each record and store it in the database.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   3.0
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			// Get the needed data
			$discount_id = $this->getState('discount_id', false);
			$discount_delete = $this->getState('discount_delete', 'N');

			// User wants to delete the product
			if ($discount_id && $discount_delete == 'Y')
			{
				$this->deleteDiscount();
			}
			elseif (!$discount_id && $discount_delete == 'Y')
			{
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_NO_DISCOUNT_ID_NO_DELETE', $this->get('discount_code')));
			}
			elseif (!$discount_id && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new products when user chooses to ignore new products
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->get('discount_code')));
			}
			else
			{
				require_once JPATH_PLUGINS . '/csviaddon/hikashop/com_hikashop/helper/com_hikashop_category_category.php';
				$this->categoryHelper = new \Com_HikashopHelperCom_Hikashop_Category_Category($this->db, $this->template, $this->log);

				// Handle the products
				$this->processProducts();

				// Handle the categories
				if ($this->getState('category_path', false) || $this->getState('category_id', false))
				{
					if ($this->getState('category_ids', false) || $this->getState('category_id', false))
					{
						$this->processCategories();
					}
				}

				// Get the currency ID
				if ($this->getState('currency_code'))
				{
					$currencyId = $this->helper->getCurrencyId($this->getState('currency_code'));

					if (!$currencyId)
					{
						$currencyId = $this->helperconfig->get('main_currency');
					}

					$this->setState('discount_currency_id', $currencyId);
				}

				// Handle the access level(s)
				$this->processAccessLevel();

				// Handle the zones
				$this->processZones();

				// Handle the tax ID
				if ($this->getState('tax_name'))
				{
					$taxId = $this->categoryHelper->getCategoryIdByPath($this->getState('tax_name'), 'tax');

					if ($taxId)
					{
						$this->setState('discount_tax_id', $taxId);
					}
				}

				// Set default values
				if (!$discount_id)
				{
					$this->setState('discount_type', $this->getState('discount_type', 'discount'));
					$this->setState('discount_tax_id', $this->getState('discount_tax_id', 0));
					$this->setState('discount_minimum_products', $this->getState('discount_minimum_products', 0));
					$this->setState('discount_quota_per_user', $this->getState('discount_quota_per_user', 0));
					$this->setState('discount_coupon_nodoubling', $this->getState('discount_coupon_nodoubling', 0));
					$this->setState('discount_coupon_product_only', $this->getState('discount_coupon_product_only', 0));
					$this->setState('discount_affiliate', $this->getState('discount_affiliate', -1));
				}

				// Bind the data
				$this->discountTable->bind($this->state);

				// Store the product
				if (!$this->discountTable->store())
				{
					return false;
				}
			}

			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Load the discount related tables.
	 *
	 * @return  void.
	 *
	 * @since   6.3.0
	 */
	public function loadTables()
	{
		// Load the main tables
		$this->discountTable = $this->getTable('Discount');
	}

	/**
	 * Cleaning the product related tables.
	 *
	 * @return  void.
	 *
	 * @since   6.3.0
	 */
	public function clearTables()
	{
		// Clean the main tables
		$this->discountTable->reset();
	}

	/**
	 * Delete a discount.
	 *
	 * @return  void.
	 *
	 * @since   6.3.0
	 */
	private function deleteDiscount()
	{
		if ($this->discountTable->delete($this->getState('discount_id')))
		{
			$this->log->addstats('deleted', 'com_csvi_discount_deleted');
		}
		else
		{
			$this->log->addStats('incorrect', 'COM_CSVI_DISCOUNT_NOT_DELETED');
		}
	}

	/**
	 * Process the products.
	 *
	 * @return  bool  True on OK | False on failure.
	 *
	 * @since   6.3.0
	 */
	private function processProducts()
	{
		$products = explode('|', $this->getState('product_code'));
		$productIds = array();

		foreach ($products as $product_code)
		{
			// Get the product ID
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('product_id'))
				->from($this->db->quoteName('#__hikashop_product'))
				->where($this->db->quoteName('product_code') . ' = ' . $this->db->quote($product_code));
			$this->db->setQuery($query);

			$productId = $this->db->loadResult();

			$this->log->add('Check for product code' . $product_code);

			if ($productId)
			{
				$productIds[] = $productId;
			}
		}

		$productIds = ArrayHelper::toInteger($productIds);

		if ($productIds)
		{
			$this->setState('discount_product_id', implode(',', $productIds));
		}
	}

	/**
	 * Process the categories.
	 *
	 * @return  bool  True on sucess | False on failure.
	 *
	 * @since   6.3.0
	 */
	private function processCategories()
	{
		$areas = $this->getState('category_ids');

		if ($areas)
		{
			// Check the categories
			if (array_key_exists('category_id', $areas))
			{
				// We have category IDs, just add them
				$this->setState('discount_category_id', implode(',', $areas['category_id']));

			}
			elseif (array_key_exists('category_path', $areas))
			{
				$categoryIds = array();

				foreach ($areas['category_path'] as $order => $path)
				{
					$this->log->add('Process path: ' . $path, false);

					$categoryIds[] = $this->categoryHelper->getCategoryIdByPath($path);
				}

				$categoryIds = ArrayHelper::toInteger($categoryIds);

				if ($categoryIds)
				{
					$this->setState('discount_category_id', implode(',', $categoryIds));
				}
			}
		}
	}

	/**
	 * Process the shopper groups.
	 *
	 * @return  bool  True on OK | False on failure.
	 *
	 * @since   4.5.2
	 */
	private function processAccessLevel()
	{
		$accessfield = $this->getState('usergroup_name', false);

		if ($accessfield && $accessfield != 'all')
		{
			// Get the shopper group names
			$names = explode('|', $accessfield);
			$ids = array();

			foreach ($names as $name)
			{
				$ids[] = $this->helper->getAccessLevelId($name);
			}

			// Clear the array
			$ids = ArrayHelper::toInteger($ids);

			// Set the access level ID for other updates
			$this->setState('discount_access', implode(',', $ids));
		}

		return true;
	}

	/**
	 * Process the zones.
	 *
	 * @return  bool  True on OK | False on failure.
	 *
	 * @since   6.3.0
	 */
	private function processZones()
	{
		$ids = array();

		// Process the states
		$states = $this->getState('state_name', false);

		if ($states)
		{
			// Get the shopper group names
			$names = explode('|', $states);

			foreach ($names as $name)
			{
				$namekey = $this->helper->getZoneNameKey($name, 'state');

				if ($namekey)
				{
					$ids[] = $namekey;
				}
			}
		}


		// Process the countries
		$countries = $this->getState('country_name', false);

		if ($countries)
		{
			// Get the shopper group names
			$names = explode('|', $countries);

			foreach ($names as $name)
			{
				$namekey = $this->helper->getZoneNameKey($name, 'country');

				if ($namekey)
				{
					$ids[] = $namekey;
				}
			}
		}

		// Set the access level ID for other updates
		$this->setState('discount_zone_id', implode(',', $ids));

		return true;
	}
}
