<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\import;

defined('_JEXEC') or die;

/**
 * Order import.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       7.0
 */
class Order extends \RantaiImportEngine
{
	/**
	 * Order table.
	 *
	 * @var    \HikaShopTableOrder
	 * @since  7.0
	 */
	private $orderTable = null;

	/**
	 * Address table.
	 *
	 * @var    \HikashopTableAddress
	 * @since  7.0
	 */
	private $addressTable = null;

	/**
	 * History table.
	 *
	 * @var    \HikashopTableOrderHistory
	 * @since  7.0
	 */
	private $orderHistoryTable = null;

	/**
	 * Process billing address.
	 *
	 * @var   Bool
	 * @since  7.0
	 */
	private $processBillingAddress = false;

	/**
	 * Process shipping address.
	 *
	 * @var   Bool
	 * @since  7.0
	 */
	private $processShippingAddress = false;

	/**
	 * Start the product import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   7.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'order_full_price':
					case 'order_discount_price':
					case 'order_discount_tax':
					case 'order_payment_price':
					case 'order_payment_tax':
					case 'order_shipping_price':
					case 'order_shipping_tax':
					case 'order_partner_price':
						$this->setState($name, $this->cleanPrice($value));
						break;
					case 'shipping_address_published':
					case 'shipping_address_default':
					case 'billing_address_published':
					case 'billing_address_default':
						switch (strtolower($value))
						{
							case 'n':
							case 'no':
							case '0':
								$value = 0;
								break;
							default:
								$value = 1;
								break;
						}

						$this->setState($name, $value);
						break;
					case 'order_created':
					case 'order_modified':
					case 'order_invoice_created':
						if (!is_numeric($value))
						{
							$this->setState($name, strtotime($value));
						}

						break;
					default:
						// Check if there is a billing or shipping field
						if (0 === strpos($name, 'billing'))
						{
							$this->processBillingAddress = true;
						}
						elseif (0 === strpos($name, 'shipping'))
						{
							$this->processShippingAddress = true;
						}

						$this->setState($name, $value);
						break;
				}
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if required fields not found.
	 *
	 * @since   7.0
	 */
	public function getProcessRecord()
	{
		// Load the order user details
		$user_cms_id = $this->getState('user_cms_id', false);
		$email = $this->getState('user_email', false);

		if (!$user_cms_id && $email)
		{
			$query = $this->db->getQuery(true);
			$query->select($this->db->quoteName('id'));
			$query->from($this->db->quoteName('#__users'));
			$query->where($this->db->quoteName('email') . ' = ' . $this->db->quote($email));
			$this->db->setQuery($query);
			$user_cms_id = $this->db->loadResult();
			$this->setState('user_cms_id', $user_cms_id);
			$this->log->add('Found user cms id' . $user_cms_id);

			$query = $this->db->getQuery(true);
			$query->select($this->db->quoteName('user_id'));
			$query->from($this->db->quoteName('#__hikashop_user'));
			$query->where($this->db->quoteName('user_email') . ' = ' . $this->db->quote($email));
			$this->db->setQuery($query);
			$userId = $this->db->loadResult();
			$this->setState('user_id', $userId);
			$this->setState('order_user_id', $userId);
			$this->log->add('Load user id ' . $userId);

			if (!$userId)
			{
				$this->log->add('Not a valid user with email: ' . $email);
				$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_NOT_PROCESS_USER', $email));

				return false;
			}
		}

		// Check if we have an order ID
		$orderId = $this->getState('order_id', false);
		$orderNumber = $this->getState('order_number', false);

		if (!$orderId && $orderNumber)
		{
			$query = $this->db->getQuery(true);
			$query->select($this->db->quoteName('order_id'));
			$query->from($this->db->quoteName('#__hikashop_order'));
			$query->where($this->db->quoteName('order_number') . ' = ' . $this->db->quote($orderNumber));
			$this->db->setQuery($query);
			$orderId = $this->db->loadResult();

			if ($orderId)
			{
				$this->setState('order_id', $orderId);
				$this->log->add('Order id found');
			}
		}

		// Do not overwrite existing orders
		if ($orderId && !$this->template->get('overwrite_existing_data'))
		{
			$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_CONTENT', $orderId));

			return false;
		}

		// Do not create new orders
		if (!$orderId && $this->template->get('ignore_non_exist'))
		{
			// Do nothing for new orders when user chooses to ignore new orders
			$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $orderNumber));

			return false;
		}

		// Load the existing order data
		$this->orderTable->load($orderId);

		// Load the order if there is an order_id
		$keepId = (bool) $this->template->get('keepid', false);

		if (!$orderId || !$keepId)
		{
			// Add a creating date if there is no order id
			$this->orderTable->order_created = $this->date->toUnix();
		}

		// Check the user currency
		if (!$this->getState('order_currency_id', false) && $this->getState('user_currency_id', false))
		{
			$this->setState('order_currency_id', $this->getState('user_currency_id', false));
		}

		// Check the payment method ID
		if (!$this->getState('order_payment_id', false))
		{
			$this->setState('order_payment_id', 0);

			// Handle the payment method ID
			if ($this->getState('order_payment_method', false))
			{
				$query = $this->db->getQuery(true);
				$query->select($this->db->quoteName('payment_id'));
				$query->from($this->db->quoteName('#__hikashop_payment'));
				$query->where($this->db->quoteName('payment_name') . ' = ' . $this->db->quote($this->getState('order_payment_method')));
				$this->db->setQuery($query);
				$this->setState('order_payment_id', $this->db->loadResult());
				$this->log->add('Found payment method');
			}
		}

		// Check the shipping method
		if (!$this->getState('order_shipping_id', false))
		{
			$this->setState('order_shipping_id', 0);

			// Handle the payment method ID
			if ($this->getState('order_shipping_method', false))
			{
				$query = $this->db->getQuery(true);
				$query->select($this->db->quoteName('shipping_id'));
				$query->from($this->db->quoteName('#__hikashop_shipping'));
				$query->where(
					$this->db->quoteName('shipping_name') . ' =
				' . $this->db->quote($this->getState('order_shipping_method'))
				);

				$this->db->setQuery($query);
				$this->setState('order_shipping_id', $this->db->loadResult());
				$this->log->add('Found shipping method');
			}
		}

		// Check and process product tax for an order
		if (!$this->getState('order_tax_info', false))
		{
			$parameters = array(
				'tax_namekey',
				'tax_amount',
				'tax_amount_for_coupon',
			);

			$taxArray = array();
			$data     = new \stdClass;
			$loaded   = $this->orderTable->get('order_id', false);

			foreach ($parameters as $parameter)
			{
				$default = '';

				if ($loaded)
				{
					$default = $this->orderTable->get($parameter);
				}

				$data->$parameter = $this->getState($parameter, $default);
			}

			$taxName = $this->getState('tax_namekey', false);

			if (!$taxName)
			{
				$query = $this->db->getQuery(true);
				$query->clear();
				$query->select($this->db->quoteName('tax_namekey'));
				$query->from($this->db->quoteName('#__hikashop_tax'));
				$this->db->setQuery($query);
				$taxName = $this->db->loadResult();
			}

			$taxArray[$taxName] = $data;

			$this->setState('order_tax_info', serialize($taxArray));
		}

		// Check the IP address
		if (!$this->getState('order_ip', false))
		{
			$this->setState('order_ip', $_SERVER['SERVER_ADDR']);
		}

		// Add the modification date
		if (!$this->getState('order_modified', false))
		{
			$this->orderTable->order_modified = $this->date->toUnix();
		}

		// Bind order data
		$this->orderTable->bind($this->state);

		try
		{
			// Check if we use a given order id
			if ($keepId && $this->getState('order_id', false))
			{
				$this->orderTable->check();
			}

			$this->orderTable->store();
			$this->log->add('Order Added');
			$this->setState('order_id', $this->orderTable->order_id);

			if ($this->getState('history_id', false))
			{
				$this->orderHistoryTable->load($this->getState('history_id', false));
			}

			if (!$this->getState('history_new_status', false))
			{
				$this->orderHistoryTable->history_new_status = 'created';
			}

			$historyDate = $this->date->toUnix();

			if ($keepId && $this->getState('order_id', false))
			{
				$historyDate = $this->orderTable->get('order_created');
			}

			$this->orderHistoryTable->history_created = $historyDate;
			$this->orderHistoryTable->history_user_id = $this->getState('user_id');
			$this->orderHistoryTable->history_ip = $this->getState('order_ip', false);
			$this->orderHistoryTable->history_order_id = $this->getState('order_id', false);
			$this->orderHistoryTable->bind($this->state);
			$this->orderHistoryTable->store();
			$this->log->add('Order history added');
		}
		catch (\Exception $e)
		{
			$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_ORDER_NOT_ADDED', $e->getMessage()));
			$this->log->add($e->getMessage(), false);
		}


		if ($this->getState('user_id'))
		{
			$data = array();

			if ($this->getState('order_id'))
			{
				$this->orderTable->load($this->getState('order_id'));
				$data['order_id'] = $this->getState('order_id');

				// Check if we need to process billing address details
				if ($this->processBillingAddress)
				{
					$this->addBillingAddress();
				}

				if ($this->getState('order_billing_address_id'))
				{
					$data['order_billing_address_id'] = $this->getState('order_billing_address_id');
				}

				// Check if we need to process shipping address details
				if ($this->processShippingAddress)
				{
					$this->addShippingAddress();
				}

				if ($this->getState('order_shipping_address_id'))
				{
					$data['order_shipping_address_id'] = $this->getState('order_shipping_address_id');
				}
				elseif (!$this->getState('order_shipping_address_id'))
				{
					$data['order_shipping_address_id'] = $this->getState('order_billing_address_id');
				}

				try
				{
					$this->orderTable->save($data);
					$this->log->add('Address id updated in orders');
				}
				catch (\Exception $e)
				{
					$this->log->add('Address not added: ' . $e->getMessage(), false);
					$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_ORDERADDRESS_NOT_ADDED', $e->getMessage()));
				}
			}
		}

		return true;
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function loadTables()
	{
		$this->orderTable        = $this->getTable('Order');
		$this->addressTable      = $this->getTable('Address');
		$this->orderHistoryTable = $this->getTable('OrderHistory');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function clearTables()
	{
		$this->orderTable->reset();
		$this->addressTable->reset();
		$this->orderHistoryTable->reset();
	}

	/**
	 * Save the billing address details
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function addBillingAddress()
	{
		$query = $this->db->getQuery(true);

		// Check for country ID
		if (!$this->getState('billing_address_country', false))
		{
			$whereCountryConditions = array();

			if ($this->getState('billing_country_zone_code_2'))
			{
				$whereCountryConditions[] = $this->db->quoteName('zone_code_2') . ' = ' . $this->db->quote($this->getState('billing_country_zone_code_2'));
			}

			if ($this->getState('billing_country_zone_code_3'))
			{
				$whereCountryConditions[] = $this->db->quoteName('zone_code_3') . ' = ' . $this->db->quote($this->getState('billing_country_zone_code_3'));
			}

			if ($this->getState('billing_country_zone_name'))
			{
				$whereCountryConditions[] = $this->db->quoteName('zone_name') . ' = ' . $this->db->quote($this->getState('billing_country_zone_name'));
			}

			if ($this->getState('billing_country_zone_name_english'))
			{
				$whereCountryConditions[] = $this->db->quoteName('zone_name_english') . ' = ' . $this->db->quote($this->getState('billing_country_zone_name_english'));
			}

			if (!empty($whereCountryConditions))
			{
				$whereCountryConditions[] = $this->db->quoteName('zone_type') . ' = ' . $this->db->quote('country');
				$query->clear()
					->select($this->db->quoteName('zone_namekey'))
					->from($this->db->quoteName('#__hikashop_zone'));
				$query->where('(' . implode(' AND ', $whereCountryConditions) . ')');
				$this->db->setQuery($query);
				$this->setState('billing_address_country', $this->db->loadResult());
			}
		}

		// Check for state ID
		if (!$this->getState('billing_address_state', false))
		{
			$whereStateConditions = array();

			if ($this->getState('billing_state_zone_code_2'))
			{
				$whereStateConditions[] = $this->db->quoteName('zone_code_2') . ' = ' . $this->db->quote($this->getState('billing_state_zone_code_2'));
			}

			if ($this->getState('billing_state_zone_code_3'))
			{
				$whereStateConditions[] = $this->db->quoteName('zone_code_3') . ' = ' . $this->db->quote($this->getState('billing_state_zone_code_3'));
			}

			if ($this->getState('billing_state_zone_name'))
			{
				$whereStateConditions[] = $this->db->quoteName('zone_name') . ' = ' . $this->db->quote($this->getState('billing_state_zone_name'));
			}

			if ($this->getState('billing_state_zone_name_english'))
			{
				$whereStateConditions[] = $this->db->quoteName('zone_name_english') . ' = ' . $this->db->quote($this->getState('billing_state_zone_name_english'));
			}

			if (!empty($whereStateConditions))
			{
				$whereStateConditions[] = $this->db->quoteName('zone_type') . ' = ' . $this->db->quote('state');
				$query->clear()
					->select($this->db->quoteName('zone_namekey'))
					->from($this->db->quoteName('#__hikashop_zone'));
				$query->where('(' . implode(' AND ', $whereStateConditions) . ')');
				$this->db->setQuery($query);
				$this->setState('billing_address_state', $this->db->loadResult());
			}
		}

		$this->addressTable->address_user_id = $this->getState('user_id', false);
		$where = array();

		// Company
		if ($this->getState('billing_address_company', false))
		{
			$this->addressTable->address_company = $this->getState('billing_address_company', false);
		}

		// Title
		if ($this->getState('billing_address_title', false))
		{
			$this->addressTable->address_title = $this->getState('billing_address_title', false);
		}

		// First name
		if ($this->getState('billing_address_firstname', false))
		{
			$this->addressTable->address_firstname = $this->getState('billing_address_firstname', false);
		}

		// Last name
		if ($this->getState('billing_address_lastname', false))
		{
			$this->addressTable->address_lastname = $this->getState('billing_address_lastname', false);
		}

		// Middle name
		if ($this->getState('billing_address_middle_name', false))
		{
			$this->addressTable->address_middle_name = $this->getState('billing_address_middle_name', false);
		}

		// Street
		if ($this->getState('billing_address_street', false))
		{
			$this->addressTable->address_street = $this->getState('billing_address_street', false);
			$where[]                            = $this->db->quoteName('address_street') . ' = ' . $this->db->quote($this->getState('billing_address_street'));
		}

		// Street2
		if ($this->getState('billing_address_street2', false))
		{
			$this->addressTable->address_street2 = $this->getState('billing_address_street2', false);
			$where[]                             = $this->db->quoteName('address_street2') . ' = ' . $this->db->quote($this->getState('billing_address_street2'));
		}

		// Post code
		if ($this->getState('billing_address_post_code', false))
		{
			$this->addressTable->address_post_code = $this->getState('billing_address_post_code', false);
			$where[]                               = $this->db->quoteName('address_post_code') . ' = ' . $this->db->quote($this->getState('billing_address_post_code'));
		}

		// City
		if ($this->getState('billing_address_city', false))
		{
			$this->addressTable->address_city = $this->getState('billing_address_city', false);
			$where[]                          = $this->db->quoteName('address_city') . ' = ' . $this->db->quote($this->getState('billing_address_city'));
		}

		// Telephone
		if ($this->getState('billing_address_telephone', false))
		{
			$this->addressTable->address_telephone = $this->getState('billing_address_telephone', false);
		}

		// Telephone2
		if ($this->getState('billing_address_telephone2', false))
		{
			$this->addressTable->address_telephone2 = $this->getState('billing_address_telephone2', false);
		}

		// State
		if ($this->getState('billing_address_state', false))
		{
			$this->addressTable->address_state = $this->getState('billing_address_state', false);
			$where[]                           = $this->db->quoteName('address_state') . ' = ' . $this->db->quote($this->getState('billing_address_state'));
		}

		// Country
		if ($this->getState('billing_address_country', false))
		{
			$this->addressTable->address_country = $this->getState('billing_address_country', false);
			$where[]                             = $this->db->quoteName('address_country') . ' = ' . $this->db->quote($this->getState('billing_address_country'));
		}

		// Vat
		if ($this->getState('billing_address_vat', false))
		{
			$this->addressTable->address_vat = $this->getState('billing_address_vat', false);
		}

		// Published
		if ($this->getState('billing_address_published', false))
		{
			$this->addressTable->address_published = $this->getState('billing_address_published', false);
		}

		// Default
		if ($this->getState('billing_address_default', false))
		{
			$this->addressTable->address_default = $this->getState('billing_address_default', false);
		}

		if (!empty($where))
		{
			$where[] = $this->db->quoteName('address_published') . ' = 1';
			$query   = $this->db->getQuery(true)
				->select($this->db->quoteName('address_id'))
				->from($this->db->quoteName('#__hikashop_address'));
			$query->where('(' . implode(' AND ', $where) . ')');
			$this->db->setQuery($query);
			$this->setState('address_id', $this->db->loadResult());
			$this->log->add('Found order billing address id');
		}

		if ($this->getState('address_id', false))
		{
			$this->addressTable->address_id = $this->getState('address_id');
		}

		// Set the default address type
		$this->addressTable->address_type    = 'billing';

		// Bind the address data
		$this->addressTable->bind($this->state);

		try
		{
			if (!empty($where))
			{
				$this->addressTable->store();
				$this->log->add('Billing address saved');
				$this->setState('order_billing_address_id', $this->addressTable->address_id);
			}

			$this->addressTable->reset();

		}
		catch (\Exception $e)
		{
			$this->log->add('Billing address not added: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_ORDERADDRESS_NOT_ADDED', $e->getMessage()));
		}
	}

	/**
	 * Save the shipping address details
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function addShippingAddress()
	{
		// Check for country ID
		if (!$this->getState('shipping_address_country', false))
		{
			$whereCountryConditions = array();

			if ($this->getState('shipping_country_zone_code_2'))
			{
				$whereCountryConditions[] = $this->db->quoteName('zone_code_2') . ' = ' . $this->db->quote($this->getState('shipping_country_zone_code_2'));
			}

			if ($this->getState('shipping_country_zone_code_3'))
			{
				$whereCountryConditions[] = $this->db->quoteName('zone_code_3') . ' = ' . $this->db->quote($this->getState('shipping_country_zone_code_3'));
			}

			if ($this->getState('shipping_country_zone_name'))
			{
				$whereCountryConditions[] = $this->db->quoteName('zone_name') . ' = ' . $this->db->quote($this->getState('shipping_country_zone_name'));
			}

			if ($this->getState('shipping_country_zone_name_english'))
			{
				$whereCountryConditions[] = $this->db->quoteName('zone_name_english') . ' = ' . $this->db->quote($this->getState('shipping_country_zone_name_english'));
			}

			if (!empty($whereCountryConditions))
			{
				$whereCountryConditions[] = $this->db->quoteName('zone_type') . ' = ' . $this->db->quote('country');
				$query                    = $this->db->getQuery(true)
					->select($this->db->quoteName('zone_namekey'))
					->from($this->db->quoteName('#__hikashop_zone'));
				$query->where('(' . implode(' AND ', $whereCountryConditions) . ')');
				$this->db->setQuery($query);
				$this->setState('shipping_address_country', $this->db->loadResult());
			}
		}

		// Check for country ID
		if (!$this->getState('shipping_address_state', false))
		{
			$whereStateConditions = array();

			if ($this->getState('shipping_state_zone_code_2'))
			{
				$whereStateConditions[] = $this->db->quoteName('zone_code_2') . ' = ' . $this->db->quote($this->getState('shipping_state_zone_code_2'));
			}

			if ($this->getState('shipping_state_zone_code_3'))
			{
				$whereStateConditions[] = $this->db->quoteName('zone_code_3') . ' = ' . $this->db->quote($this->getState('shipping_state_zone_code_3'));
			}

			if ($this->getState('shipping_state_zone_name'))
			{
				$whereStateConditions[] = $this->db->quoteName('zone_name') . ' = ' . $this->db->quote($this->getState('shipping_state_zone_name'));
			}

			if ($this->getState('shipping_state_zone_name_english'))
			{
				$whereStateConditions[] = $this->db->quoteName('zone_name_english') . ' = ' . $this->db->quote($this->getState('shipping_state_zone_name_english'));
			}

			if (!empty($whereStateConditions))
			{
				$whereStateConditions[] = $this->db->quoteName('zone_type') . ' = ' . $this->db->quote('state');

				$query = $this->db->getQuery(true)
					->select($this->db->quoteName('zone_namekey'))
					->from($this->db->quoteName('#__hikashop_zone'));
				$query->where('(' . implode(' AND ', $whereStateConditions) . ')');
				$this->db->setQuery($query);
				$this->setState('shipping_address_state', $this->db->loadResult());
			}
		}

		$this->addressTable->address_user_id = $this->getState('user_id', false);
		$where                               = array();

		// Company
		if ($this->getState('shipping_address_company', false))
		{
			$this->addressTable->address_company = $this->getState('shipping_address_company', false);
		}

		// Title
		if ($this->getState('shipping_address_title', false))
		{
			$this->addressTable->address_title = $this->getState('shipping_address_title', false);
		}

		// First name
		if ($this->getState('shipping_address_firstname', false))
		{
			$this->addressTable->address_firstname = $this->getState('shipping_address_firstname', false);
		}

		// Last name
		if ($this->getState('shipping_address_lastname', false))
		{
			$this->addressTable->address_lastname = $this->getState('shipping_address_lastname', false);
		}

		// Middle name
		if ($this->getState('shipping_address_middle_name', false))
		{
			$this->addressTable->address_middle_name = $this->getState('shipping_address_middle_name', false);
		}

		// Street
		if ($this->getState('shipping_address_street', false))
		{
			$this->addressTable->address_street = $this->getState('shipping_address_street', false);
			$where[]                            = $this->db->quoteName('address_street') . ' = ' . $this->db->quote($this->getState('shipping_address_street'));
		}

		// Street2
		if ($this->getState('shipping_address_street2', false))
		{
			$this->addressTable->address_street2 = $this->getState('shipping_address_street2', false);
			$where[]                             = $this->db->quoteName('address_street2') . ' = ' . $this->db->quote($this->getState('shipping_address_street2'));
		}

		// Post code
		if ($this->getState('shipping_address_post_code', false))
		{
			$this->addressTable->address_post_code = $this->getState('shipping_address_post_code', false);
			$where[]                               = $this->db->quoteName('address_post_code') . ' = ' . $this->db->quote($this->getState('shipping_address_post_code'));
		}

		// City
		if ($this->getState('shipping_address_city', false))
		{
			$this->addressTable->address_city = $this->getState('shipping_address_city', false);
			$where[]                          = $this->db->quoteName('address_city') . ' = ' . $this->db->quote($this->getState('shipping_address_city'));
		}

		// Telephone
		if ($this->getState('shipping_address_telephone', false))
		{
			$this->addressTable->address_telephone = $this->getState('shipping_address_telephone', false);
		}

		// Telephone2
		if ($this->getState('shipping_address_telephone2', false))
		{
			$this->addressTable->address_telephone2 = $this->getState('shipping_address_telephone2', false);
		}

		// Vat
		if ($this->getState('shipping_address_vat', false))
		{
			$this->addressTable->address_vat = $this->getState('shipping_address_vat', false);
		}

		// Country
		if ($this->getState('shipping_address_country', false))
		{
			$this->addressTable->address_country = $this->getState('shipping_address_country', false);
			$where[]                             = $this->db->quoteName('address_country') . ' = ' . $this->db->quote($this->getState('shipping_address_country'));
		}

		// State
		if ($this->getState('shipping_address_state', false))
		{
			$this->addressTable->address_state = $this->getState('shipping_address_state', false);
			$where[]                           = $this->db->quoteName('address_state') . ' = ' . $this->db->quote($this->getState('shipping_address_state'));
		}

		// Published
		if ($this->getState('shipping_address_published', false))
		{
			$this->addressTable->address_published = $this->getState('shipping_address_published', false);
		}

		// Default
		if ($this->getState('shipping_address_default', false))
		{
			$this->addressTable->address_default = $this->getState('shipping_address_default', false);
		}

		// Check if there is already a matching address details
		if (!empty($where))
		{
			$where[] = $this->db->quoteName('address_published') . ' = 1';
			$query   = $this->db->getQuery(true)
				->select($this->db->quoteName('address_id'))
				->from($this->db->quoteName('#__hikashop_address'));
			$query->where('(' . implode(' AND ', $where) . ')');
			$this->db->setQuery($query);
			$this->setState('address_id', $this->db->loadResult());
			$this->log->add('Found order shipping address id');
		}

		if ($this->getState('address_id', false))
		{
			$this->addressTable->address_id = $this->getState('address_id');
		}

		$this->addressTable->address_type = 'shipping';

		// Bind the address data
		$this->addressTable->bind($this->state);

		try
		{
			if (!empty($where))
			{
				$this->addressTable->store();
				$this->log->add('Shipping address saved');
				$this->setState('order_shipping_address_id', $this->addressTable->address_id);
			}

			$this->addressTable->reset();
		}
		catch (\Exception $e)
		{
			$this->log->add('Shipping address not saved: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_ORDERADDRESS_NOT_ADDED', $e->getMessage()));
		}
	}
}
