<?php
/**
 * @package     CSVI
 * @subpackage  Hikashop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\import;

defined('_JEXEC') or die;

/**
 * Price import.
 *
 * @package     CSVI
 * @subpackage  Hikashop
 * @since       6.5.0
 */
class Price extends \RantaiImportEngine
{
	/**
	 * Price table.
	 *
	 * @var    \HikashopTablePrice
	 * @since  6.5.0
	 */
	private $productPriceTable = null;

	/**
	 * The addon helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop
	 * @since  6.5.0
	 */
	protected $helper = null;

	/**
	 * CSVI fields
	 *
	 * @var    \CsviHelperImportfields
	 * @since  6.5.0
	 */
	protected $fields = null;

	/**
	 * Start the product import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   6.5.0
	 */
	public function getStart()
	{
		$this->setState('price_product_id', $this->helper->getProductId());

		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'price_value':
					case 'price_value_new':
						$this->setState($name, $this->toPeriod($value));
						break;
					case 'price_access':
					case 'price_access_new':
						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		// Reset loaded state
		$this->loaded = true;

		if ($this->getState('product_code', false) && $this->getState('price_value', false))
		{
			// Get the product ID if we don't already have it
			if (!$this->getState('price_product_id', false))
			{
				$this->setState('price_product_id', $this->helper->getProductId());
			}

			if ($this->getState('currency_code', false))
			{
				$this->setState('price_currency_id', $this->helper->getCurrencyId($this->getState('currency_code')));
			}

			// Check if we need to set a currency
			if (!$this->getState('currency_code', false) && !$this->getState('price_currency_id', false))
			{
				$this->setState('price_currency_id', $this->helperconfig->get('main_currency'));
			}

			// Validate the price access level
			if ($this->getState('price_access', false))
			{
				// Check if we need to change the access
				$price_access = $this->helper->processAccessLevel('price_access');

				if ($price_access)
				{
					$this->setState('price_access', $price_access);
				}
				else
				{
					$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_PRICE_ACCESSGROUP_NOT_EXIST', $this->getState('price_access', false)));

					return false;
				}
			}

			// Check if price minimum quantity is 1, set it as 0 as per Hikashop rules
			if ($this->getState('price_min_quantity', false) && $this->getState('price_min_quantity') == 1)
			{
				$this->setState('price_min_quantity', 0);
			}

			// Bind the values
			$this->productPriceTable->bind($this->state);

			if ($this->productPriceTable->check())
			{
				$this->setState('price_id', $this->productPriceTable->get('price_id'));

				// Check if we have an existing item
				if ($this->getState('price_id', 0) > 0 && !$this->template->get('overwrite_existing_data', true))
				{
					$this->log->add(\JText::sprintf('COM_CSVI_DATA_EXISTS_CONTENT', $this->getState('product_code')));
					$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_CONTENT', $this->getState('product_code')));
					$this->loaded = false;
				}
				else
				{
					// Load the current content data
					$this->productPriceTable->load();
					$this->loaded = true;
				}
			}
		}
		else
		{
			$this->loaded = false;

			$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_MISSING_REQUIRED_FIELDS', 'product_code, price_value'));
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product code or product ID can be found.
	 *
	 * @since   6.5.0
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			if (!$this->getState('price_id', false) && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new rules when user chooses to ignore new rules
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('product_code')));
			}
			else
			{
				// Check if the user wants to delete a price
				if (strtoupper($this->getState('price_delete')) == 'Y')
				{
					if ($this->getState('price_id', false))
					{
						$this->productPriceTable->delete($this->getState('price_id'));
					}
					else
					{
						$this->log->addStats('incorrect', 'COM_CSVI_PRICE_NOT_DELETED_NO_ID');
					}
				}
				else
				{
					if (!$this->getState('price_product_id'))
					{
						$this->log->add(\JText::sprintf('COM_CSVI_NO_PRODUCT_ID_FOUND', $this->getState('product_code')));
						$this->log->AddStats('skipped', \JText::sprintf('COM_CSVI_NO_PRODUCT_ID_FOUND', $this->getState('product_code')));
					}
					elseif (!$this->getState('price_id', false) && !$this->getState('price_value'))
					{
						$this->log->add('COM_CSVI_NO_PRODUCT_PRICE_FOUND');
						$this->log->AddStats('skipped', 'COM_CSVI_NO_PRODUCT_PRICE_FOUND');
					}
					else
					{
						// Check if we need to change the product price with new value
						if ($this->getState('price_value_new', false))
						{
							$this->productPriceTable->set('price_value', $this->getState('price_value_new', false));
						}

						// This is an existing price
						if ($this->getState('price_id', false))
						{
							// See if we need to change the usergroup
							if ($price_access_new = $this->getState('price_access_new', false))
							{
								if ($price_access_new != 'all')
								{
									$price_access_new = $this->helper->processAccessLevel('price_access_new');
								}

								if ($price_access_new)
								{
									$this->productPriceTable->set('price_access', $price_access_new);
								}
								else
								{
									$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_PRICE_ACCESSGROUP_NOT_EXIST', $this->getState('price_access_new', false)));

									return false;
								}
							}
						}
						else
						{
							// Check if we need to assign the all shopper group, if no shopper group has been set
							$priceAccess = $this->getState('price_access', false);
							$this->setState('price_access', ($priceAccess) ? $priceAccess : 'all');
						}

						// Price doesn't exist
						if ($this->getState('price_site_id', false))
						{
							$this->productPriceTable->set('price_site_id', $this->getState('price_site_id', false));
						}

						// Store the price
						if (!$this->productPriceTable->store())
						{
							$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_MULTIPLE_PRICES_NOT_ADDED', $this->productPriceTable->getError()));
						}
					}
				}
			}

			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   6.5.0
	 */
	public function loadTables()
	{
		$this->productPriceTable = $this->getTable('Price');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   6.5.0
	 */
	public function clearTables()
	{
		$this->productPriceTable->reset();
	}
}
