<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\import;

defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;

/**
 * Main processor for handling shipping rates.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.6.0
 */
class Shippingrate extends \RantaiImportEngine
{
	/**
	 * The shipping rates table
	 *
	 * @var    \HikashopTableShipping
	 * @since  6.6.0
	 */
	private $shippingTable;

	/**
	 * The HikaShop helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop
	 * @since  6.6.0
	 */
	protected $helper;

	/**
	 * The addon config helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop_Config
	 * @since  6.6.0
	 */
	protected $helperconfig;

	/**
	 * The category helper.
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop_Category
	 * @since  6.6.0
	 */
	private $categoryHelper;

	/**
	 * Here starts the processing.
	 *
	 * @return  bool  Returns true on success | False on failure.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \RuntimeException
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				// Check if the field needs extra treatment
				switch ($name)
				{
					case 'shipping_published':
					case 'shipping_virtual_included':
					case 'shipping_per_product':
						switch (strtolower($value))
						{
							case 'y':
							case 'yes':
							case '1':
								$value = 1;
								break;
							default:
								$value = 0;
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		if ($this->getState('shipping_name', false) && $this->getState('shipping_type', false))
		{
			// Load the current discount data
			$this->loaded = true;

			// See if we have a discount_id
			if (!$this->getState('shipping_id', false))
			{
				// Find the shipping ID
				$query = $this->db->getQuery(true)
					->select($this->db->quoteName('shipping_id'))
					->from($this->db->quoteName('#__hikashop_shipping'))
					->where($this->db->quoteName('shipping_name') . ' = ' . $this->db->quote($this->getState('shipping_name')));
				$this->db->setQuery($query);
				$this->log->add('Find the shipping ID');
				$this->setState('shipping_id', $this->db->loadResult());
			}

			if (!$this->template->get('overwrite_existing_data') && $this->shippingTable->load($this->getState('shipping_id', false)))
			{
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_CONTENT', $this->getState('shipping_name', '')));
				$this->loaded = false;
			}
		}
		else
		{
			$this->loaded = false;

			$this->log->addStats('skipped', \JText::_('COM_CSVI_NO_SHIPPING_NAME_OR_SHIPPING_TYPE_SET'));
		}

		return true;
	}

	/**
	 * Process each record and store it in the database.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \CsviException
	 * @throws  \RuntimeException
	 * @throws  \InvalidArgumentException
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			// Get the needed data
			$shipping_id = $this->getState('shipping_id', false);
			$shipping_delete = $this->getState('shippingrate_delete', 'N');

			// User wants to delete the product
			if ($shipping_id && $shipping_delete === 'Y')
			{
				$this->deleteShipping();
			}
			elseif (!$shipping_id && $shipping_delete === 'Y')
			{
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_NO_SHIPPING_ID_NO_DELETE', $this->getState('shipping_name')));
			}
			elseif (!$shipping_id && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new shipping rates when user chooses to ignore new items
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('shipping_name')));
			}
			else
			{
				// Process the currency
				$this->processCurrency();

				// Handle the access level(s)
				$this->processAccessLevel();

				// Process the warehouse
				$this->processWarehouse();

				// Process the restricted currencies
				$this->processRestrictedCurrencies();

				// Process the zones
				$this->processZones();

				// Process tax
				$this->processTax();

				// Process parameters
				$this->processParameters();

				// Set default values
				if (!$shipping_id)
				{
					$this->setState('shipping_type', $this->getState('shipping_type', 'manual'));
					$this->setState('shipping_tax_id', $this->getState('shipping_tax_id', 0));
					$this->setState('shipping_published', $this->getState('shipping_published', 0));
					$this->setState('shipping_access', $this->getState('shipping_access', 'all'));
				}

				// Bind the data
				$this->shippingTable->bind($this->state);

				// Store the product
				if (!$this->shippingTable->store())
				{
					return false;
				}
			}

			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Load the discount related tables.
	 *
	 * @return  void.
	 *
	 * @since   6.6.0
	 */
	public function loadTables()
	{
		// Load the main tables
		$this->shippingTable = $this->getTable('Shipping');
	}

	/**
	 * Cleaning the product related tables.
	 *
	 * @return  void.
	 *
	 * @since   6.3.0
	 */
	public function clearTables()
	{
		// Clean the main tables
		$this->shippingTable->reset();
	}

	/**
	 * Delete a shipping rate.
	 *
	 * @return  void.
	 *
	 * @since   6.6.0
	 */
	private function deleteShipping()
	{
		if (!$this->shippingTable->delete($this->getState('shipping_id')))
		{
			$this->log->addStats('incorrect', 'COM_CSVI_SHIPPING_NOT_DELETEDCOM_CSVI_SHIPPING_NOT_DELETED');
		}
	}

	/**
	 * Process the shipping currency.
	 *
	 * @return  void
	 *
	 * @since   6.6.0
	 */
	private function processCurrency()
	{
		// Get the currency ID
		if ($this->getState('currency_code'))
		{
			$currencyId = $this->helper->getCurrencyId($this->getState('currency_code'));

			if (!$currencyId)
			{
				$currencyId = $this->helperconfig->get('main_currency');
			}

			$this->setState('shipping_currency_id', $currencyId);
		}
	}

	/**
	 * Process the shopper groups.
	 *
	 * @return  bool  True on OK | False on failure.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \RuntimeException
	 */
	private function processAccessLevel()
	{
		$accessField = $this->getState('usergroup_name', false);

		if ($accessField && $accessField !== 'all')
		{
			// Get the shopper group names
			$names = explode('|', $accessField);
			$ids = array();

			foreach ($names as $name)
			{
				$ids[] = $this->helper->getAccessLevelId($name);
			}

			// Clear the array
			$ids = ArrayHelper::toInteger($ids);

			// Add the empty entries
			array_unshift($ids, '');

			$ids[] = '';

			// Set the access level ID for other updates
			$this->setState('shipping_access', implode(',', $ids));
		}

		return true;
	}

	/**
	 * Process the warehouse.
	 *
	 * @return  void
	 *
	 * @since   6.6.0
	 *
	 * @throws  \RuntimeException
	 */
	private function processWarehouse()
	{
		$warehouse = $this->getState('warehouse', false);

		if ($warehouse)
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('warehouse_id'))
				->from($this->db->quoteName('#__hikashop_warehouse'))
				->where($this->db->quoteName('warehouse_name') . ' = ' . $this->db->quote($warehouse));
			$this->db->setQuery($query);
			$this->setState('shipping_warehouse_filter', $this->db->loadResult());
		}
	}

	/**
	 * Process the restricted currencies.
	 *
	 * @return  void
	 *
	 * @since   6.6.0
	 *
	 * @throws  \RuntimeException
	 */
	private function processRestrictedCurrencies()
	{
		$restrictedCurrencies = $this->getState('restricted_currency_code', false);
		$this->log->add('Process restricted currencies ' . $restrictedCurrencies, false);

		if ($restrictedCurrencies)
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('currency_id'))
				->from($this->db->quoteName('#__hikashop_currency'))
				->where($this->db->quoteName('currency_code') . ' IN (' . implode(',', $this->db->quote(explode('|', $restrictedCurrencies))) . ')');
			$this->db->setQuery($query);
			$ids = $this->db->loadColumn();
			$this->log->add('Load the restricted currencies');

			array_unshift($ids, '');

			$ids[] = '';
			$this->log->add('Retrieved currencies: ' . implode(',', $ids), false);
			$this->setState('shipping_currency', implode(',', $ids));
		}
	}

	/**
	 * Process the zone.
	 *
	 * @return  bool  True on OK | False on failure.
	 *
	 * @since   6.6.0
	 */
	private function processZones()
	{
		$zone = $this->getState('zone', false);

		if ($zone)
		{
			$this->setState('shipping_zone_namekey', $this->helper->getZoneNameKey($zone));
		}

		return true;
	}

	/**
	 * Process tax.
	 *
	 * @return  void
	 *
	 * @since   6.6.0
	 *
	 * @throws  \CsviException
	 */
	private function processTax()
	{
		// Handle the tax ID
		if ($this->getState('tax_name'))
		{
			require_once JPATH_PLUGINS . '/csviaddon/hikashop/com_hikashop/helper/com_hikashop_category_category.php';
			$this->categoryHelper = new \Com_HikashopHelperCom_Hikashop_Category_Category($this->db, $this->template, $this->log);

			$taxId = $this->categoryHelper->getCategoryIdByPath($this->getState('tax_name'), 'tax');

			if ($taxId)
			{
				$this->setState('shipping_tax_id', $taxId);
			}
		}
	}

	/**
	 * Process the shipping parameters.
	 *
	 * @return  void
	 *
	 * @since   6.6.0
	 */
	private function processParameters()
	{
		if (!$this->getState('shipping_params', false))
		{
			$parameters = array(
				'shipping_percentage',
				'shipping_per_product',
				'shipping_price_per_product',
				'shipping_override_address',
				'shipping_override_address_text',
				'override_tax_zone',
				'shipping_warehouse_filter',
				'shipping_min_price',
				'shipping_max_price',
				'shipping_virtual_included',
				'shipping_price_use_tax',
				'shipping_min_quantity',
				'shipping_max_quantity',
				'shipping_min_weight',
				'shipping_weight_unit',
				'shipping_max_weight',
				'shipping_min_volume',
				'shipping_size_unit',
				'shipping_max_volume',
				'shipping_zip_prefix',
				'shipping_min_zip',
				'shipping_max_zip',
				'shipping_zip_suffix',
			);

			$data = new \stdClass;
			$loaded = $this->shippingTable->get('shipping_id', false);

			foreach ($parameters as $parameter)
			{
				$default = '';

				if ($loaded)
				{
					$default = $this->shippingTable->get($parameter);
				}

				$data->$parameter = $this->getState($parameter, $default);
			}

			$this->setState('shipping_params', serialize($data));
		}
	}
}
