<?php
/**
 * @package     CSVI
 * @subpackage  Hikashop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

require_once JPATH_PLUGINS . '/csviaddon/hikashop/com_hikashop/helper/com_hikashop_category.php';

/**
 * Hikashop maintenance.
 *
 * @package     CSVI
 * @subpackage  Hikashop
 * @since       6.6.0
 */
class Com_HikashopMaintenance
{
	/**
	 * Database connector
	 *
	 * @var    JDatabaseDriver
	 * @since  6.6.0
	 */
	private $db = null;

	/**
	 * Logger helper
	 *
	 * @var    CsviHelperLog
	 * @since  6.6.0
	 */
	private $log = null;

	/**
	 * CSVI Helper.
	 *
	 * @var    CsviHelperCsvi
	 * @since  6.6.0
	 */
	private $csvihelper = null;

	/**
	 * Constructor.
	 *
	 * @param   JDatabase       $db          The database class
	 * @param   CsviHelperLog   $log         The CSVI logger
	 * @param   CsviHelperCsvi  $csvihelper  The CSVI helper
	 *
	 * @since   6.6.0
	 */
	public function __construct($db, $log, $csvihelper)
	{
		$this->db = $db;
		$this->log = $log;
		$this->csvihelper = $csvihelper;
	}

	/**
	 * Load a number of maintenance tasks.
	 *
	 * @return  array  List of available operations.
	 *
	 * @since   6.6.0
	 */
	public function getOperations()
	{
		return array('options' => array(
			''                    => JText::_('COM_CSVI_MAKE_CHOICE'),
			'removeproductprices' => JText::_('COM_CSVI_REMOVEPRODUCTPRICES_LABEL'),
			'emptydatabase'       => JText::_('COM_CSVI_EMPTYDATABASE_LABEL'),
			'refreshsefurls'      => JText::_('COM_CSVI_REFRESHSEFURLS_LABEL'),
		)
		);
	}

	/**
	 * Load the options for a selected operation.
	 *
	 * @param   string  $operation  The operation to get the options for
	 *
	 * @return  string  The options for a selected operation.
	 *
	 * @since   6.6.0
	 */
	public function getOptions($operation)
	{
		switch ($operation)
		{
			case 'emptydatabase':
				$layout = new JLayoutFile('csvi.modal');
				$html   = '<span class="help-block">' . JText::_('COM_CSVI_' . $operation . '_DESC') . '</span>';
				$html .= $layout->render(
					array(
						'modal-header'  => JText::_('COM_CSVI_' . $operation . '_LABEL'),
						'modal-body'    => JText::_('COM_CSVI_CONFIRM_DB_DELETE'),
						'cancel-button' => true
					)
				);

				return $html;
				break;
			case 'removeproductprices':
				$helper  = new Com_HikashopHelperCom_Hikashop_Category($this->db);
				$filter  = array($this->db->quoteName('category_name', 'text'), $this->db->quoteName('category_id', 'value'));
				$options = $helper->getChildren('manufacturer', false, array(), '', false, false, false, $filter);
				array_unshift($options, array('text' => JText::_('COM_CSVI_SELECT_BRAND_OPTION'), 'value' => ''));

				return '<div class="control-group ">
							<div class="control-label">
								<label title="" class="hasTooltip" for="jform_title" id="jform_title-lbl" data-original-title=" ' . JText::_('COM_CSVI_SELECT_MANUFACTURER_DESC') . '">
									' . JText::_('COM_CSVI_SELECT_MANUFACTURER_LABEL') . '
								</label>
							</div>
							<div class="controls">
								' . JHtml::_('select.genericlist', $options, 'form[manufacturer]', 'class="input-large"') . '
							</div>
						</div>';
				break;
			case 'refreshsefurls':
				$layoutPath = JPATH_ADMINISTRATOR . '/components/com_csvi/addon/com_csvi/layouts';
				$layout     = new JLayoutFile('maintenance.' . $operation, $layoutPath);

				return $layout->render();
				break;
			default:
				return '<span class="help-block">' . JText::_('COM_CSVI_' . $operation . '_DESC') . '</span>';
				break;
		}
	}

	/**
	 * Remove all product prices.
	 *
	 * @param   JInput $input An instance of JInput.
	 *
	 * @return  bool  Always returns true.
	 *
	 * @since   6.6.0
	 */
	public function removeProductPrices(JInput $input)
	{
		$manufacturer = $input->get('manufacturer');

		if ($manufacturer && $manufacturer !== '')
		{
			$pidArray = array();
			$query    = $this->db->getQuery(true)
				->select($this->db->quoteName('product_id'))
				->from($this->db->quoteName('#__hikashop_product'))
				->where($this->db->quoteName('product_manufacturer_id') . ' = ' . (int) $manufacturer);
			$this->db->setQuery($query);
			$productIds = $this->db->loadRowList();

			foreach ($productIds as $productId)
			{
				$pidArray[] = $productId[0];
			}

			try
			{
				$query->clear()
					->delete($this->db->quoteName('#__hikashop_price'))
					->where($this->db->quoteName('price_product_id') . ' IN (' . implode(', ', $pidArray) . ')');
				$this->db->setQuery($query);
				$this->db->execute();
				$this->log->setLineNumber(1);
				$this->log->addStats('empty', JText::_('COM_CSVI_DEBUG_DELETE_PRODUCT_PRICES_MANUFACTURER'));
			} catch (Exception $e)
			{
				$this->log->addStats('error', $e->getMessage());
			}
		}
		else
		{
			$this->db->truncateTable('#__hikashop_price');
			$this->log->setLineNumber(1);
			$this->log->addStats('empty', JText::_('COM_CSVI_HIKASHOP_PRICES_EMPTIED'));
		}

		return true;
	}

	/**
	 * Threshold available fields for extension
	 *
	 * @return  int Hardcoded available fields
	 *
	 * @since   7.0
	 */
	public function availableFieldsThresholdLimit()
	{
		return 452;
	}

	/**
	 * Empty HikaShop tables.
	 *
	 * @return  bool  Always returns true.
	 *
	 * @since   7.0
	 */
	public function emptyDatabase()
	{
		$linenumber = 1;

		// Dont delete rows with these name keys
		$nameKey = array(
			'root',
			'product',
			'tax',
			'status',
			'created',
			'confirmed',
			'cancelled',
			'refunded',
			'shipped',
			'manufacturer',
			'default_tax'
		);

		// Empty the category table
		$this->log->setLinenumber($linenumber++);
		$query = $this->db->getQuery()
			->clear()
			->delete($this->db->quoteName('#__hikashop_category'))
			->where($this->db->quoteName('category_namekey') . ' NOT IN (' . implode(',', $this->db->quote($nameKey)) . ')');
		$this->db->setQuery($query);
		$this->log->add('Empty category table');

		try
		{
			$this->db->execute();
			$this->log->addStats('empty', 'COM_CSVI_CATEGORY_TABLE_HAS_BEEN_EMPTIED');
		}
		catch (Exception $e)
		{
			$this->log->add('Cannot empty category table. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', 'COM_CSVI_CATEGORY_TABLE_HAS_NOT_BEEN_EMPTIED');
		}

		// Empty the media table
		$this->log->setLinenumber($linenumber++);
		$this->db->truncateTable('#__hikashop_file');
		$this->log->add('Empty media table');

		try
		{
			$this->db->execute();
			$this->log->addStats('empty', 'COM_CSVI_MEDIA_TABLE_HAS_BEEN_EMPTIED');
		}
		catch (Exception $e)
		{
			$this->log->add('Cannot empty media table. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', 'COM_CSVI_MEDIA_TABLE_HAS_NOT_BEEN_EMPTIED');
		}

		// Empty the price table
		$this->log->setLinenumber($linenumber++);
		$this->db->truncateTable('#__hikashop_price');
		$this->log->add('Empty price table');

		try
		{
			$this->db->execute();
			$this->log->addStats('empty', 'COM_CSVI_PRICE_TABLE_HAS_BEEN_EMPTIED');
		}
		catch (Exception $e)
		{
			$this->log->add('Cannot empty price table. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', 'COM_CSVI_PRICE_TABLE_HAS_NOT_BEEN_EMPTIED');
		}

		// Empty the product table
		$this->log->setLinenumber($linenumber++);
		$this->db->truncateTable('#__hikashop_product');
		$this->log->add('Empty product table');

		try
		{
			$this->db->execute();
			$this->log->addStats('empty', 'COM_CSVI_PRODUCT_TABLE_HAS_BEEN_EMPTIED');
		}
		catch (Exception $e)
		{
			$this->log->add('Cannot empty product table. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', 'COM_CSVI_PRODUCT_TABLE_HAS_NOT_BEEN_EMPTIED');
		}

		// Empty the product category table
		$this->log->setLinenumber($linenumber++);
		$this->db->truncateTable('#__hikashop_product_category');
		$this->log->add('Empty product category table');

		try
		{
			$this->db->execute();
			$this->log->addStats('empty', 'COM_CSVI_PRODUCT_CATEGORY_TABLE_HAS_BEEN_EMPTIED');
		}
		catch (Exception $e)
		{
			$this->log->add('Cannot empty product category table. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', 'COM_CSVI_PRODUCT_CATEGORY_TABLE_HAS_NOT_BEEN_EMPTIED');
		}

		// Empty the product related table
		$this->log->setLinenumber($linenumber++);
		$this->db->truncateTable('#__hikashop_product_related');
		$this->log->add('Empty product related table');

		try
		{
			$this->db->execute();
			$this->log->addStats('empty', 'COM_CSVI_PRODUCT_RELATED_TABLE_HAS_BEEN_EMPTIED');
		}
		catch (Exception $e)
		{
			$this->log->add('Cannot empty product related table. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', 'COM_CSVI_PRODUCT_RELATED_TABLE_HAS_NOT_BEEN_EMPTIED');
		}

		// Empty the product variant table
		$this->log->setLinenumber($linenumber++);
		$this->db->truncateTable('#__hikashop_variant');
		$this->log->add('Empty variant table');

		try
		{
			$this->db->execute();
			$this->log->addStats('empty', 'COM_CSVI_VARIANT_TABLE_HAS_BEEN_EMPTIED');
		}
		catch (Exception $e)
		{
			$this->log->add('Cannot empty product variant table. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', 'COM_CSVI_VARIANT_TABLE_HAS_NOT_BEEN_EMPTIED');
		}

		// Empty the waitlist table
		$this->log->setLinenumber($linenumber++);
		$this->db->truncateTable('#__hikashop_waitlist');
		$this->log->add('Empty waitlist table');

		try
		{
			$this->db->execute();
			$this->log->addStats('empty', 'COM_CSVI_WAITLIST_TABLE_HAS_BEEN_EMPTIED');
		}
		catch (Exception $e)
		{
			$this->log->add('Cannot empty waitlist table. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', 'COM_CSVI_WAITLIST_TABLE_HAS_NOT_BEEN_EMPTIED');
		}

		// Empty the cart table
		$this->log->setLinenumber($linenumber++);
		$this->db->truncateTable('#__hikashop_cart');
		$this->log->add('Empty cart table');

		try
		{
			$this->db->execute();
			$this->log->addStats('empty', 'COM_CSVI_CART_TABLE_HAS_BEEN_EMPTIED');
		}
		catch (Exception $e)
		{
			$this->log->add('Cannot empty cart table. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', 'COM_CSVI_CART_TABLE_HAS_NOT_BEEN_EMPTIED');
		}

		// Empty the cart product table
		$this->log->setLinenumber($linenumber++);
		$this->db->truncateTable('#__hikashop_cart_product');
		$this->log->add('Empty cart product table');

		try
		{
			$this->db->execute();
			$this->log->addStats('empty', 'COM_CSVI_CART_PRODUCT_TABLE_HAS_BEEN_EMPTIED');
		}
		catch (Exception $e)
		{
			$this->log->add('Cannot empty cart product table. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', 'COM_CSVI_CART_PRODUCT_TABLE_HAS_NOT_BEEN_EMPTIED');
		}

		// Empty the vote table
		$this->log->setLinenumber($linenumber++);
		$this->db->truncateTable('#__hikashop_vote');
		$this->log->add('Empty vote table');

		try
		{
			$this->db->execute();
			$this->log->addStats('empty', 'COM_CSVI_VOTE_TABLE_HAS_BEEN_EMPTIED');
		}
		catch (Exception $e)
		{
			$this->log->add('Cannot empty vote table. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', 'COM_CSVI_VOTE_TABLE_HAS_NOT_BEEN_EMPTIED');
		}

		// Empty the vote user table
		$this->log->setLinenumber($linenumber++);
		$this->db->truncateTable('#__hikashop_vote_user');
		$this->log->add('Empty vote user table');

		try
		{
			$this->db->execute();
			$this->log->addStats('empty', 'COM_CSVI_VOTE_USER_TABLE_HAS_BEEN_EMPTIED');
		}
		catch (Exception $e)
		{
			$this->log->add('Cannot empty vote user table. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', 'COM_CSVI_VOTE_USER_TABLE_HAS_NOT_BEEN_EMPTIED');
		}

		// Store the log count
		$linenumber--;
		$this->log->setLineNumber($linenumber);

		return true;
	}

	/**
	 * Refresh the SEF URLs.
	 *
	 * @param   JInput  $input  An instance of JInput.
	 *
	 * @return  bool  Always returns true.
	 *
	 * @since   7.1.0
	 */
	public function refreshSefUrls(JInput $input)
	{
		// Load all the needed helpers
		$settings = new CsviHelperSettings($this->db);
		$template = new CsviHelperTemplate($input->getInt('template', 0));
		$sef      = new CsviHelperSef($settings, $template, $this->log);

		// Clean the SEF URL table
		if ($input->get('emptytable', false))
		{
			$this->db->truncateTable('#__csvi_sefurls');
		}

		// Get all the products
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName(array('product_id', 'product_alias')))
			->from($this->db->quoteName('#__hikashop_product'));
		$this->db->setQuery($query);

		$records = $this->db->getIterator();
		$urls    = array();

		// Keep track on the number of records
		$count = 1;

		foreach ($records as $record)
		{
			// Collect URLs
			$urls[] = 'index.php?option=com_hikashop&ctrl=product&task=show&' .
				'cid=' . $record->product_id .
				'&name=' . $record->product_alias .
				'&Itemid=' . $template->get('itemid', 1, 'int');

			// Process batches of 500 records
			if ($count > 500)
			{
				if ($urls)
				{
					$sef->getSefUrls($urls);

					$this->log->setLinenumber($this->log->getLinenumber() + count($urls));

					// Clean up
					$urls = array();
					$count = 0;
				}
			}

			$count++;
		}

		// Process the final batch of URLs
		if ($urls)
		{
			$sef->getSefUrls($urls);

			$this->log->setLinenumber($this->log->getLinenumber() + count($urls));
		}

		return true;
	}

	/**
	 * Update Custom available fields that require extra processing.
	 *
	 * @return  void.
	 *
	 * @since   7.3.0
	 */
	public function customAvailableFields()
	{
		// Update Joomla custom fields
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('name'))
			->from($this->db->quoteName('#__fields'))
			->where($this->db->quoteName('state') . ' = 1')
			->where($this->db->quoteName('context') . ' = ' . $this->db->quote('com_users.user'));
		$this->db->setQuery($query);

		$customFields = $this->db->loadRowList();

		if ($customFields)
		{
			$query->clear()
				->insert($this->db->quoteName('#__csvi_availablefields'))
				->columns($this->db->quoteName(array('csvi_name', 'component_name', 'component_table', 'component', 'action')));

			foreach ($customFields as $cfield)
			{
				$query->values(
					$this->db->quote($cfield[0]) . ',' .
					$this->db->quote($cfield[0]) . ',' .
					$this->db->quote('order') . ',' .
					$this->db->quote('com_hikashop') . ',' .
					$this->db->quote('export')
				);
			}

			$this->db->setQuery($query)->execute();
		}
	}
}
