<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * HikaShop Price table.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class HikashopTablePrice extends CsviTableDefault
{
	/**
	 * Table constructor.
	 *
	 * @param   string     $table   Name of the database table to model.
	 * @param   string     $key     Name of the primary key field in the table.
	 * @param   JDatabase  &$db     Database driver
	 * @param   array      $config  The configuration parameters array
	 *
	 * @since   4.0
	 */
	public function __construct($table, $key, &$db, $config = array())
	{
		parent::__construct('#__hikashop_price', 'price_id', $db, $config);
	}

	/**
	 * Check if a price already exists
	 *
	 * Criteria for an existing price are:
	 * - product id
	 * - price access
	 * - product currency
	 * - price min quantity
	 * If all exists, price will be updated.
	 *
	 * @return  bool  True if price exists | False if price does not exist.
	 *
	 * @since   4.0
	 */
	public function check()
	{
		$query = $this->db->getQuery(true);
		$query->select($this->db->quoteName($this->_tbl_key))
			->from($this->db->quoteName($this->_tbl))
			->where($this->db->quoteName('price_product_id') . ' = ' . (int) $this->get('price_product_id'));

		if ($this->get('price_access', false))
		{
			$query->where($this->db->quoteName('price_access') . ' = ' . $this->db->quote($this->get('price_access')));
		}

		if ($this->get('price_currency_id', false))
		{
			$query->where($this->db->quoteName('price_currency_id') . ' = ' . (int) $this->get('price_currency_id'));
		}

		if ($this->get('price_min_quantity', false))
		{
			$query->where($this->db->quoteName('price_min_quantity') . ' = ' . (int) $this->get('price_min_quantity'));
		}

		$this->db->setQuery($query);
		$this->log->add('Find an existing price');
		$id = $this->db->loadResult();

		if ($id)
		{
			$this->set('price_id', $id);
			$this->load($id);

			return true;
		}
		else
		{
			$this->set('price_id', null);

			return false;
		}
	}

	/**
	 * Reset the primary key.
	 *
	 * @return  bool  Returns true on success.
	 *
	 * @since   6.0
	 */
	public function reset()
	{
		parent::reset();

		// Empty the primary key
		$this->set('price_id', null);

		// Empty the price access level
		$this->set('price_access', null);

		return true;
	}
}
