<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * HikaShop Product table.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class HikashopTableProduct extends CsviTableDefault
{
	/**
	 * The HikaShop helper
	 *
	 * @var    Com_HikashopHelperCom_Hikashop
	 *
	 * @since  7.7.0
	 */
	protected $helper = null;

	/**
	 * Table constructor.
	 *
	 * @param   string    $table  Name of the database table to model.
	 * @param   string    $key    Name of the primary key field in the table.
	 * @param   JDatabase &$db    Database driver
	 * @param   array     $config The configuration parameters array
	 *
	 * @since   4.0
	 */
	public function __construct($table, $key, &$db, $config = array())
	{
		parent::__construct('#__hikashop_product', 'product_id', $db, $config);
	}

	/**
	 * Method to perform sanity checks on the JTable instance properties to ensure they are safe to store in the database.
	 *
	 * Child classes should override this method to make sure the data they are storing in the database is safe and as expected before storage.
	 *
	 * @return  boolean  True if the instance is sane and able to be stored in the database.
	 *
	 * @since   7.1.0
	 */
	public function check()
	{
		// Check if there is a product_type value set
		if (empty($this->product_type))
		{
			$this->set('product_type', 'main');
		}

		if ($this->product_alias)
		{
			$this->verifyAlias();
		}

		return true;
	}

	/**
	 * Create a Product alias.
	 *
	 * @return  void.
	 *
	 * @since   7.7.0
	 */
	public function createProductAlias()
	{
		// Is there a product name
		if (!$this->product_name)
		{
			$jdate               = JFactory::getDate();
			$this->product_alias = $jdate->format("Y-m-d-h-i-s") . mt_rand();
			$this->log->add('Product name is empty so date with random numbers is the new product alias');
		}
		else
		{
			// Create a new product alias
			$this->product_alias = $this->helper->createProductAlias($this->product_name);
		}
	}

	/**
	 * Check if a product alias exists.
	 *
	 * @return  Mixed false if recreate set to no else product alias.
	 *
	 * @since   7.7.0
	 */
	private function verifyAlias()
	{
		// Check if the product alias exists
		$query = $this->db->getQuery(true)
			->select('COUNT(' . $this->db->quoteName($this->_tbl_key) . ')')
			->from($this->_tbl)
			->where($this->db->quoteName('product_alias') . ' = ' . $this->db->quote($this->product_alias))
			->where($this->db->quoteName($this->_tbl_key) . ' != ' . (int) $this->product_id);
		$this->db->setQuery($query);
		$alias = $this->db->loadResult();
		$this->log->add('Check if the product alias already exists');

		if ($alias > 0)
		{
			if ($this->get('product_name'))
			{
				// See how many products we have with the same name
				$query->clear()
					->select(
						'ABS(REPLACE(' . $this->db->quoteName('product_alias') . ', ' . $this->db->quote($this->product_alias) . ', ' . $this->db->quote('') . ')) + 1'
					)
					->from($this->_tbl)
					->where(
						'('
						. $this->db->quoteName('product_alias') . ' = ' . $this->db->quote($this->product_alias)
						. ' OR '
						. $this->db->quoteName('product_alias') . ' REGEXP ' . $this->db->quote('^' . $this->product_alias . '-[[:digit:]]+$')
						. ' AND RIGHT(' . $this->db->quoteName('product_alias') . ', 1) IN (0,1,2,3,4,5,6,7,8,9)'
						. ')'
					)
					->where($this->db->quoteName($this->_tbl_key) . ' != ' . (int) $this->get('product_id'))
					->order('LENGTH(' . $this->db->quoteName('product_alias') . '), ' . $this->db->quoteName('product_alias') . ' ASC');
				$this->db->setQuery($query);
				$items = $this->db->loadColumn();
				$this->log->add('Check how many other products exist with the same alias');

				if ($items)
				{
					$count = array_pop($items);
					$this->product_alias .= '-' . $count;
				}
			}
			else
			{
				$jdate = JFactory::getDate();
				$this->product_alias .= $jdate->format("Y-m-d-h-i-s") . mt_rand();
				$this->log->add('Create new alias using date and random digits', false);
			}
		}
	}

	/**
	 * Reset the primary key.
	 *
	 * @return  bool  Returns true on success.
	 *
	 * @since   6.0
	 */
	public function reset()
	{
		parent::reset();

		// Empty the primary key
		$this->set('product_id', null);

		return true;
	}
}
