<?php
/**
 * @package     CSVI
 * @subpackage  J2Store
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace j2store\com_j2store\model\import;

defined('_JEXEC') or die;

/**
 * Address import.
 *
 * @package     CSVI
 * @subpackage  J2Store
 * @since       7.7.0
 */
class Address extends \RantaiImportEngine
{
	/**
	 * CSVI fields
	 *
	 * @var    \CsviHelperImportFields
	 * @since  7.7.0
	 */
	protected $fields;

	/**
	 * The addon helper
	 *
	 * @var    \Com_J2StoreHelperCom_J2Store
	 * @since  7.7.0
	 */
	protected $helper;

	/**
	 * Product price table
	 *
	 * @var    \J2StoreTableProductPrice
	 * @since  7.7.0
	 */
	private $addressTable;

	/**
	 * Start the menu import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   7.7.0
	 *
	 * @throws  \RuntimeException
	 * @throws  \InvalidArgumentException
	 * @throws  \UnexpectedValueException
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		$email = $this->getState('email', false);
		$type  = $this->getState('type', false);

		if (!$email || !$type)
		{
			$this->loaded = false;
			$this->log->addStats('skipped', 'COM_CSVI_NO_EMAIL_OR_TYPE_FOUND');
		}
		else
		{
			$this->loaded = true;

			if (!$this->getState('j2store_address_id', false))
			{
				$this->setState('j2store_address_id', $this->helper->getAddressId('email', $email, strtolower($type)));
			}

			if ($this->addressTable->load($this->getState('j2store_address_id', 0)))
			{
				if (!$this->template->get('overwrite_existing_data'))
				{
					$this->log->add(\JText::sprintf('COM_FIELDS_WARNING_OVERWRITING_SET_TO_NO', $this->getState('email')), false);
					$this->loaded = false;
				}
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no path or menu ID can be found.
	 *
	 * @since   7.7.0
	 *
	 * @throws  \RuntimeException
	 * @throws  \InvalidArgumentException
	 * @throws  \UnexpectedValueException
	 */
	public function getProcessRecord()
	{
		if (!$this->loaded)
		{
			return false;
		}

		if (!$this->getState('j2store_address_id', false) && $this->template->get('ignore_non_exist'))
		{
			$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('email', '')));
		}
		else
		{
			$countryField = 'country_name';

			if (!$this->getState('country_id', false) && $this->getState('country_name', false))
			{
				$countryField = 'country_name';
			}

			if (!$this->getState('country_id', false) && $this->getState('country_isocode_2', false))
			{
				$countryField = 'country_isocode_2';
			}

			if (!$this->getState('country_id', false) && $this->getState('country_isocode_3', false))
			{
				$countryField = 'country_isocode_3';
			}

			$this->setState('country_id', $this->helper->getCountryId($this->getState($countryField, false), $countryField));

			if ($this->getState('country_id', false) && ($this->getState('zone_code', false) || $this->getState('zone_name', false)))
			{
				$this->setState(
					'zone_id', $this->helper->getCountryZoneId(
					$this->getState('country_id', false),
					$this->getState('zone_code', false), $this->getState('zone_name', false)
				)
				);
			}

			// Check if there is a Joomla user with given email
			if ($this->getState('email', false))
			{
				$query = $this->db->getQuery(true)
					->select($this->db->quoteName('id'))
					->from($this->db->quoteName('#__users'))
					->where($this->db->quoteName('email') . ' = ' . $this->db->quote($this->getState('email')));
				$this->db->setQuery($query);
				$this->log->add('Find the user ID from Joomla');
				$this->setState('user_id', $this->db->loadResult());

				if (!$this->db->loadResult())
				{
					$this->log->add('Joomla user with email ' . $this->getState('email') . ' not found');
					$this->log->AddStats('skipped', \JText::sprintf('COM_CSVI_NO_JOOMLA_USER_FOUND', $this->getState('email')));

					return false;
				}
			}

			$this->addressTable->bind($this->state);

			$this->addressTable->check();

			if (strtoupper($this->getState('address_delete')) === 'Y')
			{
				if ($this->getState('j2store_address_id', false))
				{
					$this->addressTable->delete($this->getState('j2store_address_id', false));
				}
				else
				{
					$this->log->addStats('incorrect', 'COM_CSVI_ADDRESS_NOT_DELETED_NO_ID');
				}
			}
			else
			{
				try
				{
					$this->addressTable->store();
					$this->log->add('Address Processed successfully', false);
				}
				catch (\Exception $e)
				{
					$this->log->add('Cannot add address. Error: ' . $e->getMessage(), false);
					$this->log->addStats('incorrect', $e->getMessage());

					return false;
				}
			}
		}

		return true;
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @throws  \Exception
	 *
	 * @since   7.7.0
	 */
	public function loadTables()
	{
		$this->addressTable = $this->getTable('Address');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   7.7.0
	 */
	public function clearTables()
	{
		$this->addressTable->reset();
	}
}
