<?php
/**
 * @package     CSVI
 * @subpackage  K2
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace k2\com_k2\model\export;

defined('_JEXEC') or die;

/**
 * Export K2 extra fields.
 *
 * @package     CSVI
 * @subpackage  K2
 * @since       6.0
 */
class Extrafield extends \CsviModelExports
{
	/**
	 * Export the data.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Build something fancy to only get the fieldnames the user wants
			$userfields = array();
			$exportfields = $this->fields->getFields();

			// Group by fields
			$groupbyfields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupby = array();

			if (isset($groupbyfields->name))
			{
				$groupbyfields = array_flip($groupbyfields->name);
			}
			else
			{
				$groupbyfields = array();
			}

			// Sort selected fields
			$sortfields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortby = array();

			if (isset($sortfields->name))
			{
				$sortbyfields = array_flip($sortfields->name);
			}
			else
			{
				$sortbyfields = array();
			}

			foreach ($exportfields as $field)
			{
				switch ($field->field_name)
				{
					case 'option_name':
					case 'option_value':
					case 'option_target':
					case 'option_editor':
					case 'option_rows':
					case 'option_cols':
					case 'alias':
					case 'required':
					case 'showNull':
					case 'displayInFrontEnd':
						$userfields[] = $this->db->quoteName('value');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('value');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('value');
						}
						break;
					case 'group_name':
						$userfields[] = $this->db->quoteName('group');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('group');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('group');
						}
						break;
					case 'custom':
						break;
					default:
						$userfields[] = $this->db->quoteName($field->field_name);

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName($field->field_name);
						}
						break;
				}
			}

			// Build the query
			$userfields = array_unique($userfields);
			$query = $this->db->getQuery(true);
			$query->select(implode(",\n", $userfields));
			$query->from($this->db->quoteName('#__k2_extra_fields', 'e'));

			// Filter by published state
			$publish_state = $this->template->get('publish_state');

			if ($publish_state != '' && ($publish_state == 1 || $publish_state == 0))
			{
				$query->where($this->db->quoteName('e.published') . ' = ' . (int) $publish_state);
			}

			// Filter by group
			$groups = $this->template->get('extrafield_group');

			if ($groups && $groups[0] != '*')
			{
				$query->where($this->db->quoteName('group') . " IN ('" . implode("','", $groups) . "')");
			}

			// Group the fields
			$groupby = array_unique($groupby);

			if (!empty($groupby))
			{
				$query->group($groupby);
			}

			// Sort set fields
			$sortby = array_unique($sortby);

			if (!empty($sortby))
			{
				$query->order($sortby);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->db->getNumRows();

			if ($logcount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					foreach ($exportfields as $field)
					{
						$fieldname = $field->field_name;

						// Set the field value
						if (isset($record->$fieldname))
						{
							$fieldvalue = $record->$fieldname;
						}
						else
						{
							$fieldvalue = '';
						}

						// Process the field
						switch ($fieldname)
						{
							case 'option_name':
							case 'option_value':
							case 'option_target':
							case 'option_editor':
							case 'option_rows':
							case 'option_cols':
							case 'alias':
							case 'required':
							case 'showNull':
							case 'displayInFrontEnd':
								$extra = json_decode($record->value);
								$name = str_ireplace('option_', '', $fieldname);
								$fieldvalue = '';

								switch ($record->type)
								{
									case 'multipleSelect':
									case 'radio':
										if (in_array($fieldname, array('option_name', 'option_value')))
										{
											// Multiple selects are | separated
											$values = array();

											foreach ($extra as $extra_value)
											{
												if (isset($extra_value->$name))
												{
													$values[] = $extra_value->$name;
												}
											}

											$fieldvalue = implode('|', $values);
										}
										else
										{
											if (isset($extra[0]->$name))
											{
												$fieldvalue = $extra[0]->$name;
											}
										}
										break;
									case 'link':
										if (isset($extra[0]->$name))
										{
											$fieldvalue = $extra[0]->$name;
										}
										break;
									default:
										if (isset($extra[0]->$name))
										{
											$fieldvalue = $extra[0]->$name;
										}
										break;
								}
								break;
							case 'group_name':
								$query = $this->db->getQuery(true);
								$query->select('name');
								$query->from($this->db->quoteName('#__k2_extra_fields_groups'));
								$query->where($this->db->quoteName('id') . ' = ' . (int) $record->group);
								$this->db->setQuery($query);
								$fieldvalue = $this->db->loadResult();
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldvalue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}
}
