<?php
/**
 * @package     CSVI
 * @subpackage  K2
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace k2\com_k2\model\export;

defined('_JEXEC') or die;

/**
 * Export K2 items.
 *
 * @package     CSVI
 * @subpackage  K2
 * @since       6.0
 */
class Item extends \CsviModelExports
{
	/**
	 * Array of extra custom fields that can be used as available field.
	 *
	 * @var    array
	 * @since  6.0
	 */
	private $customfieldsExport = array();

	/**
	 * Array of plugin fields that can be used as available field.
	 *
	 * @var    array
	 * @since  6.0
	 */
	private $pluginfieldsExport = array();

	/**
	 * The domain name for URLs.
	 *
	 * @var    string
	 * @since  6.0
	 */
	private $domainname = null;

	/**
	 * The addon helper
	 *
	 * @var    \Com_K2HelperCom_K2
	 * @since  6.0
	 */
	protected $helper = null;

	/**
	 * Export the data.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Get some basic data
			$this->domainname = $this->settings->get('hostname');
			$this->loadExtraFields();
			$this->loadPluginFields();
			$userfields = array();
			$exportfields = $this->fields->getFields();

			// Group by fields
			$groupbyfields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupby = array();

			if (isset($groupbyfields->name))
			{
				$groupbyfields = array_flip($groupbyfields->name);
			}
			else
			{
				$groupbyfields = array();
			}

			// Sort selected fields
			$sortfields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortby = array();

			if (isset($sortfields->name))
			{
				$sortbyfields = array_flip($sortfields->name);
			}
			else
			{
				$sortbyfields = array();
			}

			foreach ($exportfields as $field)
			{
				switch ($field->field_name)
				{
					case 'category_path':
						$userfields[] = $this->db->quoteName('i.catid');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName($field->field_name);
						}
						break;
					case 'image':
					case 'image_url':
					case 'image_url_thumb':
						$userfields[] = "MD5(CONCAT('Image', " . $this->db->quoteName('id') . ")) AS " . $this->db->quoteName('image');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName($field->field_name);
						}
						break;
					case 'item_url':
					case 'tags':
						$userfields[] = $this->db->quoteName('i.id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName($field->field_name);
						}
						break;
					case 'metadata_author':
					case 'metadata_robot':
						$userfields[] = $this->db->quoteName('i.metadata');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName($field->field_name);
						}
						break;
					case 'custom':
						break;
					default:
						if (in_array($field->field_name, $this->customfieldsExport))
						{
							$userfields[] = $this->db->quoteName('extra_fields');

							if (array_key_exists($field->field_name, $groupbyfields))
							{
								$groupby[] = $this->db->quoteName('extra_fields');
							}

							if (array_key_exists($field->field_name, $sortbyfields))
							{
								$sortby[] = $this->db->quoteName('extra_fields');
							}
						}
						elseif (in_array($field->field_name, $this->pluginfieldsExport))
						{
							$userfields[] = $this->db->quoteName('i.id');
							$userfields[] = $this->db->quoteName('i.plugins');

							if (array_key_exists($field->field_name, $groupbyfields))
							{
								$groupby[] = $this->db->quoteName('i.plugins');
							}

							if (array_key_exists($field->field_name, $sortbyfields))
							{
								$sortby[] = $this->db->quoteName('i.plugins');
							}
						}
						else
						{
							$userfields[] = $this->db->quoteName($field->field_name);

							if (array_key_exists($field->field_name, $groupbyfields))
							{
								$groupby[] = $this->db->quoteName($field->field_name);
							}

							if (array_key_exists($field->field_name, $sortbyfields))
							{
								$sortby[] = $this->db->quoteName($field->field_name);
							}
						}
						break;
				}
			}

			// Build the query
			$userfields = array_unique($userfields);
			$query = $this->db->getQuery(true);
			$query->select(implode(",\n", $userfields));
			$query->from($this->db->quoteName('#__k2_items', 'i'));

			// Filter by published state
			$publish_state = $this->template->get('publish_state');

			if ($publish_state != '' && ($publish_state == 1 || $publish_state == 0))
			{
				$query->where($this->db->quoteName('i.published') . ' = ' . (int) $publish_state);
			}

			// Filter by language
			$language = $this->template->get('language');

			if ($language != '*')
			{
				$query->where($this->db->quoteName('i.language') . ' = ' . $this->db->quote($language));
			}

			// Filter by category
			$categories = $this->template->get('item_categories');

			if ($categories && $categories[0] != '')
			{
				$query->where($this->db->quoteName('catid') . " IN ('" . implode("','", $categories) . "')");
			}

			// Filter by tags
			$tags = $this->template->get('item_tags');

			if ($tags && $tags[0] != '')
			{
				$query->innerJoin(
					$this->db->quoteName('#__k2_tags_xref', 'tx')
					. ' ON ' . $this->db->quoteName('tx.itemID') . ' = ' . $this->db->quoteName('i.id') . ' AND ' .
					$this->db->quoteName('tx.tagID') . ' IN (' . implode(",", $tags) . ')'
				);
			}

			// Filter by author
			$authors = $this->template->get('item_authors');

			if ($authors && $authors[0] != '')
			{
				$query->where($this->db->quoteName('i.created_by') . " IN ('" . implode("','", $authors) . "')");
			}

			// Filter by created start date
			$createdDateStart = $this->template->get('item_createdstart', false);

			if ($createdDateStart)
			{
				$createdDate = \JFactory::getDate($createdDateStart);
				$query->where($this->db->quoteName('i.created') . ' >= ' . $this->db->quote($createdDate->toSql()));
			}
			// Filter by created end date
			$createdDateEnd = $this->template->get('item_createdend', false);

			if ($createdDateEnd)
			{
				$createdDate = \JFactory::getDate($createdDateEnd);
				$query->where($this->db->quoteName('i.created') . ' <= ' . $this->db->quote($createdDate->toSql()));
			}
			// Filter by modified start date
			$modifiedDateStart = $this->template->get('item_modifiedstart', false);

			if ($modifiedDateStart)
			{
				$modifiedDate = \JFactory::getDate($modifiedDateStart);
				$query->where($this->db->quoteName('i.modified') . ' >= ' . $this->db->quote($modifiedDate->toSql()));
			}
			// Filter by modified end date
			$modifiedDateEnd = $this->template->get('item_modifiedend', false);

			if ($modifiedDateEnd)
			{
				$modifiedDate = \JFactory::getDate($modifiedDateEnd);
				$query->where($this->db->quoteName('i.modified') . ' <= ' . $this->db->quote($modifiedDate->toSql()));
			}

			// Group the fields
			$groupby = array_unique($groupby);

			if (!empty($groupby))
			{
				$query->group($groupby);
			}

			// Sort set fields
			$sortby = array_unique($sortby);

			if (!empty($sortby))
			{
				$query->order($sortby);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->db->getNumRows();

			if ($logcount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					foreach ($exportfields as $field)
					{
						$fieldname = $field->field_name;

						// Set the field value
						if (isset($record->$fieldname))
						{
							$fieldvalue = $record->$fieldname;
						}
						else
						{
							$fieldvalue = '';
						}

						// Process the field
						switch ($fieldname)
						{
							case 'category_path':
								$fieldvalue = trim($this->helper->createCategoryPathById($record->catid));
								break;
							case 'item_url':
								// Let's create a SEF URL
								$fieldvalue = $this->sef->getSefUrl('index.php?option=com_k2&view=item&layout=item&id=' . $record->id);
								break;
							case 'image':
								// Check if there is already a product full image
								$fieldvalue = $record->image . '.jpg';
								break;
							case 'image_url':
								// Check if there is already a product full image
								$fieldvalue = $this->domainname . '/media/k2/items/cache/' . $record->image . '_' . $this->template->get('image_url', 'Generic') . '.jpg';
								break;
							case 'image_url_thumb':
								// Check if there is already a product full image
								$fieldvalue = $this->domainname . '/media/k2/items/cache/' . $record->image . '_' . $this->template->get('image_url_thumb', 'S') . '.jpg';
								break;
							case 'metadata_author':
							case 'metadata_robot':
								$metadata = explode("\n", $record->metadata);

								foreach ($metadata as $metafield)
								{
									list($name, $value) = explode('=', $metafield);

									switch ($fieldname)
									{
										case 'metadata_robot':
											if ($name == 'robots')
											{
												$fieldvalue = $value;
											}
											break;
										case 'metadata_author':
											if ($name == 'author')
											{
												$fieldvalue = $value;
											}
											break;
									}
								}
								break;
							case 'tags':
								$query = $this->db->getQuery(true)
									->select($this->db->quoteName('t.name'))
									->from($this->db->quoteName('#__k2_tags_xref', 'x'))
									->leftJoin(
										$this->db->quoteName('#__k2_tags', 't')
										. ' ON ' . $this->db->quoteName('t.id') . ' = ' . $this->db->quoteName('x.tagID')
									)
									->where($this->db->quoteName('itemID') . ' = ' . (int) $record->id);
								$this->db->setQuery($query);
								$fieldvalue = implode('|', $this->db->loadColumn());
								break;
							default:
								if (in_array($fieldname, $this->customfieldsExport))
								{
									// Get the extra fields values
									$record_extrafields = json_decode($record->extra_fields);

									if (is_array($record_extrafields))
									{
										// Extract the title from the field
										$separatorPosition = stripos($fieldname, '_');
										$title = substr($fieldname, $separatorPosition + 1);

										$query = $this->db->getQuery(true)
											->select($this->db->quoteName(array('id', 'type', 'value')))
											->from($this->db->quoteName('#__k2_extra_fields'))
											->where($this->db->quoteName('name') . ' = ' . $this->db->quote($title));
										$this->db->setQuery($query);
										$extra = $this->db->loadObject();
										$extra->value = json_decode($extra->value);

										if (is_object($extra))
										{
											// Check if this item has this field
											foreach ($record_extrafields as $extrafield)
											{
												if ($extrafield->id == $extra->id)
												{
													switch ($extra->type)
													{
														case 'multipleSelect':
															// Multiple selects are | separated
															$values = array();

															foreach ($extra->value as $extra_value)
															{
																if (in_array($extra_value->value, $extrafield->value))
																{
																	$values[] = $extra_value->name;
																}
															}

															$fieldvalue = implode('|', $values);
															break;
														case 'radio':
														case 'select':
															foreach ($extra->value as $extra_value)
															{
																if ($extra_value->value == $extrafield->value)
																{
																	$fieldvalue = $extra_value->name;
																}
															}
															break;
														case 'link':
															$fieldvalue = implode('|', $extrafield->value);
															break;
														default:
															$fieldvalue = $extrafield->value;
															break;
													}
												}
											}
										}
									}
								}
								elseif (in_array($fieldname, $this->pluginfieldsExport))
								{
									// Get value from K2 plugin
									$dispatcher = new \RantaiPluginDispatcher;
									$dispatcher->importPlugins('csviext', $this->db);
									$result = $dispatcher->trigger(
										'onExportContent',
										array(
											'extension' => 'k2',
											'id' => $record->id,
											'fieldname' => $fieldname,
											'operation' => 'item',
											'log' => $this->log
										)
									);

									$fieldvalue = $result[0];
								}
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldvalue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}

	/**
	 * Get a list of extra fields that can be used as available field.
	 *
	 * @return  void.
	 *
	 * @since   4.4.1
	 */
	private function loadExtraFields()
	{
		$query = $this->db->getQuery(true)
			->select('TRIM(
						CONCAT(' .
				$this->db->quoteName('g.name') . ', ' . $this->db->quote('_') . ', ' . $this->db->quoteName('x.name') .
				') ' .
				') AS ' . $this->db->quoteName('title')
			)
			->from($this->db->quoteName('#__k2_extra_fields', 'x'))
			->leftJoin(
				$this->db->quoteName('#__k2_extra_fields_groups', 'g')
				. ' ON ' . $this->db->quoteName('g.id') . ' = ' . $this->db->quoteName('x.group')
			);
		$this->db->setQuery($query);
		$result = $this->db->loadColumn();

		if (!is_array($result))
		{
			$result = array();
		}

		$this->customfieldsExport = $result;
	}

	/**
	 * Get a list of plugin fields that can be used as available field.
	 *
	 * @return  void.
	 *
	 * @since   6.5.0
	 */
	private function loadPluginFields()
	{
		$dispatcher = new \RantaiPluginDispatcher;
		$dispatcher->importPlugins('csviext', $this->db);
		$result = $dispatcher->trigger(
			'getAttributes',
			array(
				'extension' => 'k2',
				'operation' => 'item',
				'log' => $this->log
			)
		);

		if (is_array($result) && !empty($result))
		{
			$this->pluginfieldsExport = array_merge($this->pluginfieldsExport, $result[0]);
		}
	}
}
