<?php
/**
 * @package     CSVI
 * @subpackage  K2
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace k2\com_k2\model\import;

defined('_JEXEC') or die;

/**
 * Category import.
 *
 * @package     CSVI
 * @subpackage  K2
 * @since       6.0
 */
class Category extends \RantaiImportEngine
{
	/**
	 * Category table.
	 *
	 * @var    \K2TableCategory
	 * @since  6.0
	 */
	private $category;

	/**
	 * Category separator.
	 *
	 * @var    string
	 * @since  6.0
	 */
	private $categorySeparator;

	/**
	 * The K2 addon helper
	 *
	 * @var    \Com_K2HelperCom_K2
	 * @since  6.0
	 */
	protected $helper;

	/**
	 * Start the product import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   6.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'published':
						switch (strtoupper($value))
						{
							case 'Y':
							case 'y':
							case '1':
								$value = 'Y';
								break;
							default:
								$value = 'N';
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		// There must be an alias and catid or category_path
		if ($this->getState('alias', false) || $this->getState('category_path', false))
		{
			// Load the current coupon
			$this->loaded = true;

			if (!$this->getState('id', false))
			{
				$this->setState('id', $this->helper->getCategoryIdByPath($this->getState('category_path'), $this->getState('language', '*')));
			}

			// Load the current content data
			if ($this->category->load($this->getState('id')) && (!$this->template->get('overwrite_existing_data')))
			{
				$this->log->add(\JText::sprintf('COM_CSVI_DATA_EXISTS_PRODUCT_SKU', $this->getState('category_path', '')));
				$this->loaded = false;
			}
		}
		else
		{
			// We must have the required fields otherwise category cannot be created
			$this->loaded = false;

			$this->log->addStats('skipped', \JText::_('COM_CSVI_MISSING_REQUIRED_FIELDS'));
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   6.0
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			if ($this->getState('category_delete') === 'Y')
			{
				$this->deleteCategory();
			}
			elseif (!$this->getState('id', false) && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new products when user chooses to ignore new products
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('category_path', '')));
			}
			else
			{
				// Set the plugin attributes
				$this->setPlugins();

				if ($this->getState('id', false))
				{
					$params = json_decode($this->category->params);

					// Check if we need to set the inheritFrom parameter
					$inheritFrom = $this->getState('inheritFrom', false);

					if ($inheritFrom)
					{
						$params->inheritFrom = $this->helper->getCategoryIdByPath($inheritFrom);
					}

					// Check if we need to set the theme parameter
					$theme = $this->getState('theme', false);

					if ($theme)
					{
						$params->theme = $theme;
					}

					$this->setState('params', json_encode($params));

					// Load the category separator
					if (null === $this->categorySeparator)
					{
						$this->categorySeparator = $this->template->get('category_separator', '/');
					}

					// Divide the path into parts
					$parts = explode($this->categorySeparator, $this->getState('category_path'));

					if (empty($this->category->name))
					{
						$this->category->name = end($parts);
					}

					// Get the parent ID if needed
					if (!$this->getState('parent', false))
					{
						if (count($parts) > 1)
						{
							// Remove the last category entry
							array_pop($parts);

							// Get the parent path
							$parentpath = implode($this->categorySeparator, $parts);

							// Find the parent ID
							$this->setState('parent', $this->helper->getCategoryIdByPath($parentpath));
						}
						else
						{
							$this->setState('parent', 0);
						}
					}

					// Now store the data
					if ($this->category->save($this->state))
					{
						// Add the image if needed
						if ($this->getState('image', false))
						{
							$this->processMedia();
						}
					}
					else
					{
						return false;
					}
				}
				else
				{
					$this->log->addStats('notice', \JText::sprintf('COM_CSVI_CATEGORY_DOESNT_EXISTS', $this->getState('category_path')));
				}
			}

			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function loadTables()
	{
		$this->category = $this->getTable('Category');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function clearTables()
	{
		$this->category->reset();
	}

	/**
	 * Delete a category and its references.
	 *
	 * @return  void.
	 *
	 * @since   4.0
	 */
	private function deleteCategory()
	{
		if ($this->category->delete($this->getState('id')))
		{
			$this->log->addStats('deleted', 'COM_CSVI_CATEGORY_DELETED');
		}
		else
		{
			$this->log->addStats('incorrect', 'COM_CSVI_CATEGORY_NOT_DELETED');
		}
	}

	/**
	 * Process media files.
	 *
	 * @return  void.
	 *
	 * @since   4.0
	 */
	private function processMedia()
	{
		// Check if any image handling needs to be done
		if ($this->template->get('process_image', false))
		{
			// Get the full path and name of the image
			$filename = $this->category->get('id');
			$image = JPATH_SITE . '/media/k2/categories/' . $filename;

			// Check if the image exists, either by its upload name or new name
			if (\JFile::exists(JPATH_SITE . '/media/k2/categories/' . trim($this->getState('image'))))
			{
				// Rename original image
				\JFile::move(JPATH_SITE . '/media/k2/categories/' . trim($this->getState('image')), $image . '.jpg');
			}

			if (\JFile::exists($image . '.jpg'))
			{
				// Load some helpers
				require_once JPATH_ADMINISTRATOR . '/components/com_k2/lib/class.upload.php';

				// Set the maximum sizes
				$max_width = $this->template->get('resize_max_width', 1024);
				$max_height = $this->template->get('resize_max_height', 768);

				// Check if the thumbnail is not too big
				$thumb_sizes = getimagesize($image . '.jpg');

				if ($thumb_sizes[0] < $max_width || $thumb_sizes[1] < $max_height)
				{
					/** @var Upload $imagehelper */
					$imagehelper = new \Upload($image . '.jpg');

					$savepath = JPATH_SITE . '/media/k2/categories/';

					$imagehelper->image_resize = true;
					$imagehelper->image_ratio_y = true;
					$imagehelper->image_convert = 'jpg';
					$imagehelper->jpeg_quality = $this->helperconfig->get('imagesQuality', 85);
					$imagehelper->file_auto_rename = false;
					$imagehelper->file_overwrite = true;
					$imagehelper->file_new_name_body = $filename;
					$imagehelper->image_x = $this->helperconfig->get('catImageWidth', '100');
					$imagehelper->Process($savepath);
				}

				// Store the new image name with the category
				$this->category->set('image', $filename . '.jpg');
				$this->category->store();
			}
		}
	}

	/**
	 * Set the plugins field.
	 *
	 * @return  void.
	 *
	 * @since   6.5.0
	 */
	public function setPlugins()
	{
		// Get Value from content plugin
		$dispatcher = new \RantaiPluginDispatcher;
		$dispatcher->importPlugins('csviext', $this->db);

		// Fire the plugin to get plugin field to import
		$pluginFields = $dispatcher->trigger(
			'getAttributes',
			array(
				'extension' => 'k2',
				'operation' => 'category',
				'log'    => $this->log
			)
		);

		if (!empty($pluginFields[0]))
		{
			// Load the current plugins
			$plugins = json_decode($this->category->get('plugins'));

			if (!is_object($plugins))
			{
				$plugins = new \stdClass;
			}

			foreach ($pluginFields[0] as $plugin)
			{
				$plugins->$plugin = '';

				if ($this->getState($plugin, false))
				{
					$plugins->$plugin = $this->getState($plugin, '');
				}
			}

			// Store the new attributes
			$this->setState('plugins', json_encode($plugins));
			$this->log->add('Plugin field set for K2 swmap plugin', false);
		}
	}
}
