<?php
/**
 * @package     CSVI
 * @subpackage  K2
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace k2\com_k2\model\import;

defined('_JEXEC') or die;

/**
 * Extra field import.
 *
 * @package     CSVI
 * @subpackage  K2
 * @since       6.0
 */
class Extrafield extends \RantaiImportEngine
{
	/**
	 * Extra field table.
	 *
	 * @var    \K2TableExtrafield
	 * @since  6.0
	 */
	private $extrafield = null;

	/**
	 * Start the product import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   6.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'published':
						switch (strtoupper($value))
						{
							case 'Y':
							case 'y':
								$value = 1;
								break;
							case 'N':
								$value = 0;
								break;
						}

						$this->setState($name, $value);
						break;
					case 'group_name':
						$this->setState('group', $this->getGroupByName($value));
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		// We must have a name
		if ($this->getState('name', false))
		{
			// Load the current coupon
			$this->loaded = true;

			if (!$this->getState('id', false))
			{
				$id = $this->helper->getExtrafieldId();

				if (is_int($id))
				{
					$this->setState('id', $id);

					// Load the current content data
					if ($this->extrafield->load($id))
					{
						if (!$this->template->get('overwrite_existing_data'))
						{
							$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_EXTRAFIELD', $this->getState('name', '')));
							$this->loaded = false;
						}
					}
				}
				else
				{
					$this->setState('id', false);
				}
			}
		}
		else
		{
			// We must have the required fields otherwise category cannot be created
			$this->loaded = false;

			$this->log->addStats('skipped', \JText::_('COM_CSVI_MISSING_REQUIRED_FIELDS_EXTRAFIELD'));
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   6.0
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			if (!$this->getState('id', false) && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new extra fields when user chooses to ignore new extra fields
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW_EXTRAFIELD', $this->getState('name', '')));
			}
			else
			{
				// Prepare the value field, this is an array
				if (!$this->getState('value', false))
				{
					// We create the array
					$value                      = array();
					$value['name']              = $this->getState('option_name', '');
					$value['value']             = $this->getState('option_value');
					$value['target']            = $this->getState('option_target', '');
					$value['editor']            = $this->getState('option_editor', '');
					$value['rows']              = $this->getState('option_rows', '');
					$value['cols']              = $this->getState('option_cols', '');
					$value['alias']             = $this->getState('alias', '');
					$value['required']          = $this->getState('required', '');
					$value['showNull']          = $this->getState('showNull', '');
					$value['displayInFrontEnd'] = $this->getState('displayInFrontEnd', '');
					$values                     = array($value);
					$this->setState('value', json_encode($values));
				}

				// Bind the data
				if ($this->extrafield->save($this->state))
				{
					if ($this->queryResult() == 'UPDATE')
					{
						$this->log->addStats('updated', 'COM_CSVI_UPDATE_ITEM');
					}
					else
					{
						$this->log->addStats('added', 'COM_CSVI_ADD_ITEM');
					}

					// Store the debug message
					$this->log->add('COM_CSVI_ITEM_QUERY', true);
				}
				else
				{
					$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_EXTRAFIELD_NOT_ADDED', $this->extrafield->getError()));

					// Store the debug message
					$this->log->add('COM_CSVI_EXTRAFIELD_QUERY', true);

					return false;
				}
			}

			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function loadTables()
	{
		$this->extrafield = $this->getTable('Extrafield');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function clearTables()
	{
		$this->extrafield->reset();
	}

	/**
	 * Find the group ID by name.
	 *
	 * @param   string  $name  The name of the group to find the ID for.
	 *
	 * @return  int  The ID of the group name.
	 *
	 * @since   5.11
	 */
	private function getGroupByName($name)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('id'))
			->from($this->db->quoteName('#__k2_extra_fields_groups'))
			->where($this->db->quoteName('name') . ' = ' . $this->db->quote($name));
		$this->db->setQuery($query);
		$group_id = $this->db->loadResult();

		$this->log->add('Find group ID', true);
		$this->log->add('Group ID found: ' . $group_id);

		return $group_id;
	}
}
