<?php
/**
 * @package     CSVI
 * @subpackage  K2
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * K2 maintenance.
 *
 * @package     CSVI
 * @subpackage  K2
 * @since       6.5.0
 */
class Com_K2Maintenance
{
	/**
	 * Database connector
	 *
	 * @var    JDatabaseDriver
	 * @since  6.5.0
	 */
	private $db = null;

	/**
	 * Logger helper
	 *
	 * @var    CsviHelperLog
	 * @since  6.5.0
	 */
	private $log = null;

	/**
	 * CSVI Helper.
	 *
	 * @var    CsviHelperCsvi
	 * @since  6.5.0
	 */
	private $csvihelper = null;

	/**
	 * Constructor.
	 *
	 * @param   JDatabaseDriver  $db          The database class
	 * @param   CsviHelperLog    $log         The CSVI logger
	 * @param   CsviHelperCsvi   $csvihelper  The CSVI helper
	 * @param   bool             $isCli       Set if we are running CLI mode
	 *
	 * @since   6.5.0
	 */
	public function __construct(JDatabaseDriver $db, CsviHelperLog $log, CsviHelperCsvi $csvihelper, $isCli = false)
	{
		$this->db         = $db;
		$this->log        = $log;
		$this->csvihelper = $csvihelper;
		$this->isCli      = $isCli;
	}

	/**
	 * Load a number of maintenance tasks.
	 *
	 * @return  array  List of available operations.
	 *
	 * @since   7.1.0
	 */
	public function getOperations()
	{
		return array('options' => array(
			''               => JText::_('COM_CSVI_MAKE_CHOICE'),
			'refreshsefurls' => JText::_('COM_CSVI_REFRESHSEFURLS_LABEL'),
		)
		);
	}

	/**
	 * Load the options for a selected operation.
	 *
	 * @param   string  $operation  The operation to get the options for
	 *
	 * @return  string  The options for a selected operation.
	 *
	 * @since   7.1.0
	 */
	public function getOptions($operation)
	{
		switch ($operation)
		{
			default:
				return '<span class="help-block">' . JText::_('COM_CSVI_' . $operation . '_DESC') . '</span>';
				break;
		}
	}

	/**
	 * Update Custom available fields that require extra processing.
	 *
	 * @return  void.
	 *
	 * @since   6.5.0
	 *
	 * @throws  \RuntimeException
	 */
	public function customAvailableFields()
	{
		// Check if swmap plugin is installed so fields are inserted for item and category
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('extension_id'))
			->from($this->db->quoteName('#__extensions'))
			->where($this->db->quoteName('name') . ' = ' . $this->db->quote('plg_content_k2swmaps'))
			->where($this->db->quoteName('type') . ' = ' . $this->db->quote('plugin'))
			->where($this->db->quoteName('folder') . ' = ' . $this->db->quote('content'));
		$this->db->setQuery($query);

		$extension_id = $this->db->loadResult();

		// Insert fields only when the plugin is installed
		if ($extension_id)
		{
			// Start the query
			$query->clear()
				->insert($this->db->quoteName('#__csvi_availablefields'))
				->columns($this->db->quoteName(array('csvi_name', 'component_name', 'component_table', 'component', 'action')));

			// Insert item fields
			$itemfields = array
						(
							'address',
							'latitude',
							'lognitude',
							'type',
							'icon',
							'theicon',
							'zoom',
							'panoramioitem',
							'streetviewitem',
							'streetViewHeading',
							'streetViewPitch',
							'adsenseitem',
							'publisher',
							'formatitem',
							'positionitem',
							'customOverlay'
						);

			foreach ($itemfields as $ifield)
			{
				$query->values(
					$this->db->quote($ifield) . ',' .
					$this->db->quote($ifield) . ',' .
					$this->db->quote('item') . ',' .
					$this->db->quote('com_k2') . ',' .
					$this->db->quote('import')
				);
				$query->values(
					$this->db->quote($ifield) . ',' .
					$this->db->quote($ifield) . ',' .
					$this->db->quote('item') . ',' .
					$this->db->quote('com_k2') . ',' .
					$this->db->quote('export')
				);
			}

			// Insert category fields
			$catfields = array
						(
							'catwidthindividual',
							'catheightindividual',
							'category_zoom',
							'onlyfeaturedcat',
							'panoramiocat',
							'adsensecat',
							'publisher',
							'formatcat',
							'positioncat',
							'customOverlay'
						);

			foreach ($catfields as $cfield)
			{
				$query->values(
					$this->db->quote($cfield) . ',' .
					$this->db->quote($cfield) . ',' .
					$this->db->quote('category') . ',' .
					$this->db->quote('com_k2') . ',' .
					$this->db->quote('import')
				);
				$query->values(
					$this->db->quote($cfield) . ',' .
					$this->db->quote($cfield) . ',' .
					$this->db->quote('category') . ',' .
					$this->db->quote('com_k2') . ',' .
					$this->db->quote('export')
				);

			}

			$this->db->setQuery($query)->execute();
		}
	}

	/**
	 * Threshold available fields for extension
	 *
	 * @return  int Hardcoded available fields
	 *
	 * @since   7.0
	 */
	public function availableFieldsThresholdLimit()
	{
		return 90;
	}

	/**
	 * Refresh the SEF URLs.
	 *
	 * @param   JInput  $input  An instance of JInput.
	 *
	 * @return  bool  Always returns true.
	 *
	 * @since   7.1.0
	 */
	public function refreshSefUrls(JInput $input)
	{
		// Load all the needed helpers
		$settings = new CsviHelperSettings($this->db);
		$template = new CsviHelperTemplate($input->getInt('template', 0));
		$sef      = new CsviHelperSef($settings, $template, $this->log);

		// Make sure the languages are sorted base on locale instead of random sorting
		$languages = JLanguageHelper::createLanguageList('', JPATH_SITE, true, true);

		if (count($languages) > 1)
		{
			usort(
				$languages,
				function ($a, $b)
				{
					return strcmp($a['value'], $b['value']);
				}
			);
		}

		// Clean the SEF URL table
		if ($input->get('emptytable', false))
		{
			$this->db->truncateTable('#__csvi_sefurls');
		}

		// Get all the products
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('id'))
			->from($this->db->quoteName('#__k2_items'));
		$this->db->setQuery($query);

		$records = $this->db->getIterator();
		$urls    = array();

		// Keep track on the number of records
		$count = 1;

		foreach ($records as $record)
		{
			// Collect URLs
			$urls[] = 'index.php?option=com_k2&view=item&layout=item&id=' . $record->id;

			// Process batches of 500 records
			if ($count > 500)
			{
				if ($urls)
				{
					foreach ($languages as $language)
					{
						$sef->getSefUrls($urls, $language['value']);

						$this->log->setLinenumber($this->log->getLinenumber() + count($urls));
					}

					$this->log->setLinenumber($this->log->getLinenumber() + count($urls));

					// Clean up
					$urls = array();
					$count = 0;
				}
			}

			$count++;
		}

		// Process the final batch of URLs
		if ($urls)
		{
			foreach ($languages as $language)
			{
				$sef->getSefUrls($urls, $language['value']);

				$this->log->setLinenumber($this->log->getLinenumber() + count($urls));
			}

			$this->log->setLinenumber($this->log->getLinenumber() + count($urls));
		}

		return true;
	}
}
