<?php
/**
 * @package     CSVI
 * @subpackage  Productbuilder
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace productbuilder\com_productbuilder\model\export;

defined('_JEXEC') or die;

/**
 * Export Product Builder groups.
 *
 * @package     CSVI
 * @subpackage  Productbuilder
 * @since       6.6.0
 */
class Group extends \CsviModelExports
{
	/**
	 * The addon helper
	 *
	 * @var    \Com_ProductbuilderHelperCom_Productbuilder
	 * @since  6.6.0
	 */
	protected $helper;

	/**
	 * VirtueMart helper
	 *
	 * @var    \Com_VirtuemartHelperCom_Virtuemart
	 * @since  6.6.0
	 */
	protected $virtuemartHelper;

	/**
	 * A list of already retrieved product IDs
	 *
	 * @var    array
	 * @since  6.6.0
	 */
	private $productIds = array();

	/**
	 * A list of already retrieved SKUs
	 *
	 * @var    array
	 * @since  6.6.0
	 */
	private $productSkus = array();

	/**
	 * Export the data.
	 *
	 * @return  void.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \CsviException
	 * @throws  \RuntimeException
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Load the VirtueMart helper
			if (\JFile::exists(JPATH_PLUGINS . '/csviaddon/virtuemart/com_virtuemart/helper/com_virtuemart.php'))
			{
				require_once JPATH_PLUGINS . '/csviaddon/virtuemart/com_virtuemart/helper/com_virtuemart.php';

				$this->virtuemartHelper = new \Com_VirtuemartHelperCom_Virtuemart($this->template, $this->log, $this->fields, $this->db);
			}

			// Get some basic data
			$userFields = array();
			$exportFields = $this->fields->getFields();

			// Group by fields
			$groupFields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupBy = array();
			$groupByFields = array();

			if (isset($groupFields->name))
			{
				$groupByFields = array_flip($groupFields->name);
			}

			// Sort selected fields
			$sortFields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortBy = array();
			$sortByFields = array();

			if (isset($sortFields->name))
			{
				$sortByFields = array_flip($sortFields->name);
			}

			$userFields[] = $this->db->quoteName('g.id');

			if (array_key_exists('id', $groupByFields))
			{
				$groupBy[] = $this->db->quoteName('g.id');
			}

			if (array_key_exists('id', $sortByFields))
			{
				$sortBy[] = $this->db->quoteName('g.id');
			}

			foreach ($exportFields as $field)
			{
				switch ($field->field_name)
				{
					case 'vm_product_sku':
						$userFields[] = $this->db->quoteName('px.vm_product_id');

						if (array_key_exists('vm_product_sku', $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('px.vm_product_id');
						}

						if (array_key_exists('vm_product_sku', $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('px.vm_product_id');
						}
						break;
					case 'product_sku':
						$userFields[] = $this->db->quoteName('p.sku', 'product_sku');

						if (array_key_exists('sku', $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('p.sku');
						}

						if (array_key_exists('sku', $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('p.sku');
						}
						break;
					case 'category_path':
						$userFields[] = $this->db->quoteName('cx.vm_cat_id');

						if (array_key_exists('vm_cat_id', $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('cx.vm_cat_id');
						}

						if (array_key_exists('vm_cat_id', $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('cx.vm_cat_id');
						}
						break;
					case 'published':
					case 'language':
					case 'name':
					case 'ordering':
						$userFields[] = $this->db->quoteName('g.' . $field->field_name);

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('g.' . $field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('g.' . $field->field_name);
						}
						break;
					case 'custom':
						break;
					default:
						$userFields[] = $this->db->quoteName($field->field_name);

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName($field->field_name);
						}
						break;
				}
			}

			// Build the query
			$userFields = array_unique($userFields);
			$query = $this->db->getQuery(true)
				->select(implode(",\n", $userFields))
				->from($this->db->quoteName('#__pb_groups', 'g'))
				->leftJoin(
					$this->db->quoteName('#__pb_group_vm_cat_xref', 'cx')
					. ' ON ' . $this->db->quoteName('cx.group_id') . ' = ' . $this->db->quoteName('g.id')
				)
				->leftJoin(
					$this->db->quoteName('#__pb_group_vm_prod_xref', 'px')
					. ' ON ' . $this->db->quoteName('px.group_id') . ' = ' . $this->db->quoteName('g.id')
				)
				->leftJoin(
					$this->db->quoteName('#__pb_products', 'p')
					. ' ON ' . $this->db->quoteName('p.id') . ' = ' . $this->db->quoteName('g.product_id')
				)
				->leftJoin(
					$this->db->quoteName('#__pb_quant_group', 'q')
					. ' ON ' . $this->db->quoteName('q.group_id') . ' = ' . $this->db->quoteName('g.id')
				);

			// Filter by published state
			$publish_state = $this->template->get('publish_state');

			if ('' !== $publish_state && ($publish_state === 1 || $publish_state === 0))
			{
				$query->where($this->db->quoteName('g.published') . ' = ' . (int) $publish_state);
			}

			// Group the fields
			$groupBy = array_unique($groupBy);

			if (0 !== count($groupBy))
			{
				$query->group($groupBy);
			}

			// Sort set fields
			$sortBy = array_unique($sortBy);

			if (0 !== count($sortBy))
			{
				$query->order($sortBy);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->db->getNumRows();

			if ($logcount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					foreach ($exportFields as $field)
					{
						$fieldName = $field->field_name;

						// Set the field value
						$fieldValue = '';

						if (isset($record->$fieldName))
						{
							$fieldValue = $record->$fieldName;
						}

						// Process the field
						switch ($fieldName)
						{
							case 'category_path':
								$fieldValue = trim($this->virtuemartHelper->createCategoryPath($record->vm_cat_id));
								break;
							case 'vm_product_sku':
								if (!array_key_exists($record->id, $this->productIds))
								{
									// Get all the product IDs
									$query = $this->db->getQuery(true)
										->select($this->db->quoteName('vm_product_id'))
										->from($this->db->quoteName('#__pb_group_vm_prod_xref'))
										->where($this->db->quoteName('group_id') . ' = ' . (int) $record->id);
									$this->db->setQuery($query);
									$this->log->add('Load the VirtueMart product IDs');
									$this->productIds[$record->id] = $this->db->loadColumn();
								}

								$productIds = $this->productIds[$record->id];

								if ($productIds)
								{
									if (!array_key_exists($record->vm_product_id, $this->productSkus))
									{
										$query = $this->db->getQuery(true)
											->select($this->db->quoteName('product_sku'))
											->from($this->db->quoteName('#__virtuemart_products'))
											->where($this->db->quoteName('virtuemart_product_id') . ' IN (' . implode(',', $productIds) . ')');
										$this->db->setQuery($query);
										$this->log->add('Load the VirtueMart product SKUs');
										$this->productSkus[$record->vm_product_id] = implode('|', $this->db->loadColumn());
									}

									$fieldValue = $this->productSkus[$record->vm_product_id];
								}
								break;
							case 'connectWith':
								$fieldValue = 'Categories';

								if ((int) $record->connectWith === 1)
								{
									$fieldValue = 'Products';
								}
								break;
							case 'editable':
							case 'mandatory':
							case 'defaultProd':
								$fieldValue = \JText::_('JNO');

								if ((int) $record->$fieldName === 1)
								{
									$fieldValue = \JText::_('JYES');
								}
								break;
							default:
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldValue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}
}
