<?php
/**
 * @package     CSVI
 * @subpackage  Productbuilder
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace productbuilder\com_productbuilder\model\import;

defined('_JEXEC') or die;

/**
 * Compatibility import.
 *
 * @package     CSVI
 * @subpackage  Productbuilder
 * @since       6.6.0
 */
class Compatibility extends \RantaiImportEngine
{
	/**
	 * Custom fields array.
	 *
	 * @var    \CsviHelperImportFields
	 * @since  6.6.0
	 */
	protected $fields;

	/**
	 * Start the item import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   6.6.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$this->setState($name, $details->value);
			}
		}

		// Let's assume we are good to go
		$this->loaded = true;

		// There must be a tag_name and product_sku
		if ('' === $this->getState('tag_name', '') || !$this->getState('product_sku', false))
		{
			$this->loaded = false;
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \RuntimeException
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			$productSku = $this->getState('product_sku');

			// Check that the product exists
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('virtuemart_product_id'))
				->from($this->db->quoteName('#__virtuemart_products'))
				->where($this->db->quoteName('product_sku') . ' = ' . $this->db->quote($productSku));
			$this->db->setQuery($query);

			$this->log->add('Check if the product exists');
			$virtuemartProductId = $this->db->loadResult();

			if (!$virtuemartProductId)
			{
				$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_PRODUCT_SKU_NOT_FOUND', $productSku));

				return false;
			}

			$tagNames = explode('|', $this->getState('tag_name'));

			foreach ($tagNames as $tagName)
			{
				// Check if the tag exists
				$query->clear()
					->select($this->db->quoteName('id'))
					->from($this->db->quoteName('#__pb_tags'))
					->where($this->db->quoteName('name') . ' = ' . $this->db->quote($tagName));
				$this->db->setQuery($query);

				$this->log->add('Check if the tag exists');
				$tagId = $this->db->loadResult();

				if (!$tagId)
				{
					// Set the columns
					$columns = array(
						$this->db->quoteName('name'),
						$this->db->quoteName('published'),
						$this->db->quoteName('color'),
						$this->db->quoteName('note'),
					);

					// Set the values
					$values = array(
						$this->db->quote($tagName),
						1,
						$this->db->quote('000000'),
						$this->db->quote(''),
					);

					// Create the missing tag
					$query->clear()
						->insert($this->db->quoteName('#__pb_tags'))
						->columns($columns)
						->values(implode(',', $values));
					$this->db->setQuery($query);

					try
					{
						$this->db->execute();
						$tagId = $this->db->insertid();
						$this->log->add('Add tag name');
						$this->log->addStats('added', \JText::sprintf('COM_CSVI_PRODUCTBUILDER_ADD_TAG', $tagName));

						// Still no tag ID, giving up
						if (!$tagId)
						{
							return false;
						}
					}
					catch (\Exception $e)
					{
						$this->log->add('Cannot add tag name. Error: ' . $e->getMessage(), false);
						$this->log->addStats('incorrect', \JText::_('COM_CSVI_CANNOT_ADD_TAG_NAME'));

						return false;
					}
				}
				else
				{
					$this->log->addStats('information', \JText::sprintf('COM_CSVI_TAG_NAME_ALREADY_EXISTS', $tagName));
				}

				// Store the relationship
				$query->clear()
					->select($this->db->quoteName('tag_id'))
					->from($this->db->quoteName('#__pb_tag_xref_vmprod'))
					->where($this->db->quoteName('tag_id') . ' = ' . (int) $tagId)
					->where($this->db->quoteName('vm_prod_id') . ' = ' . (int) $virtuemartProductId);
				$this->db->setQuery($query);
				$result = $this->db->loadResult();

				if (!$result)
				{
					$columns = array(
						$this->db->quoteName('tag_id'),
						$this->db->quoteName('vm_prod_id'),
					);

					$values = array(
						$tagId,
						$virtuemartProductId,
					);

					$query->clear()
						->insert($this->db->quoteName('#__pb_tag_xref_vmprod'))
						->columns($columns)
						->values(implode(',', $values));
					$this->db->setQuery($query);

					try
					{
						$this->log->add('Store the tag product reference');
						$this->db->execute();
						$this->log->addStats('added', \JText::sprintf('COM_CSVI_ADD_TAG_CROSSREFERENCE', $tagName, $productSku));
					}
					catch (\Exception $e)
					{
						$this->log->add('Cannot add tag product reference. Error: ' . $e->getMessage(), false);
						$this->log->addStats('incorrect', \JText::_('COM_CSVI_CANNOT_ADD_TAG_CROSSREFERENCE'));

						return false;
					}
				}
				else
				{
					$this->log->addStats('information', \JText::sprintf('COM_CSVI_TAG_CROSSREFERENCE_ALREADY_EXISTS', $tagName, $productSku));
				}
			}

			return true;
		}

		return false;
	}
}
