<?php
/**
 * @package     CSVI
 * @subpackage  Productbuilder
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace productbuilder\com_productbuilder\model\import;

defined('_JEXEC') or die;

/**
 * Groups import.
 *
 * @package     CSVI
 * @subpackage  Productbuilder
 * @since       6.6.0
 */
class Group extends \RantaiImportEngine
{
	/**
	 * Group table
	 *
	 * @var    \ProductbuilderTableGroup
	 * @since  6.6.0
	 */
	private $groupTable;

	/**
	 * Quantity table
	 *
	 * @var    \ProductbuilderTableQuantity
	 * @since  6.6.0
	 */
	private $quantityTable;

	/**
	 * VirtueMart helper
	 *
	 * @var    \Com_VirtuemartHelperCom_Virtuemart
	 * @since  6.6.0
	 */
	private $virtuemartHelper;

	/**
	 * Custom fields array.
	 *
	 * @var    \CsviHelperImportFields
	 * @since  6.6.0
	 */
	protected $fields;

	/**
	 * Holds the category separator
	 *
	 * @var    string
	 * @since  6.6.0
	 */
	private $categorySeparator;

	/**
	 * A cache of category IDs
	 *
	 * @var    array
	 * @since  6.6.0
	 */
	private $categoryCache = array();

	/**
	 * Load the VirtueMart helper.
	 *
	 * @return  void
	 *
	 * @since   6.6.0
	 */
	public function onBeforeStart()
	{
		require_once JPATH_PLUGINS . '/csviaddon/virtuemart/com_virtuemart/helper/com_virtuemart.php';
		$this->virtuemartHelper = new \Com_VirtuemartHelperCom_Virtuemart($this->template, $this->log, $this->fields, $this->db);
	}

	/**
	 * Start the item import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \RuntimeException
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'product_display_type':
						switch (strtolower($value))
						{
							case 'global':
								$value = '';
								break;
							case 'dropdown':
								$value = 0;
								break;
							case 'radio':
								$value = 1;
								break;
							case 'thumbnail':
								$value = 2;
								break;
							case 'radioimage':
								$value = 3;
								break;
						}

						$this->setState('prod_display', $value);
						break;
					case 'connectWith':
						$newValue = 0;

						if (strtolower($value) === 'products')
						{
							$newValue = 1;
						}

						$this->setState($name, $newValue);
						break;
					case 'editable':
					case 'published':
					case 'displ_qbox':
						switch (strtolower($value))
						{
							case 'n':
							case 'no':
							case '0':
								$value = 0;
								break;
							default:
								$value = 1;
								break;
						}

						$this->setState($name, $value);
						break;
					case 'mandatory':
					case 'defOption':
					case 'q_box_type':
						switch (strtolower($value))
						{
							case 'y':
							case 'yes':
							case '1':
								$value = 1;
								break;
							default:
								$value = 0;
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		// There must be a tag_name and product_sku
		if ('' === $this->getState('name', '') || !$this->getState('product_sku', false))
		{
			$this->loaded = false;
		}
		else
		{
			// Find the product ID
			if ($productId = $this->helper->loadProductId($this->getState('product_sku')))
			{
				$this->state->set('product_id', $productId);

				// Load the existing group
				if ($this->groupTable->load(
						array(
							'name' => $this->getState('name'),
							'product_id' => $productId,
						)
						)
					)
				{
					if (!$this->template->get('overwrite_existing_data'))
					{
						$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_GROUP_NAME', $this->getState('name', '')));
						$this->loaded = false;
					}
				}
			}
			else
			{
				$this->log->addStats('error', \JText::sprintf('COM_CSVI_CANNOT_FIND_CUSTOM_PRODUCT_SKU', $this->getState('product_sku')));

				$this->loaded = false;
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \RuntimeException
	 * @throws  \InvalidArgumentException
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			if (!$this->groupTable->get('id') && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new products when user chooses to ignore new products
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('name')));
			}
			else
			{
				// Check for a default product
				if ($defaultProductSku = $this->getState('default_product_sku', false))
				{
					$this->setState('defaultProd', $this->virtuemartHelper->getProductIdBySku($defaultProductSku));
				}

				// Check if we have a new group
				if (!$this->groupTable->get('id'))
				{
					// Set some default values
					$this->setState('language', $this->getState('language', '*'));
					$this->setState('editable', $this->getState('editable', 1));
					$this->setState('mandatory', $this->getState('mandatory', 0));
					$this->setState('connectWith', $this->getState('connectWith', 0));
					$this->setState('defaultProd', $this->getState('defaultProd', 0));
					$this->setState('products_ordering', $this->getState('products_ordering', 'prd_lang.product_name'));
					$this->setState('products_ordering_dir', $this->getState('products_ordering_dir', 'ASC'));
					$this->setState('def_quantity', $this->getState('def_quantity', 1));
					$this->setState('displ_qbox', $this->getState('displ_qbox', 1));
					$this->setState('q_box_type', $this->getState('q_box_type', 0));
				}

				// Bind the data
				$this->groupTable->bind($this->state);

				if ($this->groupTable->store())
				{
					// Add related products or categories
					if ((int) $this->getState('connectWith') === 1)
					{
						try
						{
							$this->storeConnectedProducts();
						}
						catch (\Exception $e)
						{
							$this->log->addStats('error', \JText::_('COM_CSVI_CANNOT_ADD_CONNECTED_PRODUCTS'));
							$this->log->add($e->getMessage(), false);
						}
					}
					else
					{
						try
						{
							$this->storeConnectedCategories();
						}
						catch (\Exception $e)
						{
							$this->log->addStats('error', \JText::_('COM_CSVI_CANNOT_ADD_CONNECTED_CATEGORIES'));
							$this->log->add($e->getMessage(), false);
						}
					}

					// Add quantity options if we have an ID
					if ($groupId = $this->groupTable->get('id'))
					{
						$this->quantityTable->bind($this->state);
						$this->quantityTable->set('group_id', $groupId);

						try
						{
							$this->quantityTable->check();
							$this->quantityTable->store();
						}
						catch (\Exception $e)
						{
							$this->log->addStats('error', $e->getMessage());
							$this->log->add($e->getMessage(), false);
						}
					}
				}
			}

			return true;
		}

		return false;
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function loadTables()
	{
		$this->groupTable = $this->getTable('Group');
		$this->quantityTable = $this->getTable('Quantity');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function clearTables()
	{
		$this->groupTable->reset();
		$this->quantityTable->reset();
	}

	/**
	 * Store related products.
	 *
	 * @return  void.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \RuntimeException
	 */
	private function storeConnectedProducts()
	{
		$productIds = array();
		$products = explode('|', $this->getState('vm_product_sku'));

		foreach ($products as $productSku)
		{
			// Find the product ID
			$productId = $this->virtuemartHelper->getProductIdBySku($productSku);

			if ($productId)
			{
				$productIds[] = $productId;
			}
		}

		// Check if there are any product IDs to process
		if ($productIds)
		{
			// Delete any existing IDs
			$query = $this->db->getQuery(true)
				->delete($this->db->quoteName('#__pb_group_vm_prod_xref'))
				->where($this->db->quoteName('group_id') . ' = ' . $this->groupTable->get('id'))
				->where($this->db->quoteName('vm_product_id') . ' IN (' . implode(',', $productIds) .')');
			$this->db->setQuery($query)->execute();

			// Add the new list of IDs
			$query->clear()
				->insert($this->db->quoteName('#__pb_group_vm_prod_xref'))
				->columns($this->db->quoteName(array('group_id', 'vm_product_id')));

			foreach ($productIds as $productId)
			{
				$query->values($this->groupTable->get('id') . ',' . $productId);
			}

			$this->db->setQuery($query)->execute();

		}
	}

	/**
	 * Store related categories.
	 *
	 * @return  void.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \Exception
	 * @throws  \RuntimeException
	 */
	private function storeConnectedCategories()
	{
		$categoryIds = array();
		$categoryPaths = explode('|', $this->getState('category_path'));

		foreach ($categoryPaths as $categoryPath)
		{
			$this->log->add('Process path ' . $categoryPath, false);

			// Find the product ID
			$categoryId = $this->getCategoryId($categoryPath);

			$this->log->add('Found category ID ' . $categoryId);

			if ($categoryId)
			{
				$categoryIds[] = $categoryId;
			}
		}

		// Check if there are any product IDs to process
		if ($categoryIds)
		{
			// Delete any existing IDs
			$query = $this->db->getQuery(true)
				->delete($this->db->quoteName('#__pb_group_vm_cat_xref'))
				->where($this->db->quoteName('group_id') . ' = ' . $this->groupTable->get('id'))
				->where($this->db->quoteName('vm_cat_id') . ' IN (' . implode(',', $categoryIds) .')');
			$this->db->setQuery($query)->execute();

			// Add the new list of IDs
			$query->clear()
				->insert($this->db->quoteName('#__pb_group_vm_cat_xref'))
				->columns($this->db->quoteName(array('group_id', 'vm_cat_id')));

			foreach ($categoryIds as $categoryId)
			{
				$query->values($this->groupTable->get('id') . ',' . $categoryId);
			}

			$this->db->setQuery($query);

			$this->log->add('Store connected categories');

			$this->db->execute();
		}
	}

	/**
	 * Creates categories from slash delimited line.
	 *
	 * @param   array  $category_path  Contains the category/categories for a product.
	 * @param   int    $vendor_id      The id of the vendor the category belongs to.
	 *
	 * @return  array  Contains the category IDs.
	 *
	 * @throws  \Exception
	 *
	 * @since   3.0
	 */
	private function getCategoryId($category_path, $vendor_id = 1)
	{
		$lang = $this->template->get('language');
		$category = array();

		// Load the category separator
		if (null === $this->categorySeparator)
		{
			$this->categorySeparator = $this->template->get('category_separator', '/');
		}

		// Check if category_path is an array, if not make it one
		if (!is_array($category_path))
		{
			$category_path = array($category_path);
		}

		// Get all categories in this field delimited with |
		foreach ($category_path as $line)
		{
			// Explode slash delimited category tree into array
			$category_list = explode($this->categorySeparator, $line);
			$category_count = count($category_list);
			$category_id = null;
			$category_parent_id = '0';

			// For each category in array
			for ($i = 0; $i < $category_count; $i++)
			{
				// Check the cache first
				if (array_key_exists($category_parent_id . '.' . $category_list[$i], $this->categoryCache))
				{
					$category_id = $this->categoryCache[$category_parent_id . '.' . $category_list[$i]];
				}
				else
				{
					// See if this category exists with it's parent in xref
					$query = $this->db->getQuery(true)
						->select($this->db->quoteName('c.virtuemart_category_id'))
						->from($this->db->quoteName('#__virtuemart_categories', 'c'))
						->leftJoin(
							$this->db->quoteName('#__virtuemart_category_categories', 'x')
							. ' ON ' . $this->db->quoteName('c.virtuemart_category_id') . ' = ' . $this->db->quoteName('x.category_child_id')
						)
						->leftJoin(
							$this->db->quoteName('#__virtuemart_categories_' . $lang, 'l')
							. ' ON ' . $this->db->quoteName('l.virtuemart_category_id') . ' = ' . $this->db->quoteName('c.virtuemart_category_id')
						)
						->where($this->db->quoteName('l.category_name') . ' = ' . $this->db->quote($category_list[$i]))
						->where($this->db->quoteName('x.category_child_id') . ' = ' . $this->db->quoteName('c.virtuemart_category_id'))
						->where($this->db->quoteName('x.category_parent_id') . ' = ' . (int) $category_parent_id);
					$this->db->setQuery($query);
					$category_id = $this->db->loadResult();

					$this->log->add(\JText::sprintf('COM_CSVI_CHECK_CATEGORY_EXISTS', $category_list[$i]));

					// Add result to cache if category exists
					if ($category_id)
					{
						$this->categoryCache[$category_parent_id . '.' . $category_list[$i]] = $category_id;
					}
				}

				// Set this category as parent of next in line
				$category_parent_id = $category_id;
			}

			$category[] = $category_id;
		}

		// Return an array with the last category_ids which is where the product goes
		return $category[0];
	}
}
