<?php
/**
 * @package     CSVI
 * @subpackage  Productbuilder
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace productbuilder\com_productbuilder\model\import;

defined('_JEXEC') or die;

/**
 * Product import.
 *
 * @package     CSVI
 * @subpackage  Productbuilder
 * @since       6.6.0
 */
class Product extends \RantaiImportEngine
{
	/**
	 * Group table
	 *
	 * @var    \ProductbuilderTableProduct
	 * @since  6.6.0
	 */
	private $productTable;

	/**
	 * Custom fields array.
	 *
	 * @var    \CsviHelperImportFields
	 * @since  6.6.0
	 */
	protected $fields;

	/**
	 * Start the item import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   6.6.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($value)
				{
					case 'published':
					case 'compatibility':
						switch (strtolower($value))
						{
							case 'n':
							case 'no':
							case '0':
								$value = 0;
								break;
							default:
								$value = 1;
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		// Let's assume we are good to go
		$this->loaded = true;

		// There must be an sku value
		if (!$this->getState('sku', false))
		{
			$this->loaded = false;
		}
		else
		{
			// Find the product ID
			if ($productId = $this->helper->loadProductId($this->getState('sku')))
			{
				$this->setState('id', $productId);

				// Load the existing product
				if ($this->productTable->load($productId) && !$this->template->get('overwrite_existing_data'))
				{
					$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_PRODUCT_NAME', $this->getState('sku', '')));
					$this->loaded = false;
				}
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \InvalidArgumentException
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			$productId = $this->productTable->get('id');

			if (!$productId && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new products when user chooses to ignore new products
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('sku')));
			}
			else
			{
				// Set some default values
				if (!$productId)
				{
					$this->setState('language', $this->getState('language', '*'));
					$this->setState('compatibility', $this->getState('compatibility', 1));
				}

				// Bind the data
				$this->productTable->bind($this->state);

				if (!$this->productTable->store())
				{
					$this->log->add('Cannot store product', false);

					return false;
				}
			}

			return true;
		}

		return false;
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function loadTables()
	{
		$this->productTable = $this->getTable('Product');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function clearTables()
	{
		$this->productTable->reset();
	}
}
