<?php
/**
 * @package     CSVI
 * @subpackage  VirtueMart
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace virtuemart\com_virtuemart\model\import;

defined('_JEXEC') or die;

/**
 * Main processor for handling product images.
 *
 * @package     CSVI
 * @subpackage  VirtueMart
 * @since       6.0
 */
class Mediaproduct extends \RantaiImportEngine
{
	/**
	 * The media table
	 *
	 * @var    \VirtueMartTableMedia
	 * @since  6.0
	 */
	private $media;

	/**
	 * The product media cross reference table
	 *
	 * @var    \VirtueMartTableProductMedia
	 * @since  6.0
	 */
	private $productMedia;

	/**
	 * Here starts the processing.
	 *
	 * @return  bool  Returns true on success | False on failure.
	 *
	 * @since   3.0
	 */
	public function getStart()
	{
		$this->setState('virtuemart_product_id', $this->helper->getProductId());

		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				// Check if the field needs extra treatment
				switch ($name)
				{
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		// Set the record identifier
		$this->recordIdentity = $this->getState('product_sku', $this->getState('virtuemart_product_id'));

		return true;
	}

	/**
	 * Process each record and store it in the database.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   3.0
	 */
	public function getProcessRecord()
	{
		// Get the needed data
		$product_sku           = $this->getState('product_sku', false);
		$virtuemart_product_id = $this->getState('virtuemart_product_id', false);

		if (!$virtuemart_product_id)
		{
			$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_DEBUG_NO_SKU', $product_sku));
			$this->log->add('No product identifier or ID found', false);

			return false;
		}
		else
		{
			if (!$this->template->get('overwrite_existing_data'))
			{
				$this->log->add(\JText::sprintf('COM_CSVI_DATA_EXISTS_PRODUCT_MEDIA_SKU', $product_sku));
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_PRODUCT_MEDIA_SKU', $product_sku));
			}
			else
			{
				$this->log->add('Processing item: ' . $this->recordIdentity, false);

				// Do nothing for new products when user chooses to ignore new products
				if ($virtuemart_product_id)
				{
					$this->processMedia();
				}
			}

			return true;
		}
	}

	/**
	 * Process media files.
	 *
	 * @return  bool Returns true on OK | False on failure.
	 *
	 * @since   4.0
	 */
	private function processMedia()
	{
		$fileTitle    = false;
		$fileDesc     = false;
		$fileMeta     = false;
		$fileMimeType = false;
		$titles       = array();
		$descriptions = array();
		$metas        = array();
		$mimeTypes    = array();

		$generate_image = $this->template->get('auto_generate_image_name', false);

		// Check if any image handling needs to be done
		if (!is_null($this->file_url) || $generate_image)
		{
			// Check if we have any images
			if (is_null($this->file_url) && $generate_image)
			{
				$this->_createImageName();
			}

			// Create an array of images to process
			$images = explode('|', $this->file_url);
			$thumbs = explode('|', $this->file_url_thumb);

			// Check if there is file title to import
			if ($this->getState('file_title', false))
			{
				$titles    = explode('|', $this->file_title);
				$fileTitle = true;
			}

			// Check if there is a file description
			if ($this->getState('file_description', false))
			{
				$descriptions = explode('|', $this->file_description);
				$fileDesc     = true;
			}

			// Check if there is file meta
			if ($this->getState('file_meta', false))
			{
				$metas    = explode('|', $this->file_meta);
				$fileMeta = true;
			}

			// Check if there is mimetype
			if ($this->getState('file_mimetype', false))
			{
				$mimeTypes = explode('|', $this->file_mimetype);
				$fileMimeType = true;
			}

			$order         = explode('|', $this->file_ordering);
			$downloadables = explode('|', $this->file_downloadable);
			$ordering      = 1;
			$max_width     = $this->template->get('resize_max_width', 1024);
			$max_height    = $this->template->get('resize_max_height', 768);

			// Image handling
			$imagehelper = new \CsviHelperImage($this->template, $this->log, $this->csvihelper);

			// Delete existing image links
			if ($this->template->get('delete_product_images', false))
			{
				$query = $this->db->getQuery(true)
					->delete($this->db->quoteName('#__virtuemart_product_medias'))
					->where($this->db->quoteName('virtuemart_product_id') . '=' . $this->virtuemart_product_id);
				$this->db->setQuery($query);
				$this->db->execute();
				$this->log->add('Delete images', true);
			}

			foreach ($images as $key => $image)
			{
				$image = trim($image);
				$productFullImageOutput = '';

				// Create image name if needed
				if (count($images) == 1)
				{
					$img_counter = 0;
				}
				else
				{
					$img_counter = $key + 1;
				}

				if ($generate_image)
				{
					$productFullImageOutput = $this->_createImageName($img_counter);
				}

				if (!empty($image))
				{
					// Get the image path
					$imgpath = $this->template->get('file_location_product_files');

					// Verify the original image
					if ($imagehelper->isRemote($image))
					{
						$original = $image;
						$full_path = $imgpath;
					}
					else
					{
						// Check if the image contains the image path
						$dirname = dirname($image);

						if (strpos($imgpath, $dirname) !== false)
						{
							// Collect rest of folder path if it is more than image default path
							$imageleftpath = str_replace($imgpath, '', $dirname . '/');
							$image = basename($image);

							if ($imageleftpath)
							{
								$image = $imageleftpath . $image;
							}
						}

						$original = $imgpath . $image;

						// Get subfolders
						$path_parts = pathinfo($original);
						$full_path = $path_parts['dirname'] . '/';
					}

					// Generate image names
					if ($this->template->get('process_image', false))
					{
						if ($generate_image)
						{
							$file_details = $imagehelper->ProcessImage($original, $full_path, $productFullImageOutput);
						}
						else
						{
							$file_details = $imagehelper->ProcessImage($original, $full_path);
						}
					}
					else
					{
						$file_details['exists'] = true;
						$file_details['isimage'] = $imagehelper->isImage(JPATH_SITE . '/' . $image);
						$file_details['name'] = $image;
						$file_details['output_name'] = basename($image);

						if (file_exists(JPATH_SITE . '/' . $image))
						{
							$file_details['mime_type'] = $imagehelper->findMimeType($image);
						}
						else
						{
							$file_details['mime_type'] = '';
						}

						$file_details['output_path'] = $full_path;
					}

					// Process the file details
					if ($file_details['exists'])
					{
						// Check if the image is an external image
						if (substr($file_details['name'], 0, 4) === 'http')
						{
							$this->log->addStats('incorrect', 'COM_CSVI_VM_NOSUPPORT_URL');
						}
						else
						{
							$media = array();

							if ($fileTitle)
							{
								$firstTitle          = ($titles[0]) ? $titles[0] : '';
								$title               = (isset($titles[$key])) ? $titles[$key] : $firstTitle;
								$media['file_title'] = $title;
							}

							if ($fileDesc)
							{
								$firstDesc                 = ($descriptions[0]) ? $descriptions[0] : '';
								$description               = (isset($descriptions[$key])) ? $descriptions[$key] : $firstDesc;
								$media['file_description'] = $description;
							}

							if ($fileMeta)
							{
								$firstMeta          = ($metas[0]) ? $metas[0] : '';
								$meta               = (isset($metas[$key])) ? $metas[$key] : $firstMeta;
								$media['file_meta'] = $meta;
							}

							if ($fileMimeType)
							{
								$mimeType               = (isset($mimeTypes[$key])) ? $mimeTypes[$key] : $file_details['mime_type'];
								$media['file_mimetype'] = $mimeType;
							}

							$fileDownloadable = (isset($downloadables[$key])) ? $downloadables[$key] : (($file_details['isimage']) ? 0 : 1);
							$media['virtuemart_vendor_id'] = $this->virtuemart_vendor_id;

							if ($this->template->get('autofill'))
							{
								$media['file_title']       = $file_details['output_name'];
								$media['file_description'] = $file_details['output_name'];
								$media['file_meta']        = $file_details['output_name'];
							}

							$media['file_type']             = 'product';
							$media['file_is_product_image'] = 1;
							$media['file_is_downloadable']  = $fileDownloadable;
							$media['file_is_forSale']       = 0;
							$media['file_url']              = (empty($file_details['output_path'])) ? $file_details['output_name'] : $file_details['output_path'] . $file_details['output_name'];
							$media['published']             = $this->published;

							// load the row if exists
							$this->media->load(['file_url' => $media['file_url']]);

							// Create the thumbnail
							if ($file_details['isimage'])
							{
								$thumb = (isset($thumbs[$key])) ? $thumbs[$key] : null;

								if ($this->template->get('thumb_create'))
								{
									$thumb_sizes = getimagesize(JPATH_SITE . '/' . $media['file_url']);

									if (empty($thumb) || $generate_image)
									{
										// Get the subfolder structure
										$thumb_path = str_ireplace($imgpath, '', $full_path);

										if (empty($this->file_url_thumb))
										{
											$thumb = 'resized/' . $thumb_path . basename($media['file_url']);
										}
									}
									else
									{
										// Check if we are not overwriting any large images
										$thumb_path_parts = pathinfo($thumb);

										if ($thumb_path_parts['dirname'] === '.')
										{
											$this->log->AddStats('incorrect', 'COM_CSVI_THUMB_OVERWRITE_FULL');
											$thumb = false;
										}
									}

									if ($thumb && ($thumb_sizes[0] < $max_width || $thumb_sizes[1] < $max_height))
									{
										$media['file_url_thumb'] = $imagehelper->createThumbnail($media['file_url'], $imgpath, $thumb);
									}
									else
									{
										$media['file_url_thumb'] = '';
									}
								}
								else
								{
									$media['file_url_thumb'] = (empty($thumb)) ? $media['file_url'] : $file_details['output_path'] . $thumb;

									if (substr($media['file_url_thumb'], 0, 4) === 'http')
									{
										$this->log->add(\JText::sprintf('COM_CSVI_RESET_THUMB_NOHTTP', $media['file_url_thumb']));
										$media['file_url_thumb'] = '';
									}
								}
							}
							else
							{
								$media['file_is_product_image'] = 0;
								$media['file_url_thumb']        = '';
							}

							// Bind the media data
							$this->media->bind($media);

							// Store the media data
							if ($this->media->store())
							{
								// Watermark the image
								if ($this->template->get('full_watermark', 'image') && $file_details['isimage'])
								{
									$imagehelper->addWatermark(JPATH_SITE . '/' . $media['file_url']);
								}

								// Store the product image relation
								$data                          = array();
								$data['virtuemart_product_id'] = $this->getState('virtuemart_product_id');
								$data['virtuemart_media_id']   = $this->media->virtuemart_media_id;
								$data['ordering']              = (isset($order[$key]) && !empty($order[$key])) ? $order[$key] : $ordering;
								$this->productMedia->bind($data);

								$this->productMedia->check();

								if (!$this->productMedia->store())
								{
									$this->log->add('Can not store the product media relation', true);
									$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_MEDIA_NOT_ADDED', $this->media->getError()));
									$ordering++;
								}
							}
							else
							{
								$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_MEDIA_NOT_ADDED', $this->media->getError()));

								return false;
							}

							// Reset the product media table
							$this->media->reset();
							$this->productMedia->reset();
						} // else
					} // if
				} // if
			} // foreach
		} // if

		return true;
	}

	/**
	 * Create image name.
	 *
	 * Check if the user wants to have CSVI VirtueMart create the image names if so
	 * create the image names without path.
	 *
	 * @param   int  $ordering  The number to apply to a generated image name.
	 *
	 * @return  string  The name of the image.
	 *
	 * @since   3.0
	 */
	private function _createImageName($ordering = 0)
	{
		$this->log->add('Create image name', false);

		// Create extension
		$ext = $this->template->get('autogenerateext');

		// Check if the user wants to convert the images to a different type
		switch ($this->template->get('type_generate_image_name'))
		{
			case 'product_sku':
				$this->log->add('Create image based on product SKU', false);

				if ($this->getState('product_sku', false))
				{
					$name = $this->getState('product_sku');
				}
				else
				{
					$this->log->addStats('error', 'COM_CSVI_CANNOT_FIND_PRODUCT_SKU');

					return false;
				}
				break;
			case 'product_name':
				$this->log->add('Create image based on product name', false);

				if (!is_null($this->productLang->product_name))
				{
					$name = str_replace(' ', '_', $this->productLang->product_name);
				}
				else
				{
					$this->log->addStats('error', 'COM_CSVI_CANNOT_FIND_PRODUCT_NAME');

					return false;
				}
				break;
			case 'product_id':
				$this->log->add('Create image based on product ID', false);

				if ($this->getState('virtuemart_product_id', false))
				{
					$name = $this->getState('virtuemart_product_id');
				}
				else
				{
					$this->log->addStats('error', 'COM_CSVI_CANNOT_FIND_PRODUCT_ID');

					return false;
				}
				break;
			case 'random':
				$this->log->add('Create image based on a random value');
				$name = mt_rand();
				break;
			default:
				$this->log->addStats('error', 'COM_CSVI_CANNOT_FIND_PRODUCT_SKU');

				return false;
				break;
		}

		// Build the new name
		if ($ordering > 0)
		{
			$image_name = $name . '_' . $ordering . '.' . $ext;
		}
		else
		{
			$image_name = $name . '.' . $ext;
		}

		$this->log->add('Created image name ' . $image_name, false);
		$this->setState('product_full_image_output', $image_name);

		// Check if the user is supplying image data
		if (!$this->getState('file_url', false))
		{
			$this->setState('file_url', $this->getState('product_full_image_output'));
		}

		return $image_name;
	}

	/**
	 * Load the product related tables.
	 *
	 * @return  void.
	 *
	 * @since   3.0
	 */
	public function loadTables()
	{
		$this->media = $this->getTable('Media');
		$this->productMedia = $this->getTable('ProductMedia');
	}

	/**
	 * Cleaning the product related tables.
	 *
	 * @return  void.
	 *
	 * @since   3.0
	 */
	public function clearTables()
	{
		$this->media->reset();
		$this->productMedia->reset();
	}
}
