<?php
/**
 * @package     CSVI
 * @subpackage  Plugin.Articles
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * Related articles.
 *
 * @package     CSVI
 * @subpackage  Plugin.Articles
 * @see         https://www.daycounts.com/shop/plugins/related-articles-custom-field
 * @since       6.0
 */
class PlgCsviextArticles extends RantaiPluginDispatcher
{
	/**
	 * The unique ID of the plugin
	 *
	 * @var    string
	 * @since  6.0
	 */
	private $id = 'articles';

	/**
	 * JDatabase handler
	 *
	 * @var    JDatabase
	 * @since  6.0
	 */
	protected $db = null;

	/**
	 * Construct the class.
	 *
	 * @since   6.0
	 */
	public function __construct()
	{
		// Set the dependencies
		$this->db = JFactory::getDbo();
	}

	/**
	 * Method to get the field options.
	 *
	 * @param   string         $plugin                 The ID of the plugin.
	 * @param   array          $params                 An array of settings.
	 * @param   int            $virtuemart_product_id  The product ID.
	 * @param   int            $virtuemart_custom_id   The custom ID.
	 * @param   CsviHelperLog  $log                    The CSVI logger.
	 *
	 * @return  mixed  String if plugin needs to handle request | False if plugin does not handle request.
	 *
	 * @since   6.0
	 */
	public function getCustomParam($plugin, $params, $virtuemart_product_id, $virtuemart_custom_id, $log)
	{
		if ($plugin == $this->id)
		{
			// There are no params
			return '';
		}
		else
		{
			return false;
		}
	}

	/**
	 * Method to get the field options.
	 *
	 * @param   string         $plugin                     The ID of the plugin.
	 * @param   array          $params                     An array of settings.
	 * @param   int            $virtuemart_product_id      The product ID.
	 * @param   int            $virtuemart_custom_id       The custom ID.
	 * @param   int            $virtuemart_customfield_id  The custom field ID.
	 * @param   CsviHelperLog  $log                        The CSVI logger.
	 * @param   int            $product_parent_id          The product parent ID.
	 * @param   int            $productAsDerived           The product has to be derived product.
	 * @param   array          $multiLangValues            Multi language value array
	 *
	 * @return   mixed  Void if plugin needs to handle request | False if plugin does not handle request.
	 *
	 * @since   6.0
	 *
	 * @throws  RuntimeException
	 */
	public function onAfterStoreCustomfield($plugin, $params, $virtuemart_product_id, $virtuemart_custom_id, $virtuemart_customfield_id, $log, $product_parent_id, $productAsDerived, $multiLangValues = array())
	{
		if ($plugin == $this->id)
		{
			// Data is received in the format:
			// value1#value2;value1#value2

			// Remove existing values for this parameter
			$query = $this->db->getQuery(true)
				->delete($this->db->quoteName('#__virtuemart_product_custom_plg_articles'))
				->where($this->db->quoteName('virtuemart_product_id') . ' = ' . (int) $virtuemart_product_id);
			$this->db->setQuery($query);

			if ($this->db->execute())
			{
				// Get all the values
				list($articleTitles, $showAs) = explode('[', $params);
				$param_entries = explode('#', $articleTitles);

				if (empty($showAs))
				{
					$showAs = 'title';
				}

				foreach ($param_entries as $pkey => $pvalue)
				{
					// Split the data to title and show as
					$param_entries[$pkey] = $this->db->quote($pvalue);
				}

				// Load the article IDs
				$query = $this->db->getQuery(true)
					->select($this->db->quoteName('id'))
					->from($this->db->quoteName('#__content'))
					->where($this->db->quoteName('title') . ' IN (' . implode(',', $param_entries) . ')');
				$this->db->setQuery($query);
				$articleIds = $this->db->loadColumn();

				$log->add('Retrieve custom field value IDs');

				if (is_array($articleIds))
				{
					// Store the values
					$query = $this->db->getQuery(true)
						->insert($this->db->quoteName('#__virtuemart_product_custom_plg_articles'))
						->columns(array('virtuemart_custom_id', 'virtuemart_product_id', 'articles', 'showas'))
						->values(
							(int) $virtuemart_custom_id . ',' .
							(int) $virtuemart_product_id . ',' .
							$this->db->quote(implode(',', $articleIds)) . ',' .
							$this->db->quote($showAs)
						);
					$this->db->setQuery($query)->execute();

					$log->add('Insert values for custom field');
				}
			}
		}
		else
		{
			return false;
		}
	}

	/**
	 * Export the values.
	 *
	 * @param   string         $plugin                     The ID of the plugin.
	 * @param   string         $custom_param               The custom parameter values.
	 * @param   int            $virtuemart_product_id      The product ID.
	 * @param   int            $virtuemart_custom_id       The custom ID.
	 * @param   int            $virtuemart_customfield_id  The custom field ID.
	 * @param   CsviHelperLog  $log                        The CSVI logger.
	 *
	 * @return  array  List of values to export.
	 *
	 * @since   6.0
	 */
	public function exportCustomValues($plugin, $custom_param, $virtuemart_product_id, $virtuemart_custom_id, $virtuemart_customfield_id, $log)
	{
		if ($plugin == $this->id)
		{
			// Get the article IDs
			$query = $this->db->getQuery(true)
				->select(
					$this->db->quoteName('r.articles') . ',' .
					$this->db->quoteName('c.virtuemart_custom_id')
				)
				->from($this->db->quoteName('#__virtuemart_product_custom_plg_articles', 'r'))
				->leftJoin(
					$this->db->quoteName('#__virtuemart_customs', 'c')
					. ' ON ' . $this->db->quoteName('r.virtuemart_custom_id') . ' = ' . $this->db->quoteName('c.virtuemart_custom_id')
				)
				->leftJoin(
					$this->db->quoteName('#__virtuemart_product_customfields', 'f')
					. ' ON ' . $this->db->quoteName('c.virtuemart_custom_id') . ' = ' . $this->db->quoteName('f.virtuemart_custom_id')
					. ' AND ' . $this->db->quoteName('r.virtuemart_product_id') . ' = ' . $this->db->quoteName('f.virtuemart_product_id')
				)
				->where($this->db->quoteName('r.virtuemart_product_id') . ' = ' . (int) $virtuemart_product_id)
				->where($this->db->quoteName('r.virtuemart_custom_id') . ' = ' . (int) $virtuemart_custom_id);
			$this->db->setQuery($query);
			$options = $this->db->loadObjectList();

			// Group the data correctly
			$newoptions = array();

			foreach ($options as $option)
			{
				$articles = explode(',', $option->articles);

				foreach ($articles as $article)
				{
					// Get the article title
					$query = $this->db->getQuery(true)
						->select($this->db->quoteName('title'))
						->from($this->db->quoteName('#__content'))
						->where($this->db->quoteName('id') . ' = ' . (int) $article);
					$this->db->setQuery($query);
					$newoptions[$option->virtuemart_custom_id][] = $this->db->loadResult();
				}
			}

			$values = array();

			// Create the CSVI format
			// option1[value1#value2;option2[value1#value2
			foreach ($newoptions as $option)
			{
				$values[] = implode('#', $option);
			}

			return $values;
		}
		else
		{
			return null;
		}
	}

	/**
	 * Method to delete the custom fields for products.
	 *
	 * @param   string         $plugin                 The ID of the plugin.
	 * @param   array          $params                 An array of settings.
	 * @param   int            $virtuemart_product_id  The product ID.
	 * @param   int            $virtuemart_custom_id   The custom ID.
	 * @param   CsviHelperLog  $log                    The CSVI logger.
	 *
	 * @return  void
	 *
	 * @since   6.5.0
	 */
	public function clearCustomValues($plugin, $params, $virtuemart_product_id, $virtuemart_custom_id, $log)
	{
		if ($plugin == $this->id)
		{
			// Remove existing values for this custom plugin
			$query = $this->db->getQuery(true)
				->delete($this->db->quoteName('#__virtuemart_product_custom_plg_articles'))
				->where($this->db->quoteName('virtuemart_product_id') . ' = ' . (int) $virtuemart_product_id)
				->where($this->db->quoteName('virtuemart_custom_id') . ' = ' . (int) $virtuemart_custom_id);
			$this->db->setQuery($query)->execute();

			$log->add('Remove existing values');
		}
	}
}
