<?php
/**
 * @package     CSVI
 * @subpackage  Plugin.Param
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * Handles the Product Filter for VirtueMart import/export.
 *
 * @package     CSVI
 * @subpackage  Plugin.Param
 * @see         https://myext.eu/en/vmcustom-param-vip
 * @since       6.0
 */
class PlgCsviextParam extends RantaiPluginDispatcher
{
	/**
	 * The unique ID of the plugin
	 *
	 * @var    string
	 * @since  6.0
	 */
	private $id = 'param';

	/**
	 * JDatabase handler
	 *
	 * @var    JDatabase
	 * @since  6.0
	 */
	protected $db = null;

	/**
	 * Construct the class.
	 *
	 * @since   6.0
	 */
	public function __construct()
	{
		// Set the dependencies
		$this->db = JFactory::getDbo();
	}

	/**
	 * Method to get the field options.
	 *
	 * @param   string         $plugin                 The ID of the plugin.
	 * @param   array          $params                 An array of settings.
	 * @param   int            $virtuemart_product_id  The product ID.
	 * @param   int            $virtuemart_custom_id   The custom ID.
	 * @param   CsviHelperLog  $log                    The CSVI logger.
	 *
	 * @return  mixed  String if plugin needs to handle request | False if plugin does not handle request.
	 *
	 * @since   6.0
	 *
	 * @throws  RuntimeException
	 */
	public function getCustomParam($plugin, $params, $virtuemart_product_id, $virtuemart_custom_id, $log)
	{
		if ($plugin !== $this->id)
		{
			return false;
		}

		// Data is received in the format:
		// value1#value2;value1#value2

		// Get all the values
		$param_entries = explode('#', $params);

		// Remove existing values for this parameter
		$query = $this->db->getQuery(true)
			->delete($this->db->quoteName('#__virtuemart_product_custom_plg_param_ref'))
			->where($this->db->quoteName('virtuemart_product_id') . ' = ' . (int) $virtuemart_product_id)
			->where($this->db->quoteName('virtuemart_custom_id') . ' = ' . (int) $virtuemart_custom_id);
		$this->db->setQuery($query);

		if ($this->db->execute())
		{
			// Load the custom field parameters
			$query->clear()
				->select($this->db->quoteName('custom_params'))
				->from($this->db->quoteName('#__virtuemart_customs'))
				->where($this->db->quoteName('virtuemart_custom_id') . ' = ' . (int) $virtuemart_custom_id);
			$this->db->setQuery($query);
			$result = $this->db->loadResult();
			$log->add('Load custom field parameters');
			$cparams = explode('|', $result);

			foreach ($cparams as $cparam)
			{
				if (!empty($cparam))
				{
					$statement = preg_match("/^(?!;)(?P<key>[\w+\.\-]+?)\s*=\s*(?P<value>.+?)\s*$/", $cparam, $match);
					$fparams[$match['key']] = str_ireplace('"', '', $match['value']);
				}
			}

			// Load the IDs
			$qentries = array();

			foreach ($param_entries as $entry)
			{
				$qentries[] = $this->db->quote($entry);
			}

			// Check _values and add if !isset
			$query->clear()
				->select($this->db->quoteName('value'))
				->from($this->db->quoteName('#__virtuemart_product_custom_plg_param_values'))
				->where($this->db->quoteName('virtuemart_custom_id') . ' = ' . (int) $virtuemart_custom_id)
				->order($this->db->quoteName('id'));
			$this->db->setQuery($query);
			$values = $this->db->loadColumn();

			// Set if we need to execute the param query
			$execute = false;
			$query = $this->db->getQuery(true)
				->insert($this->db->quoteName('#__virtuemart_product_custom_plg_param_values'))
				->columns(
					$this->db->quoteName(
						array(
							'virtuemart_custom_id',
							'value',
							'status',
							'published',
							'ordering'
						)
					)
				);

			foreach ($param_entries as $entry)
			{
				if (!in_array($entry, $values, true))
				{
					$query->values((int) $virtuemart_custom_id . ',' . $this->db->quote($entry) . ',0,1,0');
					$execute = true;
				}
			}

			if ($execute)
			{
				$this->db->setQuery($query)->execute();
				$log->add('Insert new parameter values');
			}

			$query->clear()
				->select($this->db->quoteName('id'))
				->from($this->db->quoteName('#__virtuemart_product_custom_plg_param_values'))
				->where($this->db->quoteName('value') . ' IN (' . implode(',', $qentries) . ')')
				->order($this->db->quoteName('id'));
			$this->db->setQuery($query);
			$pids = $this->db->loadColumn();
			$log->add('Load the parameter value IDs');

			// Add all the new values to the database
			$query = $this->db->getQuery(true)
				->insert($this->db->quoteName('#__virtuemart_product_custom_plg_param_ref'))
				->columns(
					$this->db->quoteName(
						array(
							'virtuemart_product_id',
							'virtuemart_custom_id',
							'val',
							'intval'
						)
					)
				);

			foreach ($pids as $pid)
			{
				if (isset($fparams) && $fparams['ft'] === 'int')
				{
					$intval = reset($param_entries);
					$intval = (float) $intval;
					$pid = 0;
				}
				else
				{
					$intval = 0;
				}

				$query->values((int) $virtuemart_product_id . ',' . (int) $virtuemart_custom_id . ',' . (int) $pid . ',' . $intval);
			}

			$this->db->setQuery($query)->execute();

			$log->add('Add parameter references');
		}

		// There are no params
		return '';
	}

	/**
	 * Export the values.
	 *
	 * @param   string         $plugin                     The ID of the plugin.
	 * @param   string         $custom_param               The custom parameter values.
	 * @param   int            $virtuemart_product_id      The product ID.
	 * @param   int            $virtuemart_custom_id       The custom ID.
	 * @param   int            $virtuemart_customfield_id  The custom field ID.
	 * @param   CsviHelperLog  $log                        The CSVI logger.
	 *
	 * @return  array  List of values to export.
	 *
	 * @since   6.0
	 *
	 * @throws  RuntimeException
	 */
	public function exportCustomValues($plugin, $custom_param, $virtuemart_product_id, $virtuemart_custom_id, $virtuemart_customfield_id, $log)
	{
		if ($plugin === $this->id)
		{
			// Get the values for this custom field
			$query = $this->db->getQuery(true)
				->select(
					$this->db->quoteName('v.value') . ',' .
					$this->db->quoteName('r.val', 'val_id') . ',' .
					$this->db->quoteName('r.intval') . ',' .
					$this->db->quoteName('c.custom_title')
				)
				->from($this->db->quoteName('#__virtuemart_product_custom_plg_param_ref', 'r'))
				->leftJoin(
					$this->db->quoteName('#__virtuemart_product_custom_plg_param_values', 'v')
					. ' ON ' . $this->db->quoteName('r.val') . ' = ' . $this->db->quoteName('v.id')
				)
				->leftJoin(
					$this->db->quoteName('#__virtuemart_customs', 'c')
					. ' ON ' . $this->db->quoteName('r.virtuemart_custom_id') . ' = ' . $this->db->quoteName('c.virtuemart_custom_id')
				)
				->leftJoin(
					$this->db->quoteName('#__virtuemart_product_customfields', 'f')
					. ' ON ' . $this->db->quoteName('c.virtuemart_custom_id') . ' = ' . $this->db->quoteName('f.virtuemart_custom_id')
					. ' AND ' . $this->db->quoteName('r.virtuemart_product_id') . ' = ' . $this->db->quoteName('f.virtuemart_product_id')
				)
				->where($this->db->quoteName('r.virtuemart_product_id') . ' = ' . (int) $virtuemart_product_id)
				->where($this->db->quoteName('r.virtuemart_custom_id') . ' = ' . (int) $virtuemart_custom_id)
				->where($this->db->quoteName('f.customfield_value') . ' = ' . $this->db->quote('param'));
			$this->db->setQuery($query);
			$options = $this->db->loadObjectList();

			$log->add('Get the values for this custom field');

			// Group the data correctly
			$newoptions = array();

			foreach ($options as $option)
			{
				$newoptions[$option->custom_title][] = empty($option->val_id) ? $option->intval : $option->value;
			}

			$values = array();

			// Create the CSVI format
			// option1[value1#value2;option2[value1#value2
			foreach ($newoptions as $option)
			{
				$values[] = implode('#', $option);
			}

			return $values;
		}
		else
		{
			return null;
		}
	}

	/**
	 * Method to delete the custom fields for products.
	 *
	 * @param   string         $plugin                 The ID of the plugin.
	 * @param   array          $params                 An array of settings.
	 * @param   int            $virtuemart_product_id  The product ID.
	 * @param   int            $virtuemart_custom_id   The custom ID.
	 * @param   CsviHelperLog  $log                    The CSVI logger.
	 *
	 * @return  void
	 *
	 * @since   6.5.0
	 *
	 * @throws  RuntimeException
	 */
	public function clearCustomValues($plugin, $params, $virtuemart_product_id, $virtuemart_custom_id, $log)
	{
		if ($plugin === $this->id)
		{
			$query = $this->db->getQuery(true)
				->delete($this->db->quoteName('#__virtuemart_product_custom_plg_param_ref'))
				->where($this->db->quoteName('virtuemart_product_id') . ' = ' . (int) $virtuemart_product_id)
				->where($this->db->quoteName('virtuemart_custom_id') . ' = ' . (int) $virtuemart_custom_id);
			$this->db->setQuery($query)->execute();

			$log->add('Remove existing values');
		}
	}
}
